/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

(function ()
{
  Ext.BLANK_IMAGE_URL = 'images/s.gif';

  /*
   * If set to true (default), then Ext.encode uses the browser's native JSON.stringify method if it is available.
   * If set to false, then Ext.encode handles JSON serialization itself.
   */
  //Ext.USE_NATIVE_JSON = false;

  Ext.Loader.setConfig( {
    enabled : true, // enables dynamic (on-demand) class-loading
    disableCaching : true, // false = removes the _dc param from all loader's HTTP requests
    paths : {
      'Ext' : 'js/extjs',
      'QDesk' : 'js/quartzdesk'
    }
  } );

  Ext.require( [
    'QDesk.ApplicationConfig',
    'QDesk.CurrentUser',
    'QDesk.Log',
    'QDesk.Msg',
    'QDesk.Properties',

    /* Start of overrides */
    'QDesk.overrides.JSON',

    'QDesk.overrides.button.Button',

    'QDesk.overrides.chart.legend.SpriteLegend',

    'QDesk.overrides.data.Store',
    'QDesk.overrides.data.proxy.Server',

    /* 'QDesk.overrides.draw.Container', */
    'QDesk.overrides.draw.ContainerBase',

    'QDesk.overrides.form.Field',
    'QDesk.overrides.form.FieldContainer',
    'QDesk.overrides.form.FieldSet',
    'QDesk.overrides.form.Panel',

    'QDesk.overrides.form.field.Display',
    'QDesk.overrides.form.field.Tag',
    'QDesk.overrides.form.field.Text',
    'QDesk.overrides.form.field.Time',
    'QDesk.overrides.form.field.VTypes',

    'QDesk.overrides.grid.Panel',
    'QDesk.overrides.grid.column.Column',

    'QDesk.overrides.menu.Item',

    'QDesk.overrides.ux.IFrame',
    'QDesk.overrides.ux.form.TreeIconCombo',
    'QDesk.overrides.ux.form.field.DateTime',
    /* End of overrides */

    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.DateFormat',
    'QDesk.util.constant.EventName',

    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.Viewport'
  ] );


  /**
   * Used by native JSON.stringify as well as Ext.JSON.encodeDate methods to format Date values.
   * @returns {*|string|*} the formatted date.
   */
  Date.prototype.toJSON = function ()
  {
    return Ext.Date.format( this, QDesk.util.constant.DateFormat.timestampFormatIso8601WithoutTz );
  };


  /*
   * Register a global error handler for errors raised throughout the ExtJS framework and this application.
   * Note: to raise an error the application should use the Ext.Error.raise function.
   */
  Ext.Error.handle = function ( err )
  {
    Ext.log( { level : 'error', stack : true }, err );
    return QDesk.util.UxUtils.handleError( err );
  };

  Ext.application( {
    name : 'QDesk',
    appFolder : 'js/quartzdesk',

    controllers : [
      'HeartbeatController',

      'navigation.ConnectionsController',

      'navigation.NavigationController',

      'navigation.SettingsController',

      'scheduler.SchedulerViewController',

      'security.SecurityController',

      //'top.SwitchLocaleController',
      'top.LicenseInfoController'
    ],

    models : [
      'common.Locale',
      'common.NameValuePair',
      'common.TimeZone',

      'gui.AboutData',
      'gui.ConnectionsTreeNode',
      'gui.JSScriptingTreeNode',
      'gui.SettingsTreeNode',
      'gui.MessageMacroTreeNode',

      'customization.gui.navigation.Link',
      'customization.gui.navigation.Node',
      'customization.gui.navigation.Tab',

      'jmx.RemoteJmxService',

      'message.Message',
      'message.MessageAttachment',
      'message.MessageChannelProfile',
      'message.MessageChannelProfilesTreeNode',

      'scheduler.Scheduler',
      'scheduler.ExecHealthIndicator',
      'scheduler.ExecHealthIndicatorEntry',

      'scheduler.quartz.QuartzCalendar',
      'scheduler.quartz.QuartzExecHistory',
      'scheduler.quartz.QuartzExecNotificationRule',
      'scheduler.quartz.QuartzExecutingJob',
      'scheduler.quartz.QuartzJobChain',
      'scheduler.quartz.QuartzJobDataMapEntry',
      'scheduler.quartz.QuartzJobDetail',
      'scheduler.quartz.QuartzJobGroup',
      'scheduler.quartz.QuartzPlannedJobExecution',
      'scheduler.quartz.QuartzTrigger',
      'scheduler.quartz.QuartzTriggerGroup',
      'scheduler.quartz.QuartzTriggerWithJobDetail',
      'scheduler.quartz.stats.QuartzExecStatData'
    ],

    /* Singleton stores that are automatically instantiated and registered with the Ext.StoreManager */
    stores : [
      'EmptyStore',

      'DateFormatStore',
      'LocaleStore',
      'TimestampFormatStore',

      'enums.ExecStatusStore',
      'enums.ExpressionLanguageStore',
      'enums.HttpProxyTypeStore',
      'enums.JsLoggingLevelStore',
      'enums.MessageChannelStore',
      'enums.MessageReceiverWSVersionStore',
      'enums.NotificationSeverityStore',
      'enums.ProxyTypeStore',
      'enums.SnmpV3AuthenticationProtocolStore',
      'enums.SnmpV3PrivacyProtocolStore',
      'enums.SnmpV3SecurityLevelStore',
      'enums.WSAuthenticationTypeStore',

      'enums.JmxProtocolStore',

      'enums.scheduler.ExecNotificationRuleConditionTypeStore',
      'enums.scheduler.JobChainConditionTypeStore',
      'enums.scheduler.JobChainTargetExecTypeStore',

      'enums.scheduler.quartz.QuartzExecHistoryExportColumnStore',

      'enums.security.PermissionStore',

      'customization.gui.navigation.TabStore',

      'scheduler.quartz.QuartzJSScriptingDataTypesTreeStore',
      'scheduler.quartz.QuartzJSScriptingObjectsTreeStore',
      'scheduler.quartz.QuartzExecNotificationRuleMessageMacrosTreeStore',
      'scheduler.quartz.stats.QuartzStatisticsTimePeriodLengthStore',
      'scheduler.quartz.stats.QuartzJobStatisticsTypeStore',
      'scheduler.quartz.stats.QuartzSchedulerStatisticsTypeStore',
      'scheduler.quartz.stats.QuartzTriggerStatisticsTypeStore'
    ],

    /*
     * Executed by ExtJS from Ext.onReady
     */
    launch : function ()
    {
      /*
       * Disable all WAI-ARIA compatibility warnings printed in the browser console.
       *
       * This code is applicable to ExtJS 6.0.1 only.
       */
      Ext.enableAria = false;
      Ext.enableAriaButtons = false;
      Ext.enableAriaPanels = false;

      /* Stop propagating the contextmenu event from the application. This prevents the browser context menu from showing up. */
      Ext.getDoc().on( 'contextmenu', function ( event, targetHtmlEl )
      {
        // we need to enable context menus in "Chart preview" windows so that users can save images
        var targetEl = Ext.get( targetHtmlEl.id );
        if ( targetEl && targetEl.hasCls( 'x-chart-image' ) )
        {
          //
          return false;
        }
        event.stopEvent();
      } );

      /* Start of tooltip manager initialization. */
      // Init the singleton.  Any tag-based quick tips will start working.;
      Ext.tip.QuickTipManager.init();

      // Apply a set of config properties to the singleton
      Ext.apply( Ext.tip.QuickTipManager.getQuickTip(), {
        dismissDelay : 0, /* do not dismiss tooltips after 5000 ms period */
        maxWidth : 350,
        showDelay : 50 /* default is 500 and that is too long */
      } );
      /* End of tooltip manager initialization. */


      /* Timeout for all Ext.Ajax.request calls. */
      Ext.Ajax.setTimeout( 1000 * QDesk.ApplicationConfig.get( 'ajaxTimeout' ) );


      /**
       * Create the viewport.
       */
      this._createViewport();
    },


    /**
     * Invokes the heartbeat action to obtain user and license details, creates a new viewport
     * and fires the viewportCreated event.
     */
    _createViewport : function ()
    {
      var me = this;

      Ext.Ajax.request( {
        url : QDesk.util.constant.ActionUrl.heartbeatAction,
        callback : function ( opts, success, resp )
        {
          // Remove the "Loading..." message box before the viewport gets shown.
          var loadingMsgBox = Ext.get( 'loadingMsgBox' );
          if ( loadingMsgBox )
          {
            loadingMsgBox.destroy();
          }

          if ( success )
          {
            var json = Ext.decode( resp.responseText );
            if ( json.success )
            {
              /*
               * Update the system data.
               */
              QDesk.System.update( json.system );

              /*
               * Update the current user's data.
               */
              QDesk.CurrentUser.update( json.sessionToken, json.user, true );

              var viewport = Ext.create( 'QDesk.view.Viewport', {
                qdesk : {
                  product : json.product,
                  license : json.license,
                  user : json.user
                }
              } );

              me.fireEvent( QDesk.util.constant.EventName.VIEWPORT_CREATED, {
                viewport : viewport
              } );

              return;
            }
          }

          // there is some error invoking the heartbeat action, show a warning dialog with a Retry button
          Ext.MessageBox.show( {
            title : QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleWarning' ),
            msg : QDesk.Msg.get( 'QDesk.errorStartingGuiMsg' ),
            buttons : Ext.MessageBox.OK,
            buttonText : {
              ok : QDesk.Msg.get( 'QDesk.buttonLabelRetry' )
            },
            icon : Ext.MessageBox.WARNING,
            closable : false,
            fn : function ()
            {
              window.location.reload();
            }
          } );
        }
      } );
    }
  } );
})();

