/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Scheduler info accessor.
 */
Ext.define( 'QDesk.SchedulerInfoAccessor', {
  requires : [
    'QDesk.Log'
  ],

  /**
   * Required scheduler info config parameter with the following structure:
   * {
   *   type : QUARTZ
   *   version : {
   *     major : 1,
   *     minor : 2,
   *     maintenance : 3
   *   },
   *   timeZone : {  / Configured scheduler connection time zone! Not the actual system time zone! /
   *     id : "Europe/Prague",
   *     isoAbbr : "CEST",
   *     displayName : "(UTC+01:00) Europe/Prague (CEST)",
   *     offset : 3600
   *   },
   *
   *   publicApiInfo : {
   *     detected : true/false,
   *     loadedCorrectly : true/false,
   *     compatibleVersion : true/false,
   *     ready : true / false,
   *     version : {
   *       major : 1,
   *       minor : 2,
   *       maintenance : 3,
   *       qualifier : foobar
   *     },
   *   },
   * }
   */
  data : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.data )
      Ext.Error.raise( 'Required config parameter missing: data' );

    me.data = config.data;

    me.callParent( [ config ] );
  },


  /**
   * Returns the type of the scheduler.
   * @return the scheduler type.
   */
  getSchedulerType : function ()
  {
    var me = this;
    return me.data.type;
  },


  /**
   * Returns the object describing the version of the scheduler.
   * @return the version object.
   */
  getSchedulerVersion : function ()
  {
    var me = this;
    return me.data.version;
  },


  /**
   * Returns the object describing the configured time zone of the scheduler.
   * @return the time zone object.
   */
  getSchedulerTimeZone : function ()
  {
    var me = this;
    return me.data.timeZone;
  },


  /**
   * Returns true, if the QuartzDesk Public API Library is ready (i.e. detected, loaded correctly and compatible), false
   * otherwise.
   * @return true if ready, false otherwise.
   */
  isPublicApiReady : function ()
  {
    var me = this;
    return me.data.publicApiInfo.ready;
  },


  /**
   * Returns true, if the QuartzDesk Public API Library is detected, false otherwise.
   * @return true if detected, false otherwise.
   */
  isPublicApiDetected : function ()
  {
    var me = this;
    return me.data.publicApiInfo.detected;
  },


  /**
   * Returns true, if the QuartzDesk Public API Library is loaded correctly, false otherwise.
   * @return true if loaded correctly, false otherwise.
   */
  isPublicApiLoadedCorrectly : function ()
  {
    var me = this;
    return me.data.publicApiInfo.loadedCorrectly;
  },


  /**
   * Returns true, if the QuartzDesk Public API Library version is compatible, false otherwise.
   * @return true if compatible version, false otherwise.
   */
  isPublicApiCompatibleVersion : function ()
  {
    var me = this;
    return me.data.publicApiInfo.compatibleVersion;
  },


  /**
   * Returns the object describing the version of the QuartzDesk Public API.
   * @return the version object.
   */
  getPublicApiVersion : function ()
  {
    var me = this;
    return me.data.publicApiInfo.version;
  }
} );
