/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Audit Trail tab.
 */
Ext.define( 'QDesk.controller.command.audit.InitializeAuditTrailTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.audit.RefreshAuditTrailGrid',

    'QDesk.util.license.LicensedProductFeatures'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var mainTabPanel = Ext.ComponentQuery.query( '#mainTabPanel' )[ 0 ];
    var auditTrailTab = mainTabPanel.down( 'AuditTrailTab' );

    me._bindEventHandlers( auditTrailTab );

    auditTrailTab.setInitialized( true );
  },


  /**
   * Binds events of the specified Audit Trail tab to their respective handlers.
   */
  _bindEventHandlers : function ( auditTrailTab )
  {
    var me = this;

    var auditTrailGrid = auditTrailTab.down( '#auditTrailGrid' );
    me._bindEventHandlersAuditTrailGrid( auditTrailGrid );
  },


  /**
   * Binds the Audit Trail grid events to their handlers.
   */
  _bindEventHandlersAuditTrailGrid : function ( auditTrailGrid )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.General;
    if ( licensedFeatures.AuditTrail )
    {
      // refresh button
      auditTrailGrid.on( 'refresh',
          function ()
          {
            me._refreshAuditTrailGrid();
          } );

      // filters changed
      auditTrailGrid.on( 'changeFilters',
          function ()
          {
            me._refreshAuditTrailGrid();
          } );

      // TODO: check audit trail index is enabled
      var auditTrailIndexEnabled = false;

      auditTrailGrid.on( 'loadFailed',
          function ( operation )
          {
            auditTrailGrid.down( '#refreshButton' ).enable();
          } );

      auditTrailGrid.on( 'loadSuccess',
          function ( store, records )
          {
            auditTrailGrid.down( '#refreshButton' ).enable();
            auditTrailGrid.down( '#fetchPeriod' ).enable();
            auditTrailGrid.down( '#maxFetchSizeSelect' ).enable();
            // auditTrailGrid.down( '#fetchRelatedCheckbox' ).enable();

            // enable only if the index is enabled
            if ( auditTrailIndexEnabled )
            {
              auditTrailGrid.down( '#filterQueryText' ).enable();
            }

            // update empty text if there are no records
            var view = auditTrailGrid.getView();
            if ( Ext.isEmpty( records ) )
            {
              view.emptyText =
                  QDesk.Msg.get( 'QDesk.controller.command.audit.InitializeAuditTrailTab.emptyTextNoMatchingData' );

              view.refresh();
            }
          } );
    }
  },


  /**
   * Invoked to refresh the data in the audit trail grid.
   */
  _refreshAuditTrailGrid : function ()
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.audit.RefreshAuditTrailGrid', {} ).execute();
  }
} );
