/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified folder and refreshes (reloads) its parent folder in the
 * connections tree panel.
 */
Ext.define( 'QDesk.controller.command.connection.DeleteConnectionFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /*
   * Required ID of the folder to delete.
   */
  folderId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.folderId )
      Ext.Error.raise( 'Required config parameter missing: folderId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'confirm.folderDelete' ) )
    {
      var folderName = QDesk.controller.command.connection.ConnectionUtils.getFolderName( me.folderId );

      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteConnectionFolder.confirmMsg',
              Ext.String.htmlEncode( folderName ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._deleteFolder();
            }
          }
      );
    }
    else
    {
      me._deleteFolder();
    }
  },


  _deleteFolder : function ()
  {
    var me = this;

    var folderName = QDesk.controller.command.connection.ConnectionUtils.getFolderName( me.folderId );
    var folderPath = QDesk.controller.command.connection.ConnectionUtils.getFolderPath( me.folderId );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteConnectionFolder.maskMsg',
        Ext.String.htmlEncode( folderName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.connectionManagementActionDeleteFolder,
      params : {
        folderId : me.folderId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me.application.fireEvent( QDesk.util.constant.EventName.CONNECTION_FLD_DELETED, {
              folderId : me.folderId,
              folderPath : folderPath
            } );

            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteConnectionFolder.successMsg',
                    Ext.String.htmlEncode( folderName ) ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }

        // reload the folder's parent folder in the connections tree view
        var parentFolderId = QDesk.controller.command.connection.ConnectionUtils.getFolderParentFolderId( me.folderId );
        if ( parentFolderId )
        {
          Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
            folderId : parentFolderId
          } ).execute();
        }
        else
        {
          Ext.Error.raise( 'Parent folder ID for folder ID: ' + me.folderId + ' not found.' );
        }
      }
    } );
  }
} );
