/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits an existing connection with the specified ID in the
 * connections tree panel.
 */
Ext.define( 'QDesk.controller.command.connection.EditSchedulerConnection', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.connection.SaveSchedulerConnection',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.navigation.connections.EditSchedulerConnectionWindow'
  ],

  /*
   * Required ID of the connection to edit.
   */
  connectionId : undefined,

  /**
   * Required config parameter with the scheduler type.
   * See QDesk.model.scheduler.SchedulerType.
   */
  schedulerType : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: schedulerType' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'EditSchedulerConnectionWindow', {
      qdesk : {
        mode : QDesk.view.work.navigation.connections.EditSchedulerConnectionWindow.MODE_EDIT,
        schedulerType : me.schedulerType
      }
    } );

    // Save/Update button
    window.down( '#actionButton' ).on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.connection.SaveSchedulerConnection', {
        connection : QDesk.controller.command.connection.ConnectionUtils.createSchedulerConnection( me.schedulerType,
            window ),
        window : window
      } ).execute();
    } );

    /* Load the form data */
    me.mask( QDesk.Msg.get( 'QDesk.controller.command.connection.EditSchedulerConnection.maskLoadMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.connectionManagementActionGetConnection,
      params : {
        connectionId : me.connectionId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            // combine connection data and connection configuration data to simplify form population
            var data = {};
            Ext.apply( data, json.data );
            Ext.apply( data, json.data.configEntry );

            // set data in all forms
            Ext.each( Ext.ComponentQuery.query( 'form', window ),
                function ( formPanel )
                {
                  formPanel.setValues( data );
                } );

            window.show();
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp );
        }
      }
    } );
  }
} );
