/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Moves the folder to a new target folder.
 */
Ext.define( 'QDesk.controller.command.connection.MoveConnectionFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.connection.RefreshConnectionFolder',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /*
   * Required ID of the folder the folder should be moved to.
   */
  targetFolderId : undefined,

  /*
   * Required ID of the folder to move.
   */
  folderId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.targetFolderId )
      Ext.Error.raise( 'Required config parameter missing: targetFolderId' );

    if ( !config.folderId )
      Ext.Error.raise( 'Required config parameter missing: folderId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var folderName = QDesk.controller.command.connection.ConnectionUtils.getFolderName( me.folderId );
    var folderPath = QDesk.controller.command.connection.ConnectionUtils.getFolderPath( me.folderId );

    var targetFolderPath = QDesk.controller.command.connection.ConnectionUtils.getFolderPath(
        me.targetFolderId );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.connection.MoveConnectionFolder.maskMsg',
        Ext.String.htmlEncode( folderName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.connectionManagementActionMoveFolder,
      params : {
        targetFolderId : me.targetFolderId,
        folderId : me.folderId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me.application.fireEvent( QDesk.util.constant.EventName.CONNECTION_FLD_MOVED, {
              folderId : me.folderId,
              folderPath : folderPath,
              targetFolderId : me.targetFolderId,
              targetFolderPath : targetFolderPath
            } );

            me._refreshFolder( me.targetFolderId );

            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.connection.MoveConnectionFolder.successMsg',
                    Ext.String.htmlEncode( folderName ) ), me.success );
          }
          else
          {
            me._refreshFolder( null );

            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          me._refreshFolder( null );

          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  },


  /**
   * Refreshes the specified folder.
   */
  _refreshFolder : function ( folderId )
  {
    Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
      folderId : folderId
    } ).execute();
  }
} );
