/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Moves the connection to a new target folder.
 */
Ext.define( 'QDesk.controller.command.connection.MoveSchedulerConnection', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.connection.RefreshConnectionFolder',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /*
   * Required ID of the folder the connection should be moved to.
   */
  targetFolderId : undefined,

  /*
   * Required ID of the connection to move.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.targetFolderId )
      Ext.Error.raise( 'Required config parameter missing: targetFolderId' );

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var connectionName = QDesk.controller.command.connection.ConnectionUtils.getConnectionName( me.connectionId );

    var targetFolderPath = QDesk.controller.command.connection.ConnectionUtils.getFolderPath(
        me.targetFolderId );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.connection.MoveSchedulerConnection.maskMsg',
        Ext.String.htmlEncode( connectionName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.connectionManagementActionMoveConnection,
      params : {
        targetFolderId : me.targetFolderId,
        connectionId : me.connectionId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me.application.fireEvent( QDesk.util.constant.EventName.CONNECTION_MOVED, {
              connectionId : me.connectionId,
              targetFolderId : me.targetFolderId,
              targetFolderPath : targetFolderPath
            } );

            me._refreshFolder( me.targetFolderId );

            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.connection.MoveSchedulerConnection.successMsg',
                    Ext.String.htmlEncode( connectionName ) ), me.success );
          }
          else
          {
            me._refreshFolder( null );

            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          me._refreshFolder( null );

          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  },


  /**
   * Refreshes the specified folder.
   */
  _refreshFolder : function ( folderId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
      folderId : folderId
    } ).execute();
  }
} );
