/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens message channel profile editor.
 */
Ext.define( 'QDesk.controller.command.message.channel.AddMessageChannelProfile', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder',
    'QDesk.controller.command.message.channel.SendTestMessage',

    'QDesk.util.CommonUtils',
    'QDesk.util.MessageChannelProfileUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required ID of the folder the profile should be added to.
   */
  parentFolderId : undefined,

  /*
   * Required parameter with the message channel ({@link QDesk.model.message.channel.MessageChannel})
   * to open the editor for.
   */
  messageChannel : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.parentFolderId )
      Ext.Error.raise( 'Required config parameter missing: parentFolderId' );

    if ( !config.messageChannel )
      Ext.Error.raise( 'Required config parameter missing: messageChannel' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    QDesk.util.MessageChannelProfileUtils.cleanupEditPanel();

    var formPanel = QDesk.util.MessageChannelProfileUtils.createEmptyProfilePanel( me.messageChannel );

    formPanel.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      QDesk.util.UxUtils.focus( nameField, true );
    } );

    // Send Test Message button
    formPanel.down( '#sendTestMessageButton' ).on( 'click', function ()
    {
      me._sendTestMessage();
    } );

    // Apply button
    formPanel.down( '#saveButton' ).on( 'click', function ()
    {
      me._submitForm();
    } );

    QDesk.util.MessageChannelProfileUtils.addAndShowPanel( formPanel );
  },


  _sendTestMessage : function ()
  {
    var me = this;

    var editPanel = QDesk.util.MessageChannelProfileUtils.getEditPanel();

    var formPanel = editPanel.down( 'form' );
    var form = formPanel.getForm();

    // model.MessageChannelProfile populated with message channel specific values
    var messageChannelProfile = formPanel.createMessageChannelProfile();

    var window = QDesk.util.MessageChannelProfileUtils.getProfilesWindow();

    Ext.create( 'QDesk.controller.command.message.channel.SendTestMessage', {
      maskEl : window.getEl(),
      profile : messageChannelProfile
    } ).execute();
  },


  _submitForm : function ()
  {
    var me = this;

    var editPanel = QDesk.util.MessageChannelProfileUtils.getEditPanel();
    var formPanel = editPanel.down( 'form' );
    var form = formPanel.getForm();

    // model.MessageChannelProfile populated with message channel specific values
    var messageChannelProfile = formPanel.createMessageChannelProfile();

    var name = messageChannelProfile.get( 'name' );

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.messageChannelProfileActionAddProfile,
      params : {
        parentFolderId : me.parentFolderId,
        profile : QDesk.util.CommonUtils.encode( messageChannelProfile )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.message.channel.AddMessageChannelProfile.waitMsg',
          Ext.String.htmlEncode( name ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          QDesk.util.MessageChannelProfileUtils.cleanupEditPanel();

          me._refreshTargetFolder();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.message.channel.AddMessageChannelProfile.successMsg',
                  Ext.String.htmlEncode( name ) ), me.success );
        }
        else
        {
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  },


  /**
   * Refreshes the parent folder.
   * @private
   */
  _refreshTargetFolder : function ()
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
      folderId : me.parentFolderId
    } ).execute();
  }
} );
