/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified message channel profile and refreshes (reloads)
 * its parent folder in the message channel profiles tree panel.
 */
Ext.define( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfile', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.util.MessageChannelProfileUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],

  /*
   * Required ID of the message channel profile to delete.
   */
  profileId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.profileId )
      Ext.Error.raise( 'Required config parameter missing: profileId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var profileName = QDesk.util.MessageChannelProfileUtils.getProfileName( me.profileId );

    if ( QDesk.ApplicationConfig.get( 'confirm.msgChannelProfileDelete' ) )
    {
      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfile.confirmMsg',
              Ext.String.htmlEncode( profileName ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._deleteProfile( profileName );
            }
          }
      );
    }
    else
    {
      me._deleteProfile( profileName );
    }
  },


  _deleteProfile : function ( profileName )
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfile.maskMsg',
        Ext.String.htmlEncode( profileName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.messageChannelProfileActionDeleteProfile,
      params : {
        profileId : me.profileId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.message.channel.DeleteMessageChannelProfile.successMsg',
                    Ext.String.htmlEncode( profileName ) ), me.success );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }

        // reload the profile's parent folder
        var parentFolderId = QDesk.util.MessageChannelProfileUtils.getProfileParentFolderId( me.profileId );
        if ( parentFolderId )
        {
          Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
            folderId : parentFolderId
          } ).execute();
        }
        else
        {
          Ext.Error.raise( 'Parent folder ID for message channel profile ID: ' + me.profileId + ' not found.' );
        }
      }
    } );
  }
} );
