/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * Various Quartz-specific controller related utility methods.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils', {
  singleton : true,

  requires : [
    'QDesk.controller.command.connection.ConnectionUtils'
  ],


  /**
   * Returns the Jobs tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getJobsTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    var jobsTab = schedulerView.down( '#jobsTab' );
    if ( !jobsTab )
      Ext.Error.raise( 'Jobs tab for connection ID: ' + me.connectionId + ' not found.' );

    return jobsTab;
  },


  /**
   * Returns the Currently Executing Jobs tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getExecutingJobsTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    var executingJobsTab = schedulerView.down( '#executingJobsTab' );
    if ( !executingJobsTab )
      Ext.Error.raise( 'Executing jobs tab for connection ID: ' + me.connectionId + ' not found.' );

    return executingJobsTab;
  },


  /**
   * Returns the Planned Job Executions tab the specified connection ID.
   * @param connectionId a connection ID.
   */
  getPlannedJobExecutionsTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    var plannedJobExecutionsTab = schedulerView.down( '#plannedJobExecutionsTab' );
    if ( !plannedJobExecutionsTab )
      Ext.Error.raise( 'Planned job executions tab for connection ID: ' + me.connectionId + ' not found.' );

    return plannedJobExecutionsTab;
  },


  /**
   * Returns the Triggers tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getTriggersTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#triggersTab' );
  },


  /**
   * Returns the Scheduler Misfired Triggers tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getMisfiredTriggersTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#misfiredTriggersTab' );
  },


  /**
   * Returns the Info tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getInfoTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#infoTab' );
  },


  /**
   * Returns the Execution History tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getExecHistoryTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#execHistoryTab' );
  },


  /**
   * Returns the Statistics tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getStatisticsTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#statisticsTab' );
  },


  /**
   * Returns the Exec Notification Rules tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getExecNotificationRulesTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#execNotificationRulesTab' );
  },


  /**
   * Returns the Job Chains tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getJobChainsTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#jobChainsTab' );
  },


  /**
   * Returns the Scheduler Audit Trail tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getAuditTrailTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#auditTrailTab' );
  }
} );
