/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a modal window with the list of all jobs and prompts the user to select a job.
 * The selected QuartzJobDetail is passed to the success handler if defined.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.SelectJob', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.util.CommonUtils',
    'QDesk.util.StoreUtils',
    'QDesk.util.quartz.QuartzUtils',

    'QDesk.view.work.scheduler.quartz.SelectJobWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required title for the job-selection window.
   */
  title : undefined,

  /**
   * Optional group name of the job to preselect in the job selection window when it opens.
   */
  preselectedJobGroupName : undefined,

  /**
   * Optional name of the job to preselect in the job selection window when it opens.
   */
  preselectedJobName : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.title )
      Ext.Error.raise( 'Required config parameter missing: title' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

//    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.SelectJob.maskLoadMsg' ) );

    var window = Ext.widget( 'QuartzSelectJobWindow', {
      qdesk : {
        connectionCtx : me.connectionCtx,
        title : me.title,
        preselectedJobGroupName : me.preselectedJobGroupName,
        preselectedJobName : me.preselectedJobName
      }
    } );

    me._bindEventHandlers( window );

    window.show();
  },


  _bindEventHandlers : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );
    var selectJobGrid = formPanel.down( '#selectJobGrid' );
    var actionButton = formPanel.down( '#actionButton' );

    // the action button is disabled by default
    actionButton.disable();


    // job selection
    selectJobGrid.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          actionButton.setDisabled( selectedRecords.length == 0 );
        } );


    // Save/Update button
    actionButton.on( 'click',
        function ()
        {
          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            if ( selectJobGrid.getSelectionModel().hasSelection() )
            {
              var jobDetail = selectJobGrid.getSelectionModel().getSelection()[ 0 ];

              window.close();
              me.success.apply( me, [ jobDetail ] );
            }
          }
        } );
  }
} );
