/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the scheduler Audit Trail grid in the scheduler Audit Trail panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.audit.RefreshSchedulerAuditTrailGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.General;

    if ( licensedFeatures.AuditTrail )
    {
      var auditTrailTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getAuditTrailTab(
          me.connectionCtx.getConnectionId() );

      var auditTrailGrid = auditTrailTab.down( '#auditTrailGrid' );
      var auditTrailGridStore = auditTrailGrid.getStore();

      auditTrailGridStore.getProxy().url =
          QDesk.util.constant.ActionUrl.quartz.auditTrailActionListForScheduler;

      var extraStoreParams = {
        connectionId : me.connectionCtx.getConnectionId()
      };

      var fetchPeriod = auditTrailGrid.down( '#fetchPeriod' ).getValue();
      Ext.apply( extraStoreParams, {
        fetchPeriod : Ext.encode( fetchPeriod )
      } );

      var maxFetchSize = auditTrailGrid.down( '#maxFetchSizeSelect' ).getValue();
      if ( maxFetchSize != -1 )  // -1 == unlimited
      {
        Ext.apply( extraStoreParams, {
          maxFetchSize : maxFetchSize
        } );
      }

      var auditTrailIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzAuditTrailIndexEnabled();

      // if audit trail index enabled, we add the filter query text param
      if ( auditTrailIndexEnabled )
      {
        var filterQueryText = auditTrailGrid.down( '#filterQueryText' ).getValue();
        if ( !Ext.isEmpty( filterQueryText ) )
        {
          Ext.apply( extraStoreParams, {
            filterQuery : filterQueryText
          } );
        }
      }

      auditTrailGridStore.getProxy().extraParams = extraStoreParams;

      auditTrailGridStore.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
