/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window to view the configured execution history result.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryResult', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.ViewTextWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required config property containing the execution history record ID.
   */
  execHistoryId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.execHistoryId )
      Ext.Error.raise( 'Required config parameter missing: execHistoryId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryResult.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.execHistoryActionResult,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        execHistoryId : me.execHistoryId
      },
      scope : me,
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var formattedResult = me._formatResult( json.data );

            var window = Ext.widget( 'ViewTextWindow', {
              cls : 'quartzViewResultWindow',
              iconCls : 'windowIcon',
              title : QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryResult.windowTitle' )
            } );

            window.on( 'afterrender', function ()
            {
              window.maximize();
              window.toFront( false );
            } );

            var editor = window.down( '#editor' );
            editor.setValue( formattedResult );

            /*
             * Set focus on the editor so that the user can immediately use CTRL+A to
             * select all text in the editor. The delay is used to defer setting of the
             * focus to avoid a known issue (see http://forums.ext.net/showthread.php?3315-CLOSED-Set-Focus-to-textfield-in-logon-window).
             */
            QDesk.util.UxUtils.focus( editor, false );

            window.show();

            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  },


  /**
   * Formats the specified result.
   * @param result the result.
   * @return the formatted result as an HTML string.
   * @private
   */
  _formatResult : function ( result )
  {
    var textColor = QDesk.ApplicationConfig.get( 'viewResult.textColor' );

    var formattedResult = '<pre style="margin-bottom: 0; margin-top: 0;">';
    formattedResult += '<span style="color: ' + textColor + '">';
    formattedResult += Ext.String.htmlEncode( result );
    formattedResult += '</span>';
    formattedResult += '</pre>';

    return formattedResult;
  }
} );
