/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits the specified job.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.EditJob', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName',
    'QDesk.util.quartz.QuartzUtils',

    'QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode',
    'QDesk.view.work.scheduler.quartz.jobs.JobDetailWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required config parameter with the job detail of the job to edit.
   */
  jobDetail : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.jobDetail )
      Ext.Error.raise( 'Required config parameter missing: jobDetail' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.EditJob.maskLoadMsg' ) );

    var jobDetailClone = QDesk.util.quartz.QuartzJobUtils.cloneJobDetail( me.jobDetail );

    var formWindow = Ext.widget( 'QuartzJobDetailWindow', {
      title : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.EditJob.windowTitle' ),

      qdesk : {
        connectionCtx : me.connectionCtx,
        jobDetailFormMode : QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.EDIT,
        jobDetail : jobDetailClone,
        callback : function ( jobDetail )
        {
          var jobGroupName = jobDetail.get( 'group' ).name;
          var jobName = jobDetail.get( 'name' );

          /*
           * Important to drop id property from the data to avoid JSON deserialization issues on the server-side.
           */
          var jobDetailData = jobDetail.getData();
          delete jobDetailData.id;

          var form = formWindow.down( '#jobDetailForm' ).getForm();
          form.doAction( 'ParamsSubmit', {
            url : QDesk.util.constant.ActionUrl.quartz.jobActionUpdate,
            params : {
              connectionId : me.connectionCtx.getConnectionId(),
              jobDetail : Ext.encode( jobDetailData )
            },
            waitMsg : QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.EditJob.waitMsg',
                Ext.String.htmlEncode( jobGroupName ), Ext.String.htmlEncode( jobName ) ),
            success : function ( form, action )
            {
              var json = Ext.decode( action.response.responseText );
              if ( json.success )
              {
                formWindow.close();

                QDesk.util.UxUtils.showInfo(
                    QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.EditJob.successMsg',
                        Ext.String.htmlEncode( jobGroupName ), Ext.String.htmlEncode( jobName ) ),
                    function ()
                    {
                      me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.JOB_UPDATED, {
                        connectionId : me.connectionCtx.getConnectionId(),
                        jobDetail : jobDetailClone
                      } );

                      // invoke the success callback handler
                      if ( me.success && Ext.isFunction( me.success ) )
                      {
                        me.success.apply( me );
                      }
                    } );
              }
              else
              {
                QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
              }
            },
            failure : function ( form, action )
            {
              QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
            }
          } );
        },
        scope : me
      }
    } );

    var formStores = formWindow.getAllStores();

    /* Load the form data */
    QDesk.util.StoreUtils.onAllStoresLoad( formStores,
        function ( successful, firstFailedOperation )
        {
          me.unmask();

          if ( successful )
          {
            formWindow.show();
          }
          else
          {
            QDesk.util.UxUtils.showExtJsOperationError( firstFailedOperation, me.failure );
          }
        } );

    QDesk.util.StoreUtils.loadStores( formStores );
  }
} );
