/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Jobs tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsGrid',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecHistoryView',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshTriggersView',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshStatisticsView',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecNotificationsView'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    // refreshes the jobs grid
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsGrid', {
      connectionCtx : me.connectionCtx,
      success : function ()
      {
        me._refreshHistoryView( true );
        me._refreshTriggersView( true );
        me._refreshStatisticsView( true );
        me._refreshNotificationsView( true );
      },
      failure : function ()
      {
        me._refreshHistoryView( false );
        me._refreshTriggersView( false );
        me._refreshStatisticsView( false );
        me._refreshNotificationsView( false );
      }
    } ).execute();
  },

  /**
   * Invoked to refresh the data in the job execution history view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshHistoryView : function ( populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecHistoryView', {
      connectionCtx : me.connectionCtx,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job triggers view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshTriggersView : function ( populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshTriggersView', {
      connectionCtx : me.connectionCtx,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job statistics view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshStatisticsView : function ( populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshStatisticsView', {
      connectionCtx : me.connectionCtx,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the job notifications view.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected job.
   */
  _refreshNotificationsView : function ( populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecNotificationsView', {
      connectionCtx : me.connectionCtx,
      populate : populate
    } ).execute();
  }
} );
