/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the grid in the Planned Job Executions tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshPlannedJobExecutionsGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.PlannedJobExecutions &&
        me.connectionCtx.getJvmAgentInfo().isReady() )
    {
      var plannedJobExecutionsTab =
          QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getPlannedJobExecutionsTab(
              me.connectionCtx.getConnectionId() );

      var plannedJobExecutionsGrid = plannedJobExecutionsTab.down( '#plannedJobExecutionsGrid' );

      var store = plannedJobExecutionsGrid.getStore();

      // set defaults for reload
      store.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
      store.getProxy().extraParams = {};

      if ( plannedJobExecutionsGrid.filtersValid() )
      {
        store.getProxy().url = QDesk.util.constant.ActionUrl.quartz.jobActionListPlannedJobExecutions;

        var extraStoreParams = {
          connectionId : me.connectionCtx.getConnectionId(),
          maxResults : 1000  // TODO: make configurable
        };

        var timePeriod = plannedJobExecutionsGrid.down( '#timePeriod' ).getValue();
        Ext.apply( extraStoreParams, {
          timePeriod : Ext.encode( timePeriod )
        } );

        // if job chains index enabled, we add the filter query text param
        if ( me.connectionCtx.getJvmAgentInfo().isQuartzJobsIndexEnabled() )
        {
          var filterQueryText = plannedJobExecutionsGrid.down( '#filterQueryText' ).getValue();
          if ( !Ext.isEmpty( filterQueryText ) )
          {
            Ext.apply( extraStoreParams, {
              filterQuery : filterQueryText
            } );
          }
        }

        store.getProxy().extraParams = extraStoreParams;
      }

      store.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
