/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Triggers view in the scheduler view panel's Jobs tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshTriggersView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.util.constant.ActionUrl'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * A required flag indicating whether the view should be populated with data to reflect the currently selected job.
   */
  populate : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( config.populate == null )
      Ext.Error.raise( 'Required config parameter missing: populate' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var jobsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getJobsTab(
        me.connectionCtx.getConnectionId() );

    var jobsGrid = jobsTab.down( '#jobsGrid' );

    var selectedJobDetails = jobsGrid.getSelectionModel().getSelection();

    var triggersGrid = jobsTab.down( '#jobTriggersGrid' );
    var triggersGridStore = triggersGrid.getStore();

    var filterByTriggerGroupCombo = triggersGrid.down( '#filterByTriggerGroupCombo' );
    var filterByTriggerGroupComboStore = filterByTriggerGroupCombo.getStore();

    var reloadStores = true;

    if ( me.populate )
    {
      if ( Ext.isEmpty( selectedJobDetails ) )
      {
        filterByTriggerGroupComboStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
        filterByTriggerGroupComboStore.getProxy().extraParams = {};
      }
      else
      {
        var jobDetail = selectedJobDetails[ 0 ];

        filterByTriggerGroupComboStore.getProxy().url =
            QDesk.util.constant.ActionUrl.quartz.triggerGroupActionListForJob;

        filterByTriggerGroupComboStore.getProxy().extraParams = {
          connectionId : me.connectionCtx.getConnectionId(),
          jobGroupName : jobDetail.get( 'group' ).name,
          jobName : jobDetail.get( 'name' )
        };
      }
    }
    else
    {
      if ( filterByTriggerGroupComboStore.getProxy().url == QDesk.util.constant.ActionUrl.emptyStoreAction )
        reloadStores = false;

      filterByTriggerGroupComboStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
      filterByTriggerGroupComboStore.getProxy().extraParams = {};
    }

    // optimization to avoid repeated reloads of empty stores
    if ( reloadStores )
    {
      // once the combo has been reloaded, reload the JobTriggersGrid combo
      filterByTriggerGroupComboStore.load( {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( Ext.isEmpty( selectedJobDetails ) )
          {
            triggersGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
            triggersGridStore.getProxy().extraParams = {};
          }
          else
          {
            var jobDetail = selectedJobDetails[ 0 ];

            triggersGridStore.getProxy().url =
                QDesk.util.constant.ActionUrl.quartz.triggerActionListForJob;

            var extraStoreParams = {
              connectionId : me.connectionCtx.getConnectionId(),
              jobGroupName : jobDetail.get( 'group' ).name,
              jobName : jobDetail.get( 'name' )
            };

            // if triggers index enabled, we add the filter query text param
            if ( me.connectionCtx.getJvmAgentInfo().isQuartzTriggersIndexEnabled() )
            {
              var filterQueryText = triggersGrid.down( '#filterQueryText' ).getValue();
              if ( !Ext.isEmpty( filterQueryText ) )
              {
                Ext.apply( extraStoreParams, {
                  filterQuery : filterQueryText
                } );
              }
            }

            triggersGridStore.getProxy().extraParams = extraStoreParams;
          }

          // reload the jobs grid
          if ( filterByTriggerGroupCombo.getValue() == null )
          {
            // no need to reset the combo, just reload the grid
            triggersGridStore.loadPage( 1 );
          }
          else
          {
            // triggers the change event which reloads the job grid
            filterByTriggerGroupCombo.reset( null );
          }
        }
      } );
    }
  }
} );
