/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the misfired triggers grid in the top-level Scheduler Misfired Triggers panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.misfire.RefreshSchedulerMisfiredTriggersGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.MisfiredTriggers &&
        me.connectionCtx.getJvmAgentInfo().isReady() )
    {
      var misfiredTriggersTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getMisfiredTriggersTab(
          me.connectionCtx.getConnectionId() );

      var misfiredTriggersGrid = misfiredTriggersTab.down( '#misfiredTriggersGrid' );

      var misfiredTriggersGridStore = misfiredTriggersGrid.getStore();

      // set defaults for reload
      misfiredTriggersGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
      misfiredTriggersGridStore.getProxy().extraParams = {};

      if ( misfiredTriggersGrid.filtersValid() )
      {
        misfiredTriggersGridStore.getProxy().url =
            QDesk.util.constant.ActionUrl.quartz.misfiredTriggerActionListForScheduler;

        var extraStoreParams = {
          connectionId : me.connectionCtx.getConnectionId()
        };

        var fetchPeriod = misfiredTriggersGrid.down( '#fetchPeriod' ).getValue();
        Ext.apply( extraStoreParams, {
          fetchPeriod : Ext.encode( fetchPeriod )
        } );

        var maxFetchSize = misfiredTriggersGrid.down( '#maxFetchSizeSelect' ).getValue();
        if ( maxFetchSize != -1 )  // -1 == unlimited
        {
          Ext.apply( extraStoreParams, {
            maxFetchSize : maxFetchSize
          } );
        }

        // if misfired triggers index enabled, we add the filter query text param
        if ( me.connectionCtx.getJvmAgentInfo().isQuartzMisfiredTriggersIndexEnabled() )
        {
          var filterQueryText = misfiredTriggersGrid.down( '#filterQueryText' ).getValue();
          if ( !Ext.isEmpty( filterQueryText ) )
          {
            Ext.apply( extraStoreParams, {
              filterQuery : filterQueryText
            } );
          }
        }

        misfiredTriggersGridStore.getProxy().extraParams = extraStoreParams;
      }

      misfiredTriggersGridStore.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
