/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits an existing notification rule with the specified ID.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.scheduler.SchedulerType',
    'QDesk.model.scheduler.ExecNotificationRuleConditionType',

    'QDesk.util.CommonUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName',
    'QDesk.util.quartz.QuartzExecNotificationRuleUtils',

    'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode',
    'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required ID of the notification rule to edit.
   */
  notificationRuleId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.notificationRuleId )
      Ext.Error.raise( 'Required config parameter missing: notificationRuleId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'EditExecNotificationRuleWindow', {
      qdesk : {
        connectionCtx : me.connectionCtx,
        mode : QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.EDIT
      }
    } );

    var formPanel = window.down( '#form' );

    window.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      // QDesk.util.UxUtils.focus( nameField, true );
    } );

    // Save/Update button
    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      me._submitForm( window );
    } );


    /* Load the form data */
    me.mask(
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule.maskLoadMsg' ) );

    formPanel.load( {
      url : QDesk.util.constant.ActionUrl.quartz.execNotificationRuleActionGet,
      scope : me,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        notificationRuleId : me.notificationRuleId
      },
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          formPanel.loadSpecialFields( json.data );

          me.unmask();
          window.show();
        }
        else
        {
          me.unmask();
          window.close();
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        me.unmask();
        window.close();
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  },


  /**
   * Submits the form.
   * @param window the edit connection window.
   */
  _submitForm : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );

    var form = formPanel.getForm();
    var fieldValues = form.getFieldValues();

    var notificationRule = QDesk.util.quartz.QuartzExecNotificationRuleUtils.createNotificationRule( fieldValues );

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.quartz.execNotificationRuleActionUpdate,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        notificationRule : QDesk.util.CommonUtils.encode( notificationRule )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule.waitMsg',
          Ext.String.htmlEncode( fieldValues.name ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          window.close();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule.successMsg',
                  Ext.String.htmlEncode( fieldValues.name ) ),
              function ()
              {
                me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.EXEC_NOTIF_RULE_UPDATED, {
                  connectionId : me.connectionCtx.getConnectionId(),
                  notificationRule : notificationRule
                } );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  }
} );
