/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Scheduler Notifications tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshExecNotificationsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshJobExecNotificationRulesGrid',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationRulesGrid',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshTriggerExecNotificationRulesGrid'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var execNotificationRulesTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getExecNotificationRulesTab(
        me.connectionCtx.getConnectionId() );

    var activeView = execNotificationRulesTab.getActiveView();

    if ( activeView.getItemId() == 'schedulerExecNotificationRulesView' )
    {
      // refreshes the Scheduler-Wide job chains grid
      Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationRulesGrid', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeView.getItemId() == 'jobExecNotificationRulesView' )
    {
      // refreshes the Job-Specific job chains grid
      Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshJobExecNotificationRulesGrid', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else if ( activeView.getItemId() == 'triggerExecNotificationRulesView' )
    {
      // refreshes the Trigger-Specific job chains grid
      Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshTriggerExecNotificationRulesGrid', {
        connectionCtx : me.connectionCtx
      } ).execute();
    }
    else
    {
      Ext.Error.raise( 'Unrecognized view item ID: ' + activeView.getItemId() );
    }
  }
} );
