/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Scheduler Statistics tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.stats.InitializeSchedulerStatisticsTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.PreviewChart',
    'QDesk.controller.command.SaveChart',
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var statisticsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getStatisticsTab(
        me.connectionCtx.getConnectionId() );

    me._bindEventHandlers( statisticsTab );

    statisticsTab.setInitialized( true );
  },


  /**
   * Binds events of the specified scheduler Statistics tab to their respective handlers.
   */
  _bindEventHandlers : function ( statisticsTab )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionStatistics &&
        me.connectionCtx.getJvmAgentInfo().isReady() )
    {
      var statisticsPanel = statisticsTab.down( '#statisticsPanel' );

      // refresh button
      statisticsPanel.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshStatisticsView();
          } );

      // preview chart button
      statisticsPanel.down( '#previewChartButton' ).on( 'click',
          function ()
          {
            me._previewStatisticsChart();
          } );

      // save chart button
      //statisticsPanel.down( '#saveChartButton #saveChartAsJpeg' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( 'image/jpeg' );
      //    } );
      //
      //statisticsPanel.down( '#saveChartButton #saveChartAsPng' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( 'image/png' );
      //    } );
      //
      //statisticsPanel.down( '#saveChartButton #saveChartAsSvg' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( 'image/svg+xml' );
      //    } );


      statisticsPanel.on( 'statisticsCriteriaChange',
          function ( statisticsType, periodLength, periodStart )
          {
            me._refreshStatisticsView();
          } );
    }
  },


  _refreshStatisticsView : function ()
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab', {
      connectionCtx : me.connectionCtx
    } ).execute();
  },


  /**
   * Invoked to view the chart in a new modal window which allows the user to save the chart
   * through the browser context menu.
   */
  _previewStatisticsChart : function ()
  {
    var me = this;

    var statisticsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getStatisticsTab(
        me.connectionCtx.getConnectionId() );

    /* charts are either cartesian or polar so we use a common superclass - draw */
    var chart = statisticsTab.down( '#statisticsPanel #chartContainer draw' );

    Ext.create( 'QDesk.controller.command.PreviewChart', {
      chart : chart
    } ).execute();
  },


  /**
   * Invoked to save the chart in the scheduler statistics tab.
   * @param mimeType the chart image mime type.
   */
  _saveStatisticsChart : function ( mimeType )
  {
    var me = this;

    var statisticsTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getStatisticsTab(
        me.connectionCtx.getConnectionId() );

    /* charts are either cartesian or polar so we use a common superclass - draw */
    var chart = statisticsTab.down( '#statisticsPanel #chartContainer draw' );

    Ext.create( 'QDesk.controller.command.SaveChart', {
      chart : chart,
      mimeType : mimeType
    } ).execute();
  }
} );
