/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Audit Trail view in the scheduler view panel's Triggers tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshAuditTrailView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * A required flag indicating whether the view should be populated with data to reflect the currently selected trigger.
   */
  populate : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( config.populate == null )
      Ext.Error.raise( 'Required config parameter missing: populate' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.General;

    if ( licensedFeatures.AuditTrail )
    {
      var triggersTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getTriggersTab(
          me.connectionCtx.getConnectionId() );

      var triggersGrid = triggersTab.down( '#triggersGrid' );

      var selectedTriggers = triggersGrid.getSelectionModel().getSelection();

      var auditTrailGrid = triggersTab.down( '#auditTrailGrid' );
      var auditTrailGridStore = auditTrailGrid.getStore();

      var reloadStores = true;

      if ( me.populate )
      {
        if ( Ext.isEmpty( selectedTriggers ) )
        {
          auditTrailGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
          auditTrailGridStore.getProxy().extraParams = {};
        }
        else
        {
          var triggerWithJobDetail = selectedTriggers[ 0 ];
          var trigger = triggerWithJobDetail.get( 'trigger' );

          auditTrailGridStore.getProxy().url = QDesk.util.constant.ActionUrl.quartz.auditTrailActionListForTrigger;

          var extraStoreParams = {
            connectionId : me.connectionCtx.getConnectionId(),
            triggerGroupName : trigger.group.name,
            triggerName : trigger.name
          };

          var fetchPeriod = auditTrailGrid.down( '#fetchPeriod' ).getValue();
          Ext.apply( extraStoreParams, {
            fetchPeriod : Ext.encode( fetchPeriod )
          } );

          var maxFetchSize = auditTrailGrid.down( '#maxFetchSizeSelect' ).getValue();
          if ( maxFetchSize != -1 )  // -1 == unlimited
          {
            Ext.apply( extraStoreParams, {
              maxFetchSize : maxFetchSize
            } );
          }

          var fetchRelated = auditTrailGrid.down( '#fetchRelatedCheckbox' ).getValue();
          Ext.apply( extraStoreParams, {
            fetchRelated : fetchRelated
          } );

          var auditTrailIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzAuditTrailIndexEnabled();

          // if audit trail index enabled, we add the filter query text param
          if ( auditTrailIndexEnabled )
          {
            var filterQueryText = auditTrailGrid.down( '#filterQueryText' ).getValue();
            if ( !Ext.isEmpty( filterQueryText ) )
            {
              Ext.apply( extraStoreParams, {
                filterQuery : filterQueryText
              } );
            }
          }

          auditTrailGridStore.getProxy().extraParams = extraStoreParams;
        }
      }
      else
      {
        if ( auditTrailGridStore.getProxy().url == QDesk.util.constant.ActionUrl.emptyStoreAction )
          reloadStores = false;

        auditTrailGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
        auditTrailGridStore.getProxy().extraParams = {};
      }

      // optimization to avoid repeated reloads of empty stores
      if ( reloadStores )
        auditTrailGridStore.loadPage( 1 );
    }
  }
} );
