/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Misfired Triggers view in the scheduler view panel's Triggers tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshMisfiredTriggersView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * A required flag indicating whether the view should be populated with data to reflect the currently selected trigger.
   */
  populate : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( config.populate == null )
      Ext.Error.raise( 'Required config parameter missing: populate' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.MisfiredTriggers &&
        me.connectionCtx.getJvmAgentInfo().isReady() )
    {
      var triggersTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getTriggersTab(
          me.connectionCtx.getConnectionId() );

      var triggersGrid = triggersTab.down( '#triggersGrid' );

      var selectedTriggers = triggersGrid.getSelectionModel().getSelection();

      var misfiredTriggersGrid = triggersTab.down( '#misfiredTriggersGrid' );

      var misfiredTriggersGridStore = misfiredTriggersGrid.getStore();

      // set defaults for reload
      misfiredTriggersGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
      misfiredTriggersGridStore.getProxy().extraParams = {};

      if ( me.populate )
      {
        // if a trigger is selected and filters are valid
        if ( !Ext.isEmpty( selectedTriggers ) && misfiredTriggersGrid.filtersValid() )
        {
          var triggerWithJobDetail = selectedTriggers[ 0 ];

          var trigger = triggerWithJobDetail.get( 'trigger' );

          misfiredTriggersGridStore.getProxy().url =
              QDesk.util.constant.ActionUrl.quartz.misfiredTriggerActionListForTrigger;

          var extraStoreParams = {
            connectionId : me.connectionCtx.getConnectionId(),
            triggerGroupName : trigger.group.name,
            triggerName : trigger.name
          };

          var fetchPeriod = misfiredTriggersGrid.down( '#fetchPeriod' ).getValue();
          Ext.apply( extraStoreParams, {
            fetchPeriod : Ext.encode( fetchPeriod )
          } );

          var maxFetchSize = misfiredTriggersGrid.down( '#maxFetchSizeSelect' ).getValue();
          if ( maxFetchSize != -1 )  // -1 == unlimited
          {
            Ext.apply( extraStoreParams, {
              maxFetchSize : maxFetchSize
            } );
          }

          // if exec history index enabled, we add the filter query text param
          if ( me.connectionCtx.getJvmAgentInfo().isQuartzMisfiredTriggersIndexEnabled() )
          {
            var filterQueryText = misfiredTriggersGrid.down( '#filterQueryText' ).getValue();
            if ( !Ext.isEmpty( filterQueryText ) )
            {
              Ext.apply( extraStoreParams, {
                filterQuery : filterQueryText
              } );
            }
          }

          misfiredTriggersGridStore.getProxy().extraParams = extraStoreParams;
        }
      }

      misfiredTriggersGridStore.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
