/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Resumes the specified trigger.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.ResumeTrigger', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required config property containing the trigger to resume.
   */
  trigger : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.trigger )
      Ext.Error.raise( 'Required config parameter missing: trigger' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'quartz.confirm.triggerResume' ) )
    {
      var triggerGroupName = me.trigger.get( 'group' ).name;
      var triggerName = me.trigger.get( 'name' );

      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.ResumeTrigger.confirmMsg',
              Ext.String.htmlEncode( triggerGroupName ), Ext.String.htmlEncode( triggerName ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._resumeTrigger();
            }
          }
      );
    }
    else
    {
      me._resumeTrigger();
    }
  },


  _resumeTrigger : function ()
  {
    var me = this;

    var triggerGroupName = me.trigger.get( 'group' ).name;
    var triggerName = me.trigger.get( 'name' );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.ResumeTrigger.maskMsg',
        Ext.String.htmlEncode( triggerGroupName ), Ext.String.htmlEncode( triggerName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.triggerActionResume,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        triggerGroupName : triggerGroupName,
        triggerName : triggerName
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.ResumeTrigger.successMsg',
                    Ext.String.htmlEncode( triggerGroupName ), Ext.String.htmlEncode( triggerName ) ),
                function ()
                {
                  me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.TRIGGER_RESUMED, {
                    connectionId : me.connectionCtx.getConnectionId(),
                    triggerGroupName : triggerGroupName,
                    triggerName : triggerName
                  } );

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  }
} );
