/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Changes password of the currently logged in user.
 */
Ext.define( 'QDesk.controller.command.security.ChangePassword', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};


    if ( !config.currentPassword )
      Ext.Error.raise( 'Required config parameter missing: currentPassword' );

    if ( !config.newPassword )
      Ext.Error.raise( 'Required config parameter missing: newPassword' );

    if ( !config.newPassword2 )
      Ext.Error.raise( 'Required config parameter missing: newPassword2' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.security.ChangePassword.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.securityChangePassword,
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  }
} );
