/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a group editor for adding a new group.
 */
Ext.define( 'QDesk.controller.command.security.groups.AddGroup', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Group',

    'QDesk.util.StoreUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.security.GroupUtils',

    'QDesk.view.work.security.groups.EditGroupForm',
    'QDesk.view.work.security.groups.EditGroupFormMode'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.security.groups.AddGroup.maskLoadMsg' ) );

    // load the Group record
    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.groupActionDefaults,
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var groupDefaults = Ext.create( 'QDesk.model.security.Group', json.data );
            me._addGroup( groupDefaults );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  },


  _addGroup : function ( groupDefaults )
  {
    var me = this;

    // 1. create a new EditGroupForm panel
    var formPanel = Ext.widget( 'EditGroupForm', {
      qdesk : {
        mode : QDesk.view.work.security.groups.EditGroupFormMode.ADD
      }
    } );

    var formStores = formPanel.getFormStores();

    // 2. when all of the EditGroupForm panel's stores are loaded, load the Group record data in the form
    QDesk.util.StoreUtils.onAllStoresLoad( formStores,
        function ( successful, firstFailedOperation )
        {
          if ( successful )
          {
            formPanel.loadRecord( groupDefaults );

            var groupsTab = QDesk.util.security.GroupUtils.getGroupsTab();

            // show the edit form panel
            groupsTab.showEditGroupForm( formPanel );

            me.unmask();

            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            me.unmask();

            QDesk.util.UxUtils.showExtJsOperationError( firstFailedOperation, me.failure );
          }
        } );

    // 2. load all form stores
    QDesk.util.StoreUtils.loadStores( formStores );
  }
} );
