/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits the specified group.
 */
Ext.define( 'QDesk.controller.command.security.groups.EditGroup', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Group',

    'QDesk.util.StoreUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.security.GroupUtils',

    'QDesk.view.work.security.groups.EditGroupForm',
    'QDesk.view.work.security.groups.EditGroupFormMode'
  ],

  /**
   * Required config parameter with the GroupStub instance representing the group to edit.
   */
  groupStub : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.groupStub )
      Ext.Error.raise( 'Required config parameter missing: groupStub' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var name = me.groupStub.get( 'name' );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.security.groups.EditGroup.maskLoadMsg',
        Ext.String.htmlEncode( name ) ) );

    // load the Group record
    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.groupActionGet,
      params : {
        groupId : me.groupStub.get( 'id' )
      },
      callback : function ( opts, success, resp )
      {
        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var group = Ext.create( 'QDesk.model.security.Group', json.data );
            me._editGroup( group );
          }
          else
          {
            me.unmask();

            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          me.unmask();

          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  },


  _editGroup : function ( group )
  {
    var me = this;

    var name = me.groupStub.get( 'name' );

    // 1. create a new EditGroupForm panel
    var formPanel = Ext.widget( 'EditGroupForm', {
      qdesk : {
        mode : QDesk.view.work.security.groups.EditGroupFormMode.EDIT
      }
    } );

    var formStores = formPanel.getFormStores();

    // 3. when all of the EditGroupForm panel's stores are loaded, load the Group record data in the form
    QDesk.util.StoreUtils.onAllStoresLoad( formStores,
        function ( successful, firstFailedOperation )
        {
          if ( successful )
          {
            formPanel.loadRecord( group );

            var groupsTab = QDesk.util.security.GroupUtils.getGroupsTab();

            // show the edit form panel
            groupsTab.showEditGroupForm( formPanel );

            me.unmask();

            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            me.unmask();

            QDesk.util.UxUtils.showExtJsOperationError( firstFailedOperation, me.failure );
          }
        } );

    // 2. load all form stores
    QDesk.util.StoreUtils.loadStores( formStores );
  }
} );
