/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Saves the specified group.
 */
Ext.define( 'QDesk.controller.command.security.groups.SaveGroup', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Group',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /**
   * Required config parameter with the EditGroupForm panel.
   */
  formPanel : undefined,

  /**
   * Required config parameter with the Group instance representing the group to insert/update.
   */
  group : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.formPanel )
      Ext.Error.raise( 'Required config parameter missing: formPanel' );

    if ( !config.group )
      Ext.Error.raise( 'Required config parameter missing: group' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var isInsert = me.group.get( 'id' ) == null;

    if ( isInsert )
    {
      me._insertGroup( me.group );
    }
    else
    {
      me._updateGroup( me.group );
    }
  },


  _insertGroup : function ( group )
  {
    var me = this;

    var name = group.get( 'name' );

    var form = me.formPanel.getForm();

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.groupActionAdd,
      params : {
        group : QDesk.util.CommonUtils.encode( group )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.security.groups.SaveGroup.addWaitMsg',
          Ext.String.htmlEncode( name ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.security.groups.SaveGroup.addSuccessMsg',
                  Ext.String.htmlEncode( name ) ),
              function ()
              {
                me.application.fireEvent( QDesk.util.constant.EventName.GROUP_ADDED, {
                  group : me.group
                } );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  },


  _updateGroup : function ( group )
  {
    var me = this;

    var name = group.get( 'name' );

    var form = me.formPanel.getForm();

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.groupActionUpdate,
      params : {
        group : QDesk.util.CommonUtils.encode( group )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.security.groups.SaveGroup.updateWaitMsg',
          Ext.String.htmlEncode( name ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.security.groups.SaveGroup.updateSuccessMsg',
                  Ext.String.htmlEncode( name ) ),
              function ()
              {
                me.application.fireEvent( QDesk.util.constant.EventName.GROUP_UPDATED, {
                  group : me.group
                } );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  }
} );
