/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a user editor for adding a new user.
 */
Ext.define( 'QDesk.controller.command.security.users.AddUser', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.User',

    'QDesk.util.StoreUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.security.UserUtils',

    'QDesk.view.work.security.users.EditUserForm',
    'QDesk.view.work.security.users.EditUserFormMode'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.security.users.AddUser.maskLoadMsg' ) );

    // load the User record
    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.userActionDefaults,
      callback : function ( opts, success, resp )
      {
        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var userDefaults = Ext.create( 'QDesk.model.security.User', json.data );
            me._addUser( userDefaults );
          }
          else
          {
            me.unmask();

            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          me.unmask();

          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  },


  _addUser : function ( userDefaults )
  {
    var me = this;

    // 1. create a new EditUserForm panel
    var formPanel = Ext.widget( 'EditUserForm', {
      qdesk : {
        mode : QDesk.view.work.security.users.EditUserFormMode.ADD
      }
    } );

    var formStores = formPanel.getFormStores();

    // 2. when all of the EditUserForm panel's stores are loaded, load the User record data in the form
    QDesk.util.StoreUtils.onAllStoresLoad( formStores,
        function ( successful, firstFailedOperation )
        {
          if ( successful )
          {
            formPanel.loadRecord( userDefaults );

            var usersTab = QDesk.util.security.UserUtils.getUsersTab();

            // show the edit form panel
            usersTab.showEditUserForm( formPanel );

            me.unmask();

            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            me.unmask();

            QDesk.util.UxUtils.showExtJsOperationError( firstFailedOperation, me.failure );
          }
        } );

    // 2. load all form stores
    QDesk.util.StoreUtils.loadStores( formStores );
  }
} );
