/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Locks the specified user.
 */
Ext.define( 'QDesk.controller.command.security.users.LockUser', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /**
   * Required config parameter with the UserStub instance representing the user whose account should be locked.
   */
  userStub : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.userStub )
      Ext.Error.raise( 'Required config parameter missing: userStub' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'confirm.userLock' ) )
    {
      var username = me.userStub.get( 'username' );

      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.security.users.LockUser.confirmMsg',
              Ext.String.htmlEncode( username ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._lockUser();
            }
          }
      );
    }
    else
    {
      me._lockUser();
    }
  },


  /**
   * Locks the user.
   */
  _lockUser : function ()
  {
    var me = this;

    var userId = me.userStub.get( 'id' );
    var username = me.userStub.get( 'username' );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.security.users.LockUser.maskMsg', username ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.userActionLock,
      params : {
        userId : userId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.security.users.LockUser.successMsg',
                    Ext.String.htmlEncode( username ) ),
                function ()
                {
                  me.application.fireEvent( QDesk.util.constant.EventName.USER_LOCKED, {
                    userId : userId
                  } );

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json, me.failure );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp, me.failure );
        }
      }
    } );
  }
} );
