/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * The data model encapsulating data for the About window.
 */
Ext.define( 'QDesk.model.gui.AboutData', {
  extend : 'Ext.data.Model',

  requires : [
    'QDesk.util.FormatUtils',
    'QDesk.util.constant.DateFormat'
  ],

  fields : [
    { name : 'productName', type : 'string' },
    { name : 'productVersion', type : 'auto' },

    /* computed auxiliary field */
    {
      name : 'productNameAndVersion', type : 'string',
      convert : function ( newValue, model )
      {
        var productName = model.get( 'productName' );
        var productVersion = model.get( 'productVersion' );
        return QDesk.util.FormatUtils.formatProductNameAndVersion( productName, productVersion );
      }
    },

    /* All dates are expected in ISO 8601 format (dateFormat: QDesk.util.constant.DateFormat.timestampFormatIso8601) */
    { name : 'buildDate', type : 'date', dateFormat : QDesk.util.constant.DateFormat.dateFormat },
    { name : 'buildDateFormatted', type : 'string' },

    /* FormattedTimestamp value */
    { name : 'buildTimestamp', type : 'auto' },

    { name : 'licenseeName', type : 'string' },
    { name : 'licenseExpiresInDays', type : 'int', allowNull : true },
    { name : 'licenseExpiresInDaysText', type : 'string' },

    { name : 'javaVersion', type : 'string' },
    { name : 'javaVendor', type : 'string' },

    { name : 'osName', type : 'string' },
    { name : 'osArch', type : 'string' },
    { name : 'osVersion', type : 'string' },

    { name : 'copyrightYear', type : 'string' }
  ]
} );
