/*
 * Tooltip in label support for form fields.
 * Inspired by http://www.sencha.com/forum/showthread.php?19301
 */
Ext.define( 'QDesk.overrides.form.Field', {
  override : 'Ext.form.Field',

  requires : [
    'QDesk.Msg'
  ],

  afterRender : function ()
  {
    var label = null;
    if ( this.fieldTooltip )
    {
      label = this._findTitle( this, 'label.x-form-item-label span.x-form-item-label-inner' );
      this._maybeAddToolTipToTitle( label, this.fieldTooltip );
    }

    if ( this.boxTooltip )
    {
      label = this._findTitle( this, 'label.x-form-cb-label' );
      this._maybeAddToolTipToTitle( label, this.boxTooltip );
    }

    this.callSuper();
  },

  _findTitle : function ( field, selector )
  {
    var matchedEls = field.getEl().select( selector );
    if ( matchedEls )
    {
      var matchedLabelEls = matchedEls.child( 'label' );
      // we get the first matched label (there should not be another labels, but anyway)
      return matchedLabelEls ? matchedLabelEls.first() : null;
    }
    return null;
  },

  _maybeAddToolTipToTitle : function ( label, tooltipText )
  {
    if ( label )
    {
      if ( this.fieldTooltipClosable )
      {
        label.createChild( {
          tag : 'img',
          src : 'images/icon/tooltip-non-auto-hide-10x10.png',
          style : 'margin-bottom: -1px; margin-left: 5px; padding: 0px; width: 10px; height: 10px;',
          'data-qtip' : Ext.htmlEncode( tooltipText ),
          'data-qtitle' : Ext.htmlEncode( QDesk.Msg.get( 'QDesk.tooltipNonAutoHideTitle' ) ),
          //'data-closable' : true,
          'data-hide' : 'user'  // disables automatic tooltip hiding when the user moves the cursor outside the tooltip icon
        } );
      }
      else
      {
        label.createChild( {
          tag : 'img',
          src : 'images/icon/tooltip-10x10.png',
          style : 'margin-bottom: -1px; margin-left: 5px; padding: 0px; width: 10px; height: 10px;',
          'data-qtip' : Ext.htmlEncode( tooltipText )
        } );
      }
    }
  }
} );
