/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.store.TimestampFormatStore', {
  extend : 'Ext.data.Store',

  storeId : 'TimestampFormatStore',

  fields : [ 'valueJs', 'valueJava', 'valueLabel' ],

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      data : [
        {
          valueJs : 'Y-m-d h:i:s.u A T',  // 2015-05-31 04:00:00.123 PM CEDT
          valueJava : 'yyyy-MM-dd hh:mm:ss.SSS a z',
          valueLabel : 'yyyy-MM-dd hh:mm:ss.SSS a z'
        },
        {
          valueJs : 'Y-m-d H:i:s.u T',    // 2015-05-31 16:00:00.123 CEDT
          valueJava : 'yyyy-MM-dd HH:mm:ss.SSS z',
          valueLabel : 'yyyy-MM-dd HH:mm:ss.SSS z'
        },

        {
          valueJs : 'd-m-Y h:i:s.u A T',  // 31-05-2015 04:00:00.123 PM CEDT
          valueJava : 'dd-MM-yyyy hh:mm:ss.SSS a z',
          valueLabel : 'dd-MM-yyyy hh:mm:ss.SSS a z'
        },
        {
          valueJs : 'd-m-Y H:i:s.u T',    // 31-05-2015 16:00:00.123 CEDT
          valueJava : 'dd-MM-yyyy HH:mm:ss.SSS z',
          valueLabel : 'dd-MM-yyyy HH:mm:ss.SSS z'
        },

        {
          valueJs : 'm/d/Y h:i:s.u A T',  // 05/31/2015 04:00:00.123 PM CEDT
          valueJava : 'MM/dd/yyyy hh:mm:ss.SSS a z',
          valueLabel : 'MM/dd/yyyy hh:mm:ss.SSS a z'
        },
        {
          valueJs : 'm/d/Y H:i:s.u T',    // 05/31/2015 16:00:00.123 CEDT
          valueJava : 'MM/dd/yyyy HH:mm:ss.SSS z',
          valueLabel : 'MM/dd/yyyy HH:mm:ss.SSS z'
        },

        {
          valueJs : 'd/m/Y h:i:s.u A T',  // 31/05/2015 04:00:00.123 PM CEDT
          valueJava : 'dd/MM/yyyy hh:mm:ss.SSS a z',
          valueLabel : 'dd/MM/yyyy hh:mm:ss.SSS a z'
        },
        {
          valueJs : 'd/m/Y H:i:s.u T',    // 31/05/2015 16:00:00.123 CEDT
          valueJava : 'dd/MM/yyyy HH:mm:ss.SSS z',
          valueLabel : 'dd/MM/yyyy HH:mm:ss.SSS z'
        },

        {
          valueJs : 'd.m.Y h:i:s.u A T',  // 31.05.2015 04:00:00.123 PM CEDT
          valueJava : 'dd.MM.yyyy hh:mm:ss.SSS a z',
          valueLabel : 'dd.MM.yyyy hh:mm:ss.SSS a z'
        },
        {
          valueJs : 'd.m.Y H:i:s.u T',    // 31.05.2015 16:00:00.123 CEDT
          valueJava : 'dd.MM.yyyy HH:mm:ss.SSS z',
          valueLabel : 'dd.MM.yyyy HH:mm:ss.SSS z'
        },

        {
          valueJs : 'Y.m.d h:i:s.u A T',  // 2015.05.31 04:00:00.123 PM CEDT
          valueJava : 'yyyy.MM.dd hh:mm:ss.SSS a z',
          valueLabel : 'yyyy.MM.dd hh:mm:ss.SSS a z'
        },
        {
          valueJs : 'Y.m.d H:i:s.u T',    // 2015.05.31 16:00:00.123 CEDT
          valueJava : 'yyyy.MM.dd HH:mm:ss.SSS z',
          valueLabel : 'yyyy.MM.dd HH:mm:ss.SSS z'
        },

        {
          valueJs : 'M d, Y h:i:s.u A T',  // May 31, 2015 04:00:00.123 PM CEDT
          valueJava : 'MMM dd, yyyy hh:mm:ss.SSS a z',
          valueLabel : 'MMM dd, yyyy hh:mm:ss.SSS a z'
        },
        {
          valueJs : 'M d, Y H:i:s.u T',    // May 31, 2015 16:00:00.123 CEDT
          valueJava : 'MMM dd, yyyy HH:mm:ss.SSS z',
          valueLabel : 'MMM dd, yyyy HH:mm:ss.SSS z'
        }
      ]
    } );

    me.callParent( [ config ] );
  }
} );
