/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.store.enums.scheduler.quartz.QuartzCronTriggerMisfireInstructionStore', {
  extend : 'Ext.data.Store',

  requires : [
    'QDesk.model.scheduler.quartz.QuartzCronTriggerMisfireInstruction',
    'QDesk.util.quartz.QuartzEnumUtils'
  ],

  fields : [ 'value', 'valueLabel' ],

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.schedulerVersion )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerVersion' );

    var data = [];

    if ( config.qdesk.schedulerVersion.major >= 2 )
    {
      data.push( {
        /* Quartz >= 2.0.0 */
        value : -1,
        valueLabel : QDesk.util.quartz.QuartzEnumUtils.getCronTriggerMisfireInstructionName(
            QDesk.model.scheduler.quartz.QuartzCronTriggerMisfireInstruction.IGNORE_MISFIRE_POLICY )
      } );
    }

    data.push( {
      value : 0,
      valueLabel : QDesk.util.quartz.QuartzEnumUtils.getCronTriggerMisfireInstructionName(
          QDesk.model.scheduler.quartz.QuartzCronTriggerMisfireInstruction.SMART_POLICY )
    } );

    data.push( {
      value : 1,
      valueLabel : QDesk.util.quartz.QuartzEnumUtils.getCronTriggerMisfireInstructionName(
          QDesk.model.scheduler.quartz.QuartzCronTriggerMisfireInstruction.FIRE_ONCE_NOW )
    } );

    data.push( {
      value : 2,
      valueLabel : QDesk.util.quartz.QuartzEnumUtils.getCronTriggerMisfireInstructionName(
          QDesk.model.scheduler.quartz.QuartzCronTriggerMisfireInstruction.DO_NOTHING )
    } );

    Ext.apply( config, {
      data : data
    } );

    me.callParent( [ config ] );
  }
} );