/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.store.enums.scheduler.quartz.QuartzSimpleTriggerMisfireInstructionStore', {
  extend : 'Ext.data.Store',

  requires : [
    'QDesk.model.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction',
    'QDesk.util.quartz.QuartzEnumUtils'
  ],

  fields : [ 'value', 'valueLabel' ],

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.schedulerVersion )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerVersion' );

    var data = [];

    if ( config.qdesk.schedulerVersion.major >= 2 )
    {
      data.push( {
        /* Quartz >= 2.0.0 */
        value : -1,
        valueLabel : QDesk.util.quartz.QuartzEnumUtils.getSimpleTriggerMisfireInstructionName(
            QDesk.model.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.IGNORE_MISFIRE_POLICY )
      } );
    }

    data.push( {
      value : 0,
      valueLabel : QDesk.util.quartz.QuartzEnumUtils.getSimpleTriggerMisfireInstructionName(
          QDesk.model.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.SMART_POLICY )
    } );

    data.push( {
      value : 1,
      valueLabel : QDesk.util.quartz.QuartzEnumUtils.getSimpleTriggerMisfireInstructionName(
          QDesk.model.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.FIRE_NOW )
    } );


    data.push( {
      value : 2,
      valueLabel : QDesk.util.quartz.QuartzEnumUtils.getSimpleTriggerMisfireInstructionName(
          QDesk.model.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.RESCHEDULE_NOW_WITH_EXISTING_REPEAT_COUNT )
    } );

    data.push( {
      value : 3,
      valueLabel : QDesk.util.quartz.QuartzEnumUtils.getSimpleTriggerMisfireInstructionName(
          QDesk.model.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.RESCHEDULE_NOW_WITH_REMAINING_REPEAT_COUNT )
    } );


    data.push( {
      value : 5,
      valueLabel : QDesk.util.quartz.QuartzEnumUtils.getSimpleTriggerMisfireInstructionName(
          QDesk.model.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.RESCHEDULE_NEXT_WITH_EXISTING_COUNT )
    } );

    data.push( {
      value : 4,
      valueLabel : QDesk.util.quartz.QuartzEnumUtils.getSimpleTriggerMisfireInstructionName(
          QDesk.model.scheduler.quartz.QuartzSimpleTriggerMisfireInstruction.RESCHEDULE_NEXT_WITH_REMAINING_COUNT )
    } );


    Ext.apply( config, {
      data : data
    } );

    me.callParent( [ config ] );
  }
} );