/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


Ext.define( 'QDesk.util.FormUtils', {
  requires : [
    'QDesk.Log',

    'QDesk.util.UxUtils'
  ],

  singleton : true,


  /**
   * Places the focus in the first editable form field.
   * @param formPanel a form panel.
   * @param select a boolean flag indicating if the contents of the focused field should be also selected.
   */
  focusFirstEditableField : function ( formPanel, select )
  {
    Ext.each( Ext.ComponentQuery.query( 'component', formPanel ),
        function ( item )
        {
          if ( item.isFormField )
          {
            if ( item.isVisible() && !item.isDisabled() && !item.readOnly )
            {
              //alert( 'Placing focus on field: ' + item + ', itemId: ' + item.getItemId() );
              QDesk.util.UxUtils.focus( item, select );
              return false;
            }
          }
        } );
  },


  /**
   * Places the focus on the specified form's default button if it is specified.
   * @param formPanel a form panel.
   */
  focusDefaultButton : function ( formPanel )
  {
    if ( !Ext.isEmpty( formPanel.defaultButton ) )
    {
      var defaultButton = formPanel.down( '#' + formPanel.defaultButton );
      if ( defaultButton )
        QDesk.util.UxUtils.focus( defaultButton );
    }
  },


  /**
   * Inserts the specified text at the current cursor position in the specified
   * textfield, or textarea.
   * @param textField a text field.
   * @param text a text to insert.
   */
  insertTextAtCursor : function ( textField, text )
  {
    var caretPosition = textField.getCaretPosition();

    // if caret position is not available, text will be inserted at the beginning
    if ( caretPosition == null )
      caretPosition = 0;

    var inputId = textField.getInputId();
    var inputField = Ext.get( inputId ).dom;

    var newTextValue = inputField.value.substring( 0, caretPosition )
        + text
        + inputField.value.substring( caretPosition, inputField.value.length );

    textField.setValue( newTextValue );

    textField.setCaretPosition( caretPosition + text.length );

    textField.validate();  // required to revalidate the text field
  }
} );
