/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * QDesk configuration singleton that encapsulates action URLs.
 */
Ext.define( 'QDesk.util.constant.ActionUrl', {

  singleton : true,

  /*
   * Common actions
   */
  emptyStoreAction : 'common/extjs/EmptyStore.action',

  /*
   * Uncategorized actions
   */
  aboutActionData : 'About.action?data=',

  heartbeatAction : 'Heartbeat.action?heartbeat=',

  imageActionCheck : 'Image.action?check=',

  svgActionPrepare : 'SaveSvg.action?prepare=',
  svgActionDownload : 'SaveSvg.action?download=',


  saveLogActionPrepare : 'SaveLog.action?prepare=',
  saveLogActionDownload : 'SaveLog.action?download=',

  /*
   * Config actions
   */
  applicationConfigActionSaveChanges : 'config/ApplicationConfig.action?saveChanges=',
  applicationConfigActionSaveGridConfig : 'config/ApplicationConfig.action?saveGridConfig=',

  connectionConfigActionSaveChanges : 'config/ConnectionConfig.action?saveChanges=',
  connectionConfigActionSaveGridConfig : 'config/ConnectionConfig.action?saveGridConfig=',

  /*
   * Audit Trail actions
   */
  auditTrailActionList : 'audit/AuditTrail.action?list=',
  auditTrailActionListForUser : 'audit/AuditTrail.action?listForUser=',

  /*
   * User actions
   */
  userActionList : 'security/User.action?list=',
  userActionAdd : 'security/User.action?add=',
  userActionUpdate : 'security/User.action?update=',
  userActionGet : 'security/User.action?get=',
  userActionDefaults : 'security/User.action?defaults=',
  userActionDelete : 'security/User.action?delete=',
  userActionLock : 'security/User.action?lock=',
  userActionUnlock : 'security/User.action?unlock=',
  userActionGetEffectivePermissions : 'security/User.action?getEffectivePermissions=',
  userActionUpdateProfile : 'security/User.action?updateProfile=',

  /*
   * Group actions
   */
  groupActionList : 'security/Group.action?list=',
  groupActionListMembers : 'security/Group.action?listMembers=',
  groupActionAdd : 'security/Group.action?add=',
  groupActionUpdate : 'security/Group.action?update=',
  groupActionGet : 'security/Group.action?get=',
  groupActionDefaults : 'security/Group.action?defaults=',
  groupActionDelete : 'security/Group.action?delete=',


  /*
   * Connection actions
   */
  connectionActionTree : 'connection/ConnectionTree.action?tree=',

  // connection actions
  connectionManagementActionAddConnection : 'connection/ConnectionManagement.action?addConnection=',
  connectionManagementActionGetConnection : 'connection/ConnectionManagement.action?getConnection=',
  connectionManagementActionUpdateConnection : 'connection/ConnectionManagement.action?updateConnection=',
  connectionManagementActionDeleteConnection : 'connection/ConnectionManagement.action?deleteConnection=',
  connectionManagementActionMoveConnection : 'connection/ConnectionManagement.action?moveConnection=',
  connectionManagementActionGetDefaultConnectionConfig : 'connection/ConnectionManagement.action?getDefaultConnectionConfig=',

  // folder actions
  connectionManagementActionAddFolder : 'connection/ConnectionManagement.action?addFolder=',
  connectionManagementActionUpdateFolder : 'connection/ConnectionManagement.action?updateFolder=',
  connectionManagementActionDeleteFolder : 'connection/ConnectionManagement.action?deleteFolder=',
  connectionManagementActionMoveFolder : 'connection/ConnectionManagement.action?moveFolder=',

  connectionManagementActionStartAll : 'connection/ConnectionManagement.action?startAll=',
  connectionManagementActionPauseAll : 'connection/ConnectionManagement.action?pauseAll=',
  connectionManagementActionStopAll : 'connection/ConnectionManagement.action?stopAll=',


  /*
   * Customization actions
   */
  customizationGuiGetNavigationPanelDefinition : 'customization/gui/NavigationPanel.action?getDefinition=',


  /*
   * License actions
   */
  licenseActionV10Info : 'license/v1_0/License.action?info=',
  licenseActionV10GeneralInfoTree : 'license/v1_0/License.action?generalInfoTree=',
  licenseActionV10FeaturesInfoTree : 'license/v1_0/License.action?featuresInfoTree=',


  /*
   * Locale actions
   */
  localeActionLocales : 'locale/Locale.action?locales=',
  localeActionSwitch : 'locale/Locale.action?switch=',


  /*
   * Message actions
   */
  messageActionListForCreator : 'message/Message.action?listForCreator=',
  messageActionDownloadAttachment : 'message/Message.action?downloadAttachment=',

  // message channel actions
  messageChannelActionTest : 'message/MessageChannel.action?test=',

  // message channel profile action
  messageChannelProfileActionTree : 'message/MessageChannelProfile.action?tree=',
  messageChannelProfileActionGetProfile : 'message/MessageChannelProfile.action?getProfile=',
  messageChannelProfileActionAddProfile : 'message/MessageChannelProfile.action?addProfile=',
  messageChannelProfileActionDeleteProfile : 'message/MessageChannelProfile.action?deleteProfile=',
  messageChannelProfileActionMoveProfile : 'message/MessageChannelProfile.action?moveProfile=',
  messageChannelProfileActionUpdateProfile : 'message/MessageChannelProfile.action?updateProfile=',
  messageChannelProfileActionSendTestMessage : 'message/MessageChannelProfile.action?sendTestMessage=',

  messageChannelProfileActionAddFolder : 'message/MessageChannelProfile.action?addFolder=',
  messageChannelProfileActionDeleteFolder : 'message/MessageChannelProfile.action?deleteFolder=',
  messageChannelProfileActionMoveFolder : 'message/MessageChannelProfile.action?moveFolder=',
  messageChannelProfileActionUpdateFolder : 'message/MessageChannelProfile.action?updateFolder=',

  snmpGenerateMib : 'message/Snmp.action?generateMib=',


  /*
   * Scheduler actions
   */
  schedulerActionStart : 'scheduler/Scheduler.action?start=',
  schedulerActionPause : 'scheduler/Scheduler.action?pause=',
  schedulerActionStop : 'scheduler/Scheduler.action?stop=',

  schedulerActionDiscover : 'scheduler/Scheduler.action?discover=',
  schedulerActionTestConnection : 'scheduler/Scheduler.action?testConnection=',
  schedulerActionTestConnectionData : 'scheduler/Scheduler.action?testConnectionData=',

  schedulerActionAgentInfo : 'scheduler/Scheduler.action?agentInfo=',
  schedulerActionInfo : 'scheduler/Scheduler.action?schedulerInfo=',


  /*
   * Security actions
   */
  securityChangePassword : 'security/Security.action?changePassword=',
  securityLoginAction : 'security/Security.action?login=',
  securityLogoutAction : 'security/Security.action?logout=',
  securityRecoverUsername : 'security/Security.action?recoverUsername=',
  securityResetPasswordInit : 'security/Security.action?resetPasswordInit=',
  securityResetPasswordFinish : 'security/Security.action?resetPasswordFinish=',

  /*
   * System actions
   */
  systemServiceEndpointRegistryActionListServices : 'system/ws/EndpointRegistry.action?listServices=',
  systemServiceEndpointRegistryActionListEndpoints : 'system/ws/EndpointRegistry.action?listEndpoints=',
  systemServiceEndpointRegistryActionStartEndpoint : 'system/ws/EndpointRegistry.action?startEndpoint=',
  systemServiceEndpointRegistryActionStopEndpoint : 'system/ws/EndpointRegistry.action?stopEndpoint=',
  systemServiceEndpointRegistryActionResetEndpointStatistics : 'system/ws/EndpointRegistry.action?resetEndpointStatistics=',


  expressionActionValidate : 'expression/Expression.action?validate',


  /*
   * Quartz scheduler actions
   */
  quartz : {
    calendarActionList : 'scheduler/quartz/QuartzCalendar.action?list=',

    infoActionInfo : 'scheduler/quartz/QuartzInfo.action?info=',

    /*
     * Quartz job actions
     */
    jobActionList : 'scheduler/quartz/QuartzJob.action?list=',
    jobActionTrigger : 'scheduler/quartz/QuartzJob.action?trigger=',
    jobActionAdd : 'scheduler/quartz/QuartzJob.action?add=',
    jobActionUpdate : 'scheduler/quartz/QuartzJob.action?update=',
    jobActionDelete : 'scheduler/quartz/QuartzJob.action?delete=',
    jobActionInterrupt : 'scheduler/quartz/QuartzJob.action?interrupt=',
    jobActionListNextFireTimes : 'scheduler/quartz/QuartzJob.action?listNextFireTimes=',

    jobActionListExecuting : 'scheduler/quartz/QuartzJob.action?listExecuting=',
    jobActionGetThreadInfo : 'scheduler/quartz/QuartzJob.action?getThreadInfo=',
    jobActionListAvailableJobClasses : 'scheduler/quartz/QuartzJob.action?listAvailableJobClasses=',
    jobActionListPlannedJobExecutions : 'scheduler/quartz/QuartzJob.action?listPlannedJobExecutions=',

    jobGroupActionList : 'scheduler/quartz/QuartzJobGroup.action?list=',
    jobGroupActionResume : 'scheduler/quartz/QuartzJobGroup.action?resume=',
    jobGroupActionPause : 'scheduler/quartz/QuartzJobGroup.action?pause=',

    /*
   * Quartz trigger actions
     */
    triggerActionListForJob : 'scheduler/quartz/QuartzTrigger.action?listForJob=',
    triggerActionList : 'scheduler/quartz/QuartzTrigger.action?list=',
    triggerActionResume : 'scheduler/quartz/QuartzTrigger.action?resume=',
    triggerActionPause : 'scheduler/quartz/QuartzTrigger.action?pause=',
    triggerActionAdd : 'scheduler/quartz/QuartzTrigger.action?add=',
    triggerActionUpdate : 'scheduler/quartz/QuartzTrigger.action?update=',
    triggerActionDelete : 'scheduler/quartz/QuartzTrigger.action?delete=',
    triggerActionListNextFireTimes : 'scheduler/quartz/QuartzTrigger.action?listNextFireTimes=',
    triggerActionListNextFireTimes2 : 'scheduler/quartz/QuartzTrigger.action?listNextFireTimes2=',

    triggerGroupActionList : 'scheduler/quartz/QuartzTriggerGroup.action?list=',
    triggerGroupActionListForJob : 'scheduler/quartz/QuartzTriggerGroup.action?listForJob=',
    triggerGroupActionResume : 'scheduler/quartz/QuartzTriggerGroup.action?resume=',
    triggerGroupActionPause : 'scheduler/quartz/QuartzTriggerGroup.action?pause=',

    /*
     * Quartz misfired trigger actions
     */
    misfiredTriggerActionListForScheduler : 'scheduler/quartz/QuartzMisfiredTrigger.action?listForScheduler=',
    misfiredTriggerActionListForJob : 'scheduler/quartz/QuartzMisfiredTrigger.action?listForJob=',
    misfiredTriggerActionListForTrigger : 'scheduler/quartz/QuartzMisfiredTrigger.action?listForTrigger=',

    /*
     * Quartz execution history actions
     */
    execHistoryActionListForScheduler : 'scheduler/quartz/QuartzExecHistory.action?listForScheduler=',
    execHistoryActionListForJob : 'scheduler/quartz/QuartzExecHistory.action?listForJob=',
    execHistoryActionListForTrigger : 'scheduler/quartz/QuartzExecHistory.action?listForTrigger=',

    execHistoryActionResult : 'scheduler/quartz/QuartzExecHistory.action?result=',
    execHistoryActionUserData : 'scheduler/quartz/QuartzExecHistory.action?userData=',

    execHistoryActionExecutingJobLog : 'scheduler/quartz/QuartzExecHistory.action?executingJobLog=',
    execHistoryActionExecutingJobLogSave : 'scheduler/quartz/QuartzExecHistory.action?executingJobLogSave=',
    execHistoryActionExecutingJobLogDownload : 'scheduler/quartz/QuartzExecHistory.action?executingJobLogDownload=',

    execHistoryActionExportForScheduler : 'scheduler/quartz/QuartzExecHistory.action?exportForScheduler=',
    execHistoryActionExportForJob : 'scheduler/quartz/QuartzExecHistory.action?exportForJob=',
    execHistoryActionExportForTrigger : 'scheduler/quartz/QuartzExecHistory.action?exportForTrigger=',
    execHistoryActionExportDownload : 'scheduler/quartz/QuartzExecHistory.action?exportDownload=',

    execHistoryActionLog : 'scheduler/quartz/QuartzExecHistory.action?log=',
    execHistoryActionLogSave : 'scheduler/quartz/QuartzExecHistory.action?logSave=',
    execHistoryActionLogDownload : 'scheduler/quartz/QuartzExecHistory.action?logDownload=',

    /*
     * Quartz execution notification rule actions
     */
    execNotificationRuleActionListForScheduler : 'scheduler/quartz/notif/exec/QuartzExecNotificationRule.action?listForScheduler=',
    execNotificationRuleActionListForJob : 'scheduler/quartz/notif/exec/QuartzExecNotificationRule.action?listForJob=',
    execNotificationRuleActionListForJobs : 'scheduler/quartz/notif/exec/QuartzExecNotificationRule.action?listForJobs=',
    execNotificationRuleActionListForTrigger : 'scheduler/quartz/notif/exec/QuartzExecNotificationRule.action?listForTrigger=',
    execNotificationRuleActionListForTriggers : 'scheduler/quartz/notif/exec/QuartzExecNotificationRule.action?listForTriggers=',
    execNotificationRuleActionGet : 'scheduler/quartz/notif/exec/QuartzExecNotificationRule.action?get=',
    execNotificationRuleActionAdd : 'scheduler/quartz/notif/exec/QuartzExecNotificationRule.action?add=',
    execNotificationRuleActionUpdate : 'scheduler/quartz/notif/exec/QuartzExecNotificationRule.action?update=',
    execNotificationRuleActionDelete : 'scheduler/quartz/notif/exec/QuartzExecNotificationRule.action?delete=',

    execNotificationRuleActionEnable : 'scheduler/quartz/notif/exec/QuartzExecNotificationRule.action?enable=',
    execNotificationRuleActionDisable : 'scheduler/quartz/notif/exec/QuartzExecNotificationRule.action?disable=',

    execNotificationRuleSendTestMessageActionTest : 'scheduler/quartz/notif/exec/QuartzSendTestMessage.action?test=',

    /*
     * Quartz statistics actions
     */
    jobStatisticsActionListPeriods : 'scheduler/quartz/stats/JobStatistics.action?listPeriods=',
    jobStatisticsActionChartData : 'scheduler/quartz/stats/JobStatistics.action?chartData=',

    triggerStatisticsActionListPeriods : 'scheduler/quartz/stats/TriggerStatistics.action?listPeriods=',
    triggerStatisticsActionChartData : 'scheduler/quartz/stats/TriggerStatistics.action?chartData=',

    schedulerStatisticsActionListPeriods : 'scheduler/quartz/stats/SchedulerStatistics.action?listPeriods=',
    schedulerStatisticsActionChartData : 'scheduler/quartz/stats/SchedulerStatistics.action?chartData=',

    /*
     * Quartz job chain actions
     */
    jobChainActionListForScheduler : 'scheduler/quartz/chain/QuartzJobChain.action?listForScheduler=',
    jobChainActionListForJob : 'scheduler/quartz/chain/QuartzJobChain.action?listForJob=',
    jobChainActionListForJobs : 'scheduler/quartz/chain/QuartzJobChain.action?listForJobs=',
    jobChainActionListForTrigger : 'scheduler/quartz/chain/QuartzJobChain.action?listForTrigger=',
    jobChainActionListForTriggers : 'scheduler/quartz/chain/QuartzJobChain.action?listForTriggers=',
    jobChainActionGet : 'scheduler/quartz/chain/QuartzJobChain.action?get=',
    jobChainActionAdd : 'scheduler/quartz/chain/QuartzJobChain.action?add=',
    jobChainActionUpdate : 'scheduler/quartz/chain/QuartzJobChain.action?update=',
    jobChainActionDelete : 'scheduler/quartz/chain/QuartzJobChain.action?delete=',

    jobChainActionEnable : 'scheduler/quartz/chain/QuartzJobChain.action?enable=',
    jobChainActionDisable : 'scheduler/quartz/chain/QuartzJobChain.action?disable=',

    /*
     * Quartz audit trail actions
     */
    auditTrailActionListForScheduler : 'scheduler/quartz/audit/QuartzAuditTrail.action?listForScheduler=',
    auditTrailActionListForJob : 'scheduler/quartz/audit/QuartzAuditTrail.action?listForJob=',
    auditTrailActionListForTrigger : 'scheduler/quartz/audit/QuartzAuditTrail.action?listForTrigger=',
    auditTrailActionListForTargetObject : 'scheduler/quartz/audit/QuartzAuditTrail.action?listForTargetObject='
  }
} );
