/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various message queue-related utility methods.
 */
Ext.define( 'QDesk.util.message.MessageQueueUtils', {
  singleton : true,

  requires : [
    'QDesk.Log',
    'QDesk.Msg',

    'QDesk.util.CommonUtils',
    'QDesk.util.constant.ActionUrl'
  ],


  /**
   * Creates and returns HTML code with message details displayed by the rowexpander in the MessageQueueGrid.
   * @param values a message value object (not a Message model instance!).
   * @param ownerGrid the owner grid.
   * @returns {string}
   */
  createRowDetailsHtml : function ( values, ownerGrid )
  {
    var html = '<table class="rowDetailsTable">' +
        // '<thead>' +
        // '<tr><th colspan="2" style="width: 100px;">' +
        // QDesk.Msg.get( 'QDesk.util.message.MessageQueueUtils.rowDetailsTable.columnNameDetails' ) +
        // '</th></tr>' +
        // '</thead>' +
        '<tbody>';

    if ( !Ext.isEmpty( values.toAddr ) )
    {
      html += '<tr>';
      html += '<td style="font-weight: bold; width: 100px;">' +
          QDesk.Msg.get( 'QDesk.util.message.MessageQueueUtils.rowDetailsTable.rowToAddr' ) + '</td>';
      html += '<td>' + Ext.String.htmlEncode( values.toAddr ) + '</td>';
      html += '</tr>';
    }

    if ( !Ext.isEmpty( values.subject ) )
    {
      html += '<tr>';
      html += '<td style="font-weight: bold; width: 100px;">' +
          QDesk.Msg.get( 'QDesk.util.message.MessageQueueUtils.rowDetailsTable.rowSubject' ) + '</td>';
      html += '<td><pre>' + Ext.String.htmlEncode( values.subject ) + '</pre></td>';
      html += '</tr>';
    }

    if ( !Ext.isEmpty( values.body ) )
    {
      html += '<tr>';
      html += '<td style="font-weight: bold; width: 100px;">' +
          QDesk.Msg.get( 'QDesk.util.message.MessageQueueUtils.rowDetailsTable.rowBody' ) + '</td>';
      html += '<td><pre>' + Ext.String.htmlEncode( values.body ) + '</pre></td>';
      html += '</tr>';
    }

    if ( !Ext.isEmpty( values.attachment ) )
    {
      html += '<tr>';
      html += '<td style="font-weight: bold; width: 100px;">' +
          QDesk.Msg.get( 'QDesk.util.message.MessageQueueUtils.rowDetailsTable.rowAttachments' ) + '</td>';
      html += '<td>';

      Ext.each( values.attachment, function ( attachment )
      {
        var attachmentDownloadUrl = QDesk.util.constant.ActionUrl.messageActionDownloadAttachment +
            '&connectionId=' + ownerGrid.qdesk.connectionId +
            '&attachmentId=' + attachment.id;

        html += '<a href="' + attachmentDownloadUrl + '" target="msg-' + attachment.id + '">' +
            '<pre>' + Ext.String.htmlEncode( attachment.name ) + '</pre></a>';

        html += '<iframe name="msg-' + attachment.id + '" height="0" width="0" style="display:none"></iframe>';
      } );

      html += '</td>';
      html += '</tr>';
    }

    html += '</tbody>' +
        '</table>';

    return html;
  }
} );
