/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various Quartz job-specific utility methods.
 */
Ext.define( 'QDesk.util.quartz.QuartzJobUtils', {
  requires : [
    'QDesk.Msg',

    'QDesk.util.license.LicensedProductFeatures'
  ],

  singleton : true,


  /**
   * Creates and returns HTML code with job details displayed by the rowexpander in the QuartzJobsGrid.
   * @param values a job detail value object (not a QuartzJobDetail model instance!).
   * @param ownerGrid the owner grid.
   * @returns {string}
   */
  createRowDetailsHtml : function ( values, ownerGrid )
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var html = '<table class="rowDetailsTable">' +
        '<tbody>';

    /* Start of Description */
    var description = Ext.isEmpty( values.description ) ? QDesk.Msg.get( 'QDesk.notAvailable' ) : values.description;

    html += '<tr>';
    html += '<td style="font-weight: bold; width: 100px;">' + QDesk.Msg.get( 'QDesk.description' ) + '</td>';
    html += '<td>' + description + '</td>';
    html += '</tr>';
    /* End of Description */

    /* Start of Monitoring URL */
    html += '<tr>';
    html += '<td style="font-weight: bold; width: 100px;">' + QDesk.Msg.get( 'QDesk.monitoringUrl' ) + '</td>';
    html += '<td>';

    var monitoringLinkUrl = values.monitoringUrl;
    var monitoringLinkTooltip;
    if ( licensedFeatures.PollMonitoring )
    {
      monitoringLinkTooltip =
          Ext.String.htmlEncode(
              QDesk.Msg.get( 'QDesk.util.quartz.QuartzJobUtils.rowDetailsTable.tooltipMonitoringUrl' ) );

      html += '<a href="' + monitoringLinkUrl +
          '" class="newWindow" target="_blank" data-qtip="' + monitoringLinkTooltip + '">' + monitoringLinkUrl + '</a>';
    }
    else
    {
      monitoringLinkTooltip = Ext.String.htmlEncode(
          QDesk.Msg.get( 'QDesk.util.quartz.QuartzJobUtils.rowDetailsTable.tooltipMonitoringUrlNotLicensed' ) );

      html += '<a href="' + monitoringLinkUrl +
          '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
          monitoringLinkTooltip + '">' + monitoringLinkUrl + '</a>';
    }

    html += '</td></tr>';
    /* End of Monitoring URL */

    html += '</tbody>' +
        '</table>';

    return html;
  },


  /**
   * Creates a clone of the specified QDesk.model.scheduler.quartz.QuartzJobDetail instance.
   * @param jobDetail a QDesk.model.scheduler.quartz.QuartzJobDetail instance.
   * @return the clone.
   */
  cloneJobDetail : function ( jobDetail )
  {
    var clone = jobDetail.copy();  // does not clone the jobDataMap array

    var jobDataMap = jobDetail.get( 'jobDataMap' );

    if ( jobDataMap )
    {
      // assert Ext.isArray(jobDataMap)
      clone.set( 'jobDataMap', Ext.Array.clone( jobDataMap ) );
    }

    return clone;
  }
} );
