/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various group-related utility methods.
 */
Ext.define( 'QDesk.util.security.GroupUtils', {
  singleton : true,

  requires : [
    'QDesk.model.security.Group'
  ],


  /**
   * Returns the GroupsTab if it is currently opened, null otherwise.
   * @return the GroupsTab if it is currently opened, null otherwise.
   */
  getGroupsTab : function ()
  {
    var groupsTab = Ext.ComponentQuery.query( '#mainTabPanel > GroupsTab' );
    return Ext.isEmpty( groupsTab ) ? null : groupsTab[ 0 ];
  },


  /**
   * Creates a QDesk.model.security.Group model instance and populates it with the specified EditGroupForm field values.
   * @param fieldValues a EditGroupForm field values.
   * @return the created QDesk.model.security.Group model instance.
   */
  createGroup : function ( fieldValues )
  {
    var me = this;

    var group = Ext.create( 'QDesk.model.security.Group' );

    // set ID only for an updated group that has a numerical ID; newly added groups have textual ExtJS IDs
    if ( Ext.isNumeric( fieldValues.id ) )
    {
      group.set( 'id', fieldValues.id );
    }
    else
    {
      group.set( 'id', null );
    }

    if ( !Ext.isEmpty( fieldValues.system ) )
      group.set( 'system', fieldValues.system );

    if ( !Ext.isEmpty( fieldValues.name ) )
      group.set( 'name', fieldValues.name );

    if ( !Ext.isEmpty( fieldValues.permissionIds ) )
      group.set( 'permissionIds', fieldValues.permissionIds );

    if ( !Ext.isEmpty( fieldValues.userIds ) )
      group.set( 'userIds', fieldValues.userIds );

    return group;
  }
} );
