/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various service endpoint-related utility methods.
 */
Ext.define( 'QDesk.util.system.ws.ServiceEndpointUtils', {
  singleton : true,

  requires : [
    'QDesk.Log',
    'QDesk.Msg',

    'QDesk.util.CommonUtils',
    'QDesk.util.FormatUtils'
  ],


  /**
   * Creates and returns HTML code with service endpoint details displayed by the rowexpander in the ServiceEndpointsGrid.
   * @param values a service endpoint value object (not a ServiceEndpoint model instance!).
   * @param ownerGrid the owner grid.
   * @returns {string}
   */
  createRowDetailsHtml : function ( values, ownerGrid )
  {
    var me = this;

    var html = '<table class="rowDetailsTable">' +
        // '<thead>' +
        // '<tr><th colspan="2" style="width: 100px;">' +
        // QDesk.Msg.get( 'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnDetails' ) +
        // '</th></tr>' +
        // '</thead>' +
        '<tbody>';

    html += '<tr>';
    html += '<td style="font-weight: bold; width: 100px;">' +
        QDesk.Msg.get( 'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.rowEndpointUrl' ) + '</td>';
    html += '<td><a href="' + values.url + '" class="newWindow" target="_blank" data-qtip="' +
        QDesk.Msg.get( 'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.tooltipEndpointUrl' ) +
        '">' + values.url +
        '</a></td>';
    html += '</tr>';

    html += '<tr>';
    html += '<td style="font-weight: bold; width: 100px;">' +
        QDesk.Msg.get( 'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.rowEndpointWsdlUrl' ) + '</td>';
    html += '<td><a href="' + values.wsdlUrl + '" class="newWindow" target="_blank" data-qtip="' +
        QDesk.Msg.get( 'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.tooltipEndpointWsdlUrl' ) +
        '">' + values.wsdlUrl + '</a></td>';
    html += '</tr>';

    html += '<tr>';
    html += '<td style="font-weight: bold; width: 100px;">' +
        QDesk.Msg.get( 'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.rowEndpointWsdlPath' ) + '</td>';
    html += '<td><span class="pre">' + values.wsdlPath + '</span></td>';
    html += '</tr>';

    html += '</tbody>' +
        '</table>';

    html += me._createOperationsTable( values.operation );

    return html;
  },


  _createOperationsTable : function ( operations )
  {
    var html = '<table class="rowDetailsTable">' +
        '<thead>' +
        '<tr>' +

        '<th style="min-width: 160px" data-qtip="' + Ext.String.htmlEncode(
            QDesk.Msg.get( 'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnTooltipOpName' ) ) + '">' +
        QDesk.Msg.get( 'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnOpName' ) +
        '</th>' +

        '<th width="15%" data-qtip="' + Ext.String.htmlEncode( QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnTooltipOpLastStartedAt' ) ) + '">' +
        QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnOpLastStartedAt' ) +
        '</th>' +

        '<th width="15%" data-qtip="' + Ext.String.htmlEncode( QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnTooltipOpLastFinishedAt' ) ) + '">' +
        QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnOpLastFinishedAt' ) +
        '</th>' +

        '<th width="10%" data-qtip="' + Ext.String.htmlEncode(
            QDesk.Msg.get( 'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnTooltipOpLastDuration' ) ) +
        '">' +
        QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnOpLastDuration' ) +
        '</th>' +

        '<th width="10%" style="text-align: center;" data-qtip="' + Ext.String.htmlEncode( QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnTooltipOpLastErrorDesc' ) ) + '">' +
        QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnOpLastErrorDesc' ) +
        '</th>' +

        '<th width="10%" data-qtip="' + Ext.String.htmlEncode(
            QDesk.Msg.get( 'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnTooltipOpSuccessCount' ) ) +
        '">' +
        QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnOpSuccessCount' ) +
        '</th>' +

        '<th width="10%" data-qtip="' + Ext.String.htmlEncode( QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnTooltipOpAvgSuccessDuration' ) ) + '">' +
        QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnOpAvgSuccessDuration' ) +
        '</th>' +

        '<th width="10%" data-qtip="' + Ext.String.htmlEncode(
            QDesk.Msg.get( 'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnTooltipOpErrorCount' ) ) +
        '">' +
        QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnOpErrorCount' ) +
        '</th>' +

        '<th width="10%" data-qtip="' + Ext.String.htmlEncode( QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnTooltipOpAvgErrorDuration' ) ) + '">' +
        QDesk.Msg.get(
            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.columnOpAvgErrorDuration' ) +
        '</th>' +

        '</tr>' +
        '</thead>' +
        '<tbody>';

    if ( Ext.isEmpty( operations ) )
    {
      html += '<tr><td colspan="9" align="center">' +
          QDesk.Msg.get( 'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.msgNoOperationsFound' ) +
          '</td></tr>';
    }
    else
    {
      Ext.each( operations,
          function ( operation )
          {
            html += '<tr>';
            html += '<td>' + operation.name + '</td>';
            html +=
                '<td>' + ( operation.lastStartedAt == null ? '' : operation.lastStartedAt.targetFmtValue ) + '</td>';
            html +=
                '<td>' + ( operation.lastFinishedAt == null ? '' : operation.lastFinishedAt.targetFmtValue ) + '</td>';

            html += '<td>' +
                ( operation.lastDuration == null ? '' :
                    QDesk.util.FormatUtils.formatDuration( operation.lastDuration ) ) +
                '</td>';

            html += '<td align="center">' +
                ( Ext.isEmpty( operation.lastErrorDesc ) ? '' :
                    '<img class="clickable" src="images/icon/error-16x16.png" data-qtip="' +
                    Ext.String.htmlEncode(
                        QDesk.Msg.get(
                            'QDesk.util.system.ws.ServiceEndpointUtils.rowDetailsTable.iconTooltipOpLastErrorDesc' ) ) +
                    '" data-operation-id="' + operation.id + '"/>' ) +
                '</td>';

            html += '<td>' + operation.countSuccess + '</td>';
            html += '<td>' + ( operation.countSuccess === 0 ? '' :
                QDesk.util.FormatUtils.formatDuration(
                    Math.round( operation.timeSuccess / operation.countSuccess ) ) ) +
                '</td>';

            html += '<td>' + operation.countError + '</td>';
            html += '<td>' + ( operation.countError === 0 ? '' :
                QDesk.util.FormatUtils.formatDuration( Math.round( operation.timeError / operation.countError ) ) ) +
                '</td>';

            html += '</tr>';
          } );
    }

    html += '</tbody>' +
        '</table>';

    return html;
  }
} );
