/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.Viewport', {
  extend : 'Ext.container.Viewport',

  requires : [
    'QDesk.view.lock.LockWindow',
    'QDesk.view.security.SecurityPanel',
    'QDesk.view.work.WorkPanel'
  ],

  border : false,

  layout : {
    type : 'card'
  },

  qdesk : {
    /*
     * Internal flag indicating if the viewport has been locked or not.
     */
    locked : false
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.product )
      Ext.Error.raise( 'Required config parameter missing: qdesk.product' );

    if ( !config.qdesk.license )
      Ext.Error.raise( 'Required config parameter missing: qdesk.license' );

    if ( !config.qdesk.user )
      Ext.Error.raise( 'Required config parameter missing: qdesk.user' );

    var productInfo = config.qdesk.product;
    var licenseInfo = config.qdesk.license;
    var userInfo = config.qdesk.user;

    // we do not want the work panel to cache the qdesk data
    delete config.qdesk;

    Ext.apply( config, {
      items : [
        {
          xtype : 'SecurityPanel',
          itemId : 'securityPanel',
          qdesk : {
            license : licenseInfo
          }
        },
        {
          xtype : 'WorkPanel',
          itemId : 'workPanel',
          qdesk : {
            product : productInfo,
            license : licenseInfo
          }
        }
      ]
    } );

    me.callParent( [ config ] );

    if ( userInfo.authenticated )
    {
      me.setActiveItem( 1 );
    }
    else
    {
      me.setActiveItem( 0 );
    }
  },


  activateSecurityLoginPanel : function ()
  {
    var me = this;

    /*
     * Close all opened windows (floating components).
     */
    Ext.WindowManager.each(
        function ( comp )
        {
          if ( comp.close )
            comp.close();
        } );

    var workPanel = me.down( '#workPanel' );

    /*
     * Close all closable tabs in the MainTabPanel (direct children of MainTabPanel). The WelcomePanel is not closable
     * and as such is not closed.
     */
    Ext.each( Ext.ComponentQuery.query( '#mainTabPanel > panel', workPanel ),
        function ( panel )
        {
          if ( panel.closable )
            panel.close();
        } );


    var securityPanel = me.down( '#securityPanel' );
    securityPanel.activateLoginPanel();
    me.setActiveItem( securityPanel );
  },


  activateWorkPanel : function ()
  {
    var me = this;

    /*
     * Close all opened windows (floating components).
     */
    Ext.WindowManager.each(
        function ( comp )
        {
          if ( comp.close )
            comp.close();
        } );

    var workPanel = me.down( '#workPanel' );
    me.setActiveItem( workPanel );
  },


  /**
   * Locks the GUI by opening a lock window showing the specified message. If the GUI has already been
   * locked, then this function simply returns.
   * @param msg a message to show.
   */
  lock : function ( msg )
  {
    var me = this;
    if ( me.qdesk.locked )
    {
      // already locked - no need to open the lock window
      var lockWindows = Ext.ComponentQuery.query( 'LockWindow' );
      if ( Ext.isEmpty( lockWindows ) )
      {
        // should not happen (viewport locked, but there is no lock window)
      }
      else
      {
        var lockWindow = lockWindows[ 0 ];
        lockWindow.updateMessage( msg );
      }
    }
    else
    {
      me.qdesk.locked = true;

      Ext.widget( 'LockWindow', {
        qdesk : {
          msg : msg
        }
      } ).show();
    }
  },


  /**
   * Unlocks the GUI if it is currently locked.
   */
  unlock : function ()
  {
    var me = this;

    if ( me.qdesk.locked )
    {
      Ext.each( Ext.ComponentQuery.query( 'LockWindow' ), function ( lockWindow )
      {
        lockWindow.close();
      } );

      me.qdesk.locked = false;
    }
  }
} );