/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.widget.expression.JSScriptingObjectsForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.view.widget.expression.JSScriptingDataTypesPanel',
    'QDesk.view.widget.expression.JSScriptingObjectsPanel'
  ],

  alias : 'widget.expression.JSScriptingObjectsForm',

  fieldDefaults : {
    labelWidth : 80
  },

  /* Override the default 10px body padding. We want 0 body padding for this particular form. */
  bodyPadding : 0,

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  // required config parameter with the objects tree store
  objectsStore : null,

  // required config parameter with the data types tree store
  dataTypesStore : null,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.objectsStore )
      Ext.Error.raise( 'Required config parameter missing: objectsStore' );

    if ( !config.dataTypesStore )
      Ext.Error.raise( 'Required config parameter missing: dataTypesStore' );

    Ext.apply( config, {
          items : [
            {
              xtype : 'tabpanel',
              itemId : 'jsScriptingObjectsTabPanel',
              activeTab : 0,
              border : false,
              flex : 1,
              defaults : {
                layout : {
                  type : 'vbox',
                  align : 'stretch'
                }
              },

              items : [
                {
                  xtype : 'panel',
                  itemId : 'objectsTab',
                  title : QDesk.Msg.get( 'QDesk.view.widget.expression.JSScriptingObjectsWindow.titleObjectsTab' ),
                  border : false,
                  closable : false,
                  items : [
                    {
                      xtype : 'expression.JSScriptingObjectsPanel',
                      itemId : 'objectsPanel',
                      store : config.objectsStore,
                      flex : 1
                    }
                  ]
                },
                {
                  xtype : 'panel',
                  itemId : 'dataTypesTab',
                  title : QDesk.Msg.get( 'QDesk.view.widget.expression.JSScriptingObjectsWindow.titleDataTypesTab' ),
                  border : false,
                  closable : false,
                  items : [
                    {
                      xtype : 'expression.JSScriptingDataTypesPanel',
                      itemId : 'dataTypesPanel',
                      store : config.dataTypesStore,
                      flex : 1
                    }
                  ]
                }
              ]
            }
          ],

          buttons : [
            {
              itemId : 'closeButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelClose' )
            }
          ]
        }
    );

    me.callParent( [ config ] );
  }
} );
