/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * A closable tab with an iframe that displays the specified URL. The iframe fills in the entire area of the tab.
 */
Ext.define( 'QDesk.view.work.ViewUrlTab', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.util.UxUtils'
  ],

  alias : 'widget.ViewUrlTab',

  cls : 'viewUrlTab',
  bodyCls : 'viewUrlTabBody',

  layout : 'fit',

  closable : true,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.itemId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.itemId' );

    if ( !config.qdesk.url )
      Ext.Error.raise( 'Required config parameter missing: qdesk.url' );

    if ( !config.qdesk.title )
      Ext.Error.raise( 'Required config parameter missing: qdesk.title' );

    var itemId = config.qdesk.itemId;
    var url = config.qdesk.url;
    var title = config.qdesk.title;
    var icon = config.qdesk.icon;

    Ext.apply( config, {
      title : title,
      itemId : itemId,
      items : [
        {
          xtype : 'uxiframe',
          loadMask : null  /* delete the default load mask so that we can use our own load mask */
        }
      ]
    } );

    if ( icon )
    {
      Ext.apply( config, {
        icon : icon
      } );
    }

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.on( 'render', function ()
    {
      var iframe = me.down( 'uxiframe' );

      iframe.on( 'beforeload', function ()
      {
        QDesk.util.UxUtils.maskBody( QDesk.Msg.get( 'QDesk.waitMsgOpen', me.title ) );
      } );

      iframe.on( 'load', function ()
      {
        me._maybeDisableContextMenu( iframe );
        QDesk.util.UxUtils.unmaskBody();
      } );

      iframe.on( 'error', function ()
      {
        me._maybeDisableContextMenu( iframe );
        QDesk.util.UxUtils.unmaskBody();
      } );

      // we must call the load function after the iframe has been rendered (i.e. there is an iframe element)
      iframe.on( 'render', function ()
      {
        iframe.load( me.qdesk.url );
      } );
    } );

    me.callParent();
  },


  /**
   * Disables the browser context menu if the iframe is loaded from the same domain.
   * @param iframe an iframe.
   * @private
   */
  _maybeDisableContextMenu : function ( iframe )
  {
    var doc = iframe.getDoc();
    if ( doc == null )
    {
      // getDoc() returns null if the document is not accessible (e.g. the iframe is loaded from a different domain)
    }
    else
    {
      doc.oncontextmenu = function ()
      {
        return false;
      }
    }
  }
} );
