/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.WorkPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.view.work.MainTabPanel',
    'QDesk.view.work.navigation.NavigationPanel',
    'QDesk.view.work.top.TopContainer',

    'QDesk.util.constant.EventName'
  ],

  alias : 'widget.WorkPanel',

  border : false,

  layout : {
    type : 'border'
  },

  minWidth : 700,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.product )
      Ext.Error.raise( 'Required config parameter missing: qdesk.product' );

    if ( !config.qdesk.license )
      Ext.Error.raise( 'Required config parameter missing: qdesk.license' );

    var productInfo = config.qdesk.product;
    var licenseInfo = config.qdesk.license;

    // we do not want the work panel to cache the qdesk data
    delete config.qdesk;

    Ext.apply( config, {
      items : [
        {
          xtype : 'TopContainer',
          itemId : 'topContainer',
          region : 'north',
          height : 40,
          maxHeight : 40,
          minHeight : 40,
          qdesk : {
            product : productInfo,
            license : licenseInfo
          }
        },
        {
          xtype : 'NavigationPanel',
          itemId : 'navigationPanel',
          region : 'west',
          width : 230,
          minHeight : 100
        },
        {
          xtype : 'MainTabPanel',
          itemId : 'mainTabPanel',
          region : 'center'
        }
      ]
    } );

    me.callParent( [ config ] );

    me.on( 'activate', function ()
    {
      QDesk.getApplication().fireEvent( QDesk.util.constant.EventName.WORK_PANEL_ACTIVATED, {
        workPanel : this
      } );
    } );

    me.on( 'deactivate', function ()
    {
      QDesk.getApplication().fireEvent( QDesk.util.constant.EventName.WORK_PANEL_DEACTIVATED, {
        workPanel : this
      } );
    } );
  }
} );