/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.audit.AuditTrailTab', {
  extend : 'QDesk.view.work.BaseWorkTab',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.work.audit.AuditTrailGrid'
  ],

  alias : 'widget.AuditTrailTab',

  cls : 'auditTrailTab',
  bodyCls : 'auditTrailTabBody',

  iconCls : 'auditTrailTabIcon',

  // border : false,
  // closable : true,
  // scrollable : true,

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  qdRequiresAnyPermissions : [ QDesk.model.security.Permission.AUDIT_TRAIL_VIEW ],
  qdNotAuthorizedIconCls : 'auditTrailTabIconNAUTH',


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.General;

    if ( licensedFeatures.AuditTrail )
    {
      var items = [];

      items.push(
          {
            xtype : 'AuditTrailGrid',
            itemId : 'auditTrailGrid',
            flex : 1,
            qdesk : {
              gridId : 'auditTrailGrid'
            }
          }
      );

      /*
       * Intentionally over-nested a bit so that we use identical layout to other tabs that
       * have a license restrictions related message panel above the grid.
       */
      Ext.apply( config, {
        items : [
          {
            xtype : 'panel',
            border : false,
            layout : 'fit',
            flex : 1,
            items : [
              {
                xtype : 'panel',
                border : false,
                layout : {
                  type : 'vbox',
                  align : 'stretch'
                },
                items : items
              }
            ]
          }
        ]
      } );
    }
    else
    {
      Ext.apply( config, {
        items : [
          {
            xtype : 'WarningMessagePanel',
            style : {
              margin : '5px'
            },
            qdesk : {
              msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
                  QDesk.util.license.LicensedProduct.productName )
            }
          }
        ]
      } );
    }

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailTab.title' ),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailTab.tooltip' )
    } );

    me.callParent( [ config ] );

    //me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    var usersGrid = me.down( '#usersGrid' );

    usersGrid.on( 'addUser', function ()
    {
      me._cleanupEditPanel();

      Ext.create( 'QDesk.controller.command.security.users.AddUser', {} ).execute();
    } );
  }
} );