/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.im.yahoo.YahooProfilePanel', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.view.work.message.channel.ProxyPanel',
    'QDesk.view.work.message.channel.im.yahoo.YahooAdvancedPanel',
    'QDesk.view.work.message.channel.im.yahoo.YahooBasicPanel'
  ],

  alias : 'widget.ImYahooProfilePanel',

  defaultButton : 'saveButton',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  bodyPadding : 0,

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '3px 10px 5px 10px',
          items : [
            {
              xtype : 'panel',
              bodyCls : 'IM_YAHOO',
              border : false,
              width : 32,
              height : 32,
              html : QDesk.Msg.get( 'QDesk.view.work.message.channel.im.yahoo.YahooProfilePanel.title' )
            },

            {
              xtype : 'displayfield',
              value : QDesk.Msg.get( 'QDesk.view.work.message.channel.im.yahoo.YahooProfilePanel.disclaimerMsg' ),
              fieldCls : 'disclaimerMessageChannel'
            },

            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.work.message.channel.im.yahoo.YahooProfilePanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.im.yahoo.YahooProfilePanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.im.yahoo.YahooProfilePanel.fieldEmptyTxtName' ),
              value : QDesk.Msg.get( 'QDesk.unnamed' ),
              allowBlank : false,
              maxLength : 128,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'tabpanel',
          border : false,
          bodyPadding : 10,
          items : [
            {
              xtype : 'ImYahooBasicPanel'
            },
            {
              xtype : 'ImYahooAdvancedPanel'
            },
            {
              xtype : 'ProxyPanel'
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'sendTestMessageButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSendTestMessageEllipsis' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        },
        {
          itemId : 'saveButton',
          reference : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  /**
   * Populates this form panel from the specified profile object.
   * @param profile a profile object.
   */
  populate : function ( profile )
  {
    var me = this;

    var values = {};

    Ext.apply( values, {
      id : profile.id,
      name : profile.name,
      username : profile.channelConfig.username,
      password : profile.channelConfig.password
    } );

    if ( profile.channelConfig.loginHost )
    {
      Ext.apply( values, {
        loginHost : profile.channelConfig.loginHost
      } );
    }

//    if ( profile.channelConfig.loginPort )
//    {
//      Ext.apply( values, {
//        loginPort: profile.channelConfig.loginPort
//      } )
//    }

    var proxyPanel = me.down( 'ProxyPanel' );
    Ext.apply( values, proxyPanel.proxyConfig2FieldValues( profile.channelConfig.proxy ) );

    // based on the selected proxy type make the proxy host & port form fields required
    proxyPanel.updateProxyFieldsAllowsBlank( profile.channelConfig.proxy );

    me.getForm().setValues( values );
  },

  /**
   * Creates a new {@link MessageChannelProfile} model instance populated with
   * form data.
   */
  createMessageChannelProfile : function ()
  {
    var me = this;

    var profile = Ext.create( 'QDesk.model.message.MessageChannelProfile' );

    var fieldValues = me.getForm().getFieldValues();

    if ( !Ext.isEmpty( fieldValues.id ) )
      profile.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.name ) )
      profile.set( 'name', fieldValues.name );

    profile.set( 'channel', QDesk.model.message.channel.MessageChannel.IM_YAHOO );

    var channelConfig = {};

    Ext.apply( channelConfig, {
      username : fieldValues.username,
      password : fieldValues.password
    } );

    if ( !Ext.isEmpty( fieldValues.loginHost ) )
    {
      Ext.apply( channelConfig, {
        loginHost : fieldValues.loginHost
      } );
    }

//    if ( !Ext.isEmpty( fieldValues.loginPort ) )
//    {
//      Ext.apply( channelConfig, {
//        loginPort: fieldValues.loginPort
//      } );
//    }

    var proxyPanel = me.down( 'ProxyPanel' );
    var proxy = proxyPanel.fieldValues2ProxyConfig( fieldValues );
    if ( !Ext.isEmpty( proxy ) )
    {
      Ext.apply( channelConfig, {
        proxy : proxy
      } );
    }

    profile.set( 'channelConfig', channelConfig );

    return profile;
  }
} );
