/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.incident_mgmt.jira.JiraProfilePanel', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.util.FormatUtils',

    'QDesk.view.work.message.channel.HttpProxyPanel',
    'QDesk.view.work.message.channel.incident_mgmt.jira.JiraBasicPanel'
  ],

  alias : 'widget.IncidentMgmtJiraProfilePanel',

  defaultButton : 'saveButton',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  bodyPadding : 0,

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '3 10 5 10',
          items : [
            {
              xtype : 'panel',
              bodyCls : 'INCIDENT_MGMT_JIRA',
              border : false,
              width : 32,
              height : 32,
              html : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.jira.JiraProfilePanel.title' )
            },

            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.jira.JiraProfilePanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.jira.JiraProfilePanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.incident_mgmt.jira.JiraProfilePanel.fieldEmptyTxtName' ),
              value : QDesk.Msg.get( 'QDesk.unnamed' ),
              allowBlank : false,
              maxLength : 128,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'tabpanel',
          border : false,
          bodyPadding : 10,
          items : [
            {
              xtype : 'IncidentMgmtJiraBasicPanel'
            },
            {
              xtype : 'HttpProxyPanel'
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'sendTestMessageButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSendTestMessage' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        },
        {
          itemId : 'saveButton',
          reference : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  /**
   * Populates this form panel from the specified profile object.
   * @param profile a profile object.
   */
  populate : function ( profile )
  {
    var me = this;

    var values = {};

    Ext.apply( values, {
      id : profile.id,
      name : profile.name,
      username : profile.channelConfig.username,
      apiToken : profile.channelConfig.apiToken,
      baseUrl : profile.channelConfig.baseUrl,
      projectKey : profile.channelConfig.projectKey,
      issueType : profile.channelConfig.issueType
    } );

    // if ( !Ext.isEmpty( profile.channelConfig.label ) )
    // {
    //   Ext.apply( values, {
    //     tag : profile.channelConfig.label.join( ', ' )
    //   } );
    // }

    var proxyPanel = me.down( 'HttpProxyPanel' );
    Ext.apply( values, proxyPanel.proxyConfig2FieldValues( profile.channelConfig.proxy ) );

    // based on the selected proxy type make the proxy host & port form fields required
    proxyPanel.updateProxyFieldsAllowsBlank( profile.channelConfig.proxy );

    me.getForm().setValues( values );
  },


  /**
   * Creates a new {@link MessageChannelProfile} model instance populated with form data.
   */
  createMessageChannelProfile : function ()
  {
    var me = this;

    var profile = Ext.create( 'QDesk.model.message.MessageChannelProfile' );

    var fieldValues = me.getForm().getFieldValues();

    if ( !Ext.isEmpty( fieldValues.id ) )
      profile.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.name ) )
      profile.set( 'name', fieldValues.name );

    profile.set( 'channel', QDesk.model.message.channel.MessageChannel.INCIDENT_MGMT_JIRA );

    var channelConfig = {};

    Ext.apply( channelConfig, {
      username : fieldValues.username,
      apiToken : fieldValues.apiToken,
      baseUrl : fieldValues.baseUrl,
      projectKey : fieldValues.projectKey,
      issueType : fieldValues.issueType
    } );

    // // label array
    // if ( !Ext.isEmpty( fieldValues.label ) )
    // {
    //   var labelArray = QDesk.util.FormatUtils.parseCommaSeparatedList( fieldValues.label );
    //   if ( labelArray.length > 0 )
    //   {
    //     Ext.apply( channelConfig, {
    //       label : labelArray
    //     } );
    //   }
    // }

    var proxyPanel = me.down( 'HttpProxyPanel' );
    var proxy = proxyPanel.fieldValues2ProxyConfig( fieldValues );
    if ( !Ext.isEmpty( proxy ) )
    {
      Ext.apply( channelConfig, {
        proxy : proxy
      } );
    }

    profile.set( 'channelConfig', channelConfig );

    return profile;
  }
} );
