/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.Properties',
    'QDesk.controller.command.message.channel.snmp.GenerateMib',
    'QDesk.model.message.channel.MessageChannel'
  ],

  alias : 'widget.SnmpV3TrapBasicPanel',

  border : false,

  layout : 'anchor',

  defaults : {
    anchor : '100%',
    labelWidth : 100
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.tabLabelBasic' ),

      items : [
        {
          xtype : 'textfield',
          itemId : 'host',
          name : 'host',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldLabelHost' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldTooltipHost' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldEmptyTxtHost' ),
          allowBlank : false,
          maxLength : 64,
          qdesk : {
            required : true
          }
        },

        {
          xtype : 'numberfield',
          itemId : 'port',
          name : 'port',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldLabelPort' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldTooltipPort' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldEmptyTxtPort' ),
          anchor : '60%',
          value : 162, /* default SNMP UDP port */
          allowBlank : false,
          minValue : 1,
          maxValue : 65535,
          qdesk : {
            required : true
          }
        },

        {
          xtype : 'textfield',
          itemId : 'trapOid',
          name : 'trapOid',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldLabelTrapOid' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldTooltipTrapOid' ),
          readOnly : true,
          value : QDesk.Properties.get( 'oid.messageTrapV2' ),
          allowBlank : false,
          maxLength : 64,
          qdesk : {
            required : true
          }
        },

        {
          xtype : 'textfield',
          itemId : 'systemDescription',
          name : 'systemDescription',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldLabelSystemDescription' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldTooltipSystemDescription' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldEmptyTxtSystemDescription' ),
          allowBlank : true,
          maxLength : 255,
          qdesk : {
            required : false
          }
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get(
              'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldsetLabelExtraVariables' ),
          padding : '5 0 0 5',
          defaults : {
            labelWidth : 95
          },
          items : [
            {
              xtype : 'numberfield',
              itemId : 'variableMessageTrapCustomType',
              name : 'variableMessageTrapCustomType',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldLabelVariableMessageTrapCustomType' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldTooltipVariableMessageTrapCustomType' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.fieldEmptyTxtVariableMessageTrapCustomType' ),
              value : 1,
              allowBlank : false,
              minValue : 1,
              maxValue : 65535,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'displayfield',
          renderer : function ( value, field )
          {
            var tooltip = Ext.String.htmlEncode(
                QDesk.Msg.get( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.tooltipLinkDownloadMibs' ) );
            return '<a href="files/snmp-mib.zip" class="newWindow" target="_blank" data-qtip="' + tooltip +
                '">' +
                QDesk.Msg.get( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel.textLinkDownloadMibs' ) +
                '</a>';
          }
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

//    me.down( '#generateMibButton' ).on( 'click', function ()
//    {
//      var messageChannelProfile = me.up( 'SnmpV3TrapProfilePanel' ).createMessageChannelProfile();
//
//      Ext.create( 'QDesk.controller.command.message.channel.snmp.GenerateMib', {
//        maskEl : me.up( 'window' ).getEl(),
//        channel : QDesk.model.message.channel.MessageChannel.SNMPV3_TRAP,
//        channelConfig : messageChannelProfile.get( 'channelConfig' )
//      } ).execute();
//    } );
  }
} );
