/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapProfilePanel', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.view.work.message.channel.snmp.SnmpV3TrapBasicPanel',
    'QDesk.view.work.message.channel.snmp.SnmpV3TrapSecurityPanel'
  ],

  alias : 'widget.SnmpV3TrapProfilePanel',

  defaultButton : 'saveButton',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  bodyPadding : 0,

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '3px 10px 5px 10px',
          items : [
            {
              xtype : 'panel',
              bodyCls : 'SNMPV3_TRAP',
              border : false,
              width : 32,
              height : 32,
              html : QDesk.Msg.get( 'QDesk.view.work.message.channel.snmp.SnmpV3TrapProfilePanel.title' )
            },

            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.snmp.SnmpV3TrapProfilePanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.snmp.SnmpV3TrapProfilePanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.snmp.SnmpV3TrapProfilePanel.fieldEmptyTxtName' ),
              value : QDesk.Msg.get( 'QDesk.unnamed' ),
              allowBlank : false,
              maxLength : 128,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'tabpanel',
          border : false,
          bodyPadding : 10,
          items : [
            {
              xtype : 'SnmpV3TrapBasicPanel'
            },
            {
              xtype : 'SnmpV3TrapSecurityPanel'
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'sendTestMessageButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSendTestMessage' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        },
        {
          itemId : 'saveButton',
          reference : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  /**
   * Populates this form panel from the specified profile object.
   * @param profile a profile object.
   */
  populate : function ( profile )
  {
    var me = this;

    var values = {};

    Ext.apply( values, {
      id : profile.id,
      name : profile.name
    } );

    if ( profile.channelConfig.host )
    {
      Ext.apply( values, {
        host : profile.channelConfig.host
      } );
    }

    if ( profile.channelConfig.port )
    {
      Ext.apply( values, {
        port : profile.channelConfig.port
      } );
    }

    if ( profile.channelConfig.community )
    {
      Ext.apply( values, {
        community : profile.channelConfig.community
      } );
    }

//    if ( profile.channelConfig.trapOid )
//    {
//      Ext.apply( values, {
//        trapOid : profile.channelConfig.trapOid
//      } );
//    }

    if ( profile.channelConfig.systemDescription )
    {
      Ext.apply( values, {
        systemDescription : profile.channelConfig.systemDescription
      } );
    }

    if ( profile.channelConfig.variableMessageTrapCustomType )
    {
      Ext.apply( values, {
        variableMessageTrapCustomType : profile.channelConfig.variableMessageTrapCustomType
      } );
    }

    var securityPanel = me.down( 'SnmpV3TrapSecurityPanel' );
    Ext.apply( values, securityPanel.securityConfig2FieldValues( profile.channelConfig.security ) );

    me.getForm().setValues( values );
  },


  /**
   * Creates a new {@link MessageChannelProfile} model instance populated with
   * form data.
   */
  createMessageChannelProfile : function ()
  {
    var me = this;

    var profile = Ext.create( 'QDesk.model.message.MessageChannelProfile' );

    var fieldValues = me.getForm().getFieldValues();

    if ( !Ext.isEmpty( fieldValues.id ) )
      profile.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.name ) )
      profile.set( 'name', fieldValues.name );

    profile.set( 'channel', QDesk.model.message.channel.MessageChannel.SNMPV3_TRAP );

    var channelConfig = {};

    Ext.apply( channelConfig, {
      host : fieldValues.host,
      port : fieldValues.port,
      community : fieldValues.community,
      //trapOid : fieldValues.trapOid,
      systemDescription : fieldValues.systemDescription,
      variableMessageTrapCustomType : fieldValues.variableMessageTrapCustomType
    } );

    var securityPanel = me.down( 'SnmpV3TrapSecurityPanel' );
    var security = securityPanel.fieldValues2SecurityConfig( fieldValues );

    Ext.apply( channelConfig, {
      security : security
    } );

    profile.set( 'channelConfig', channelConfig );

    return profile;
  }
} );
