/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.navigation.connections.EditConnectionFolderForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.view.work.navigation.connections.EditConnectionFolderFormMode'
  ],

  alias : 'widget.EditConnectionFolderForm',

  defaultButton : 'actionButton',

  qdesk : {
    /*
     * Mode of the form (see EditConnectionFolderFormMode).
     */
    mode : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    Ext.apply( config, {
      items : [
        {
          xtype : 'textfield',
          itemId : 'folderName',
          name : 'name',
          value : QDesk.Msg.get( 'QDesk.unnamed' ),
          allowBlank : false,
          maxLength : 128
        }
      ],

      buttons : [
        {
          itemId : 'cancelButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },
        {
          itemId : 'actionButton',
          reference : 'actionButton',
          text : me._getActionButtonText( config.qdesk.mode ),
          formBind : true
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  _getActionButtonText : function ( mode )
  {
    var me = this;

    switch ( mode )
    {
      case QDesk.view.work.navigation.connections.EditConnectionFolderFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.work.navigation.connections.EditConnectionFolderFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }
  }
} );
