/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.navigation.connections.EditConnectionFolderWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',

    'QDesk.util.UxUtils',

    'QDesk.view.work.navigation.connections.EditConnectionFolderForm'
  ],

  alias : 'widget.EditFolderWindow',

  cls : 'editConnectionFolderWindow',
  iconCls : 'windowIcon',

  scrollable : true,

  modal : true,
  resizable : false,

  width : 300,

  qdesk : {
    /*
     * Mode of the form (see EditConnectionFolderFormMode).
     */
    mode : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    Ext.apply( config, {
      title : me._getWindowTitle( config.qdesk.mode ),
      items : [
        {
          xtype : 'EditConnectionFolderForm',
          itemId : 'form',

          qdesk : {
            mode : config.qdesk.mode
          }
        }
      ]
    } );

    me.callParent( [ config ] );

    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } )
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me.on( 'afterrender', function ()
    {
      var folderNameField = me.down( '#folderName' );

      if ( me.qdesk.mode == QDesk.view.work.navigation.connections.EditConnectionFolderFormMode.ADD )
      {
        QDesk.util.UxUtils.focus( folderNameField, true );
      }
    } );
  },


  /**
   * Returns the title of this window.
   * @param mode a QDesk.view.work.navigation.connections.EditConnectionFolderFormMode value.
   */
  _getWindowTitle : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.navigation.connections.EditConnectionFolderFormMode.ADD:
        title = QDesk.Msg.get( 'QDesk.view.work.navigation.connections.EditConnectionFolderWindow.windowTitleAdd' );
        break;

      case QDesk.view.work.navigation.connections.EditConnectionFolderFormMode.EDIT:
        title = QDesk.Msg.get( 'QDesk.view.work.navigation.connections.EditConnectionFolderWindow.windowTitleEdit' );
        break;

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }

    return title;
  }
} )
;