/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm', {
  extend : 'QDesk.view.base.BaseConfigFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.connection.DiscoverSchedulers',
    'QDesk.controller.command.connection.TestSchedulerConnectionData',

    'QDesk.model.jmx.JmxProtocol',
    'QDesk.model.scheduler.Scheduler',

    'QDesk.util.EnumUtils'
  ],

  alias : 'widget.SchedulerConnectionDataForm',

  qdesk : {
    /*
     * Mode of the form (see SchedulerConnectionDataFormMode).
     */
    mode : undefined,

    /*
     * Scheduler type (see QDesk.model.scheduler.SchedulerType).
     */
    schedulerType : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerType' );

    // initially empty store used by the schedulerObjectName combo
    var schedulerObjectNameStore = Ext.create( 'Ext.data.ArrayStore', {
      model : 'QDesk.model.scheduler.Scheduler'
    } );

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'fieldcontainer',
          layout : 'anchor',
          defaults : {
            anchor : '100%'
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldLabelName' ),
              labelWidth : 60,
              allowBlank : false,
              maxLength : 128,
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldTooltipName',
                  QDesk.util.EnumUtils.getSchedulerTypeName( config.qdesk.schedulerType ) ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldEmptyTxtName' ),
              value : QDesk.Msg.get( 'QDesk.unnamed' )
            },
            {
              xtype : 'fieldset',
              title : QDesk.Msg.get(
                  'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldsetLabelRemoteJmxService' ),
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'radiofield',
                  itemId : 'serviceParamsRadio',
                  boxLabel : QDesk.Msg.get(
                      'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.boxLabelServiceParams' )
                },
                {
                  xtype : 'fieldcontainer',
                  itemId : 'serviceParamsFields',
                  padding : '0 0 0 10',
                  layout : 'anchor',
                  defaults : {
                    anchor : '100%',
                    labelWidth : 70
                  },
                  items : [
                    {
                      xtype : 'combo',
                      itemId : 'protocol',
                      name : 'protocol',
                      fieldLabel : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldLabelProtocol' ),
                      fieldTooltip : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldTooltipProtocol',
                          QDesk.util.EnumUtils.getSchedulerTypeName( config.qdesk.schedulerType ) ),
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldEmptyTxtProtocol' ),
                      //value : QDesk.model.jmx.JmxProtocol.RMI,
                      allowBlank : false,
                      editable : false,
                      forceSelection : true,
                      queryMode : 'local',
                      store : 'enums.JmxProtocolStore',
                      valueField : 'value',
                      displayField : 'valueLabel'
                    },
                    {
                      xtype : 'textfield',
                      itemId : 'host',
                      name : 'host',
                      fieldLabel : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldLabelHost' ),
                      allowBlank : false,
                      maxLength : 128,
                      fieldTooltip : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldTooltipHost',
                          QDesk.util.EnumUtils.getSchedulerTypeName( config.qdesk.schedulerType ) ),
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldEmptyTxtHost' )
                    },
                    {
                      xtype : 'numberfield',
                      itemId : 'port',
                      name : 'port',
                      fieldLabel : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldLabelPort' ),
                      allowBlank : false,
                      minValue : 1,
                      maxValue : 65535,
                      fieldTooltip : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldTooltipPort',
                          QDesk.util.EnumUtils.getSchedulerTypeName( config.qdesk.schedulerType ) ),
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldEmptyTxtPort' )
                    }
                  ]
                },


                {
                  xtype : 'radiofield',
                  itemId : 'serviceUrlRadio',
                  boxLabel : QDesk.Msg.get(
                      'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.boxLabelServiceUrl' )
                },
                {
                  xtype : 'fieldcontainer',
                  itemId : 'serviceUrlFields',
                  padding : '0 0 0 10',
                  layout : 'anchor',
                  defaults : {
                    anchor : '100%',
                    labelWidth : 70
                  },
                  items : [

                    {
                      xtype : 'textfield',
                      itemId : 'serviceUrl',
                      name : 'serviceUrl',
                      fieldLabel : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldLabelServiceUrl' ),
                      fieldTooltip : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldTooltipServiceUrl',
                          QDesk.util.EnumUtils.getSchedulerTypeName( config.qdesk.schedulerType ) ),
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldEmptyTxtServiceUrl' ),
                      allowBlank : false,
                      maxLength : 128
                    }
                  ]
                },


                {
                  xtype : 'fieldset',
                  title : QDesk.Msg.get(
                      'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldsetLabelAuthentication' ),
                  tooltip : QDesk.Msg.get(
                      'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldsetTooltipAuthentication' ),
                  layout : 'anchor',
                  defaults : {
                    anchor : '100%',
                    labelWidth : 70
                  },
                  items : [
                    {
                      xtype : 'textfield',
                      itemId : 'username',
                      name : 'username',
                      fieldLabel : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldLabelUsername' ),
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldEmptyTxtUsername' ),
                      allowBlank : true,
                      maxLength : 128
                    },
                    {
                      xtype : 'textfield',
                      itemId : 'password',
                      name : 'password',
                      fieldLabel : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldLabelPassword' ),
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldEmptyTxtPassword' ),
                      inputType : 'password',
                      allowBlank : true,
                      maxLength : 128
                    }
                  ]
                },

                {
                  xtype : 'checkboxfield',
                  itemId : 'secure',
                  name : 'secure',
                  //boxLabel : QDesk.Msg.get(
                  //   'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.boxLabelSecure_RMI' ),
                  //boxTooltip : QDesk.Msg.get(
                  //   'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.boxTooltipSecure_RMI' ),
                  labelAlign : 'right',
                  allowBlank : true
                }
              ]
            },
            {
              xtype : 'fieldset',
              title : QDesk.Msg.get(
                  'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldsetTitleSchedulerObjectName' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldsetTooltipSchedulerObjectName',
                  QDesk.util.EnumUtils.getSchedulerTypeName( config.qdesk.schedulerType ) ),
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'panel',
                  border : false,
                  layout : {
                    type : 'hbox'
                  },
                  style : {
                    marginBottom : '5px'
                  },
                  items : [
                    {
                      xtype : 'combo',
                      itemId : 'schedulerObjectName',
                      name : 'schedulerObjectName',
                      allowBlank : false,
                      maxLength : 128,
                      queryMode : 'local',
                      store : schedulerObjectNameStore,
                      valueField : 'objectName',
                      displayField : 'objectName',
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.fieldEmptyTxtSchedulerObjectName' ),
                      flex : 1
                    },
                    {
                      xtype : 'button',
                      itemId : 'discoverButton',
                      margin : '0 0 0 5',
                      text : QDesk.Msg.get(
                          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.buttonLabelDiscover' )
                    }
                  ]
                }
              ]
            },

            {
              xtype : 'panel',
              border : false,
              layout : {
                type : 'hbox',
                pack : 'end'
              },
              items : [
                {
                  xtype : 'button',
                  itemId : 'testButton',
                  text : QDesk.Msg.get(
                      'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.buttonLabelTest' ),
                  formBind : true,
                  iconCls : 'testConnection'
                }
              ]
            }
          ],

          /* All fields in the ConnectionData form are available in all product editions. */
          qdesk : {
            licensedFor : [
              QDesk.model.license.ProductEdition.LITE,
              QDesk.model.license.ProductEdition.STANDARD,
              QDesk.model.license.ProductEdition.ENTERPRISE
            ]
          }
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    var protocolCombo = me.down( '#protocol' );
    var secureCheckbox = me.down( '#secure' );

    me.on( 'beforerender', function ()
    {
      var serviceUrl = me.getForm().findField( 'serviceUrl' ).getValue();

      if ( Ext.isEmpty( serviceUrl ) )
      {
        me.down( '#serviceParamsRadio' ).setValue( true );
        me.down( '#serviceParamsFields' ).setDisabled( false );

        me.down( '#serviceUrlRadio' ).setValue( false );
        me.down( '#serviceUrlFields' ).setDisabled( true );
      }
      else
      {
        me.down( '#serviceParamsRadio' ).setValue( false );
        me.down( '#serviceParamsFields' ).setDisabled( true );

        me.down( '#serviceUrlRadio' ).setValue( true );
        me.down( '#serviceUrlFields' ).setDisabled( false );
      }

      // if REMOTING-JMX protocol is selected, check and disable the Secure checkbox
      if ( protocolCombo.getValue() == QDesk.model.jmx.JmxProtocol.REMOTING_JMX )
      {
        secureCheckbox.setValue( true );
        secureCheckbox.setDisabled( true );
      }
    } );

    // Discover button
    me.down( '#discoverButton' ).on( 'click', function ()
    {
      var me = this;
      Ext.create( 'QDesk.controller.command.connection.DiscoverSchedulers', {
        form : me,
        schedulerType : me.qdesk.schedulerType
      } ).execute();
    }, me );

    // Test Connection button
    me.down( '#testButton' ).on( 'click', function ()
    {
      var me = this;
      Ext.create( 'QDesk.controller.command.connection.TestSchedulerConnectionData', {
        form : me,
        schedulerType : me.qdesk.schedulerType
      } ).execute();
    }, me );

    me.down( '#serviceParamsRadio' ).on( 'change', me._serviceModeChanged, me );
    me.down( '#serviceUrlRadio' ).on( 'change', me._serviceModeChanged, me );

    protocolCombo.on( 'change', me._protocolChanged, me );
    me.down( '#host' ).on( 'change', me._maybeEnableButtons, me );
    me.down( '#port' ).on( 'change', me._maybeEnableButtons, me );

    me.down( '#serviceUrl' ).on( 'change', me._maybeEnableButtons, me );

    me.down( '#username' ).on( 'change', me._maybeEnableButtons, me );
    me.down( '#password' ).on( 'change', me._maybeEnableButtons, me );

    me.down( '#schedulerObjectName' ).on( 'change', me._maybeEnableButtons, me );


    me.down( '#secure' ).on( 'change', me._secureChanged, me );

    // set default protocol value (change of the value invokes the _protocolChanged method)
    protocolCombo.setValue( QDesk.model.jmx.JmxProtocol.RMI );

    // set default value of the secure checkbox
    secureCheckbox.setValue( false );
  },


  /**
   * Change the box label and tool tip of the Secure checkbox.
   * @param protocolCombo the protocol selection select box.
   * @param newValue the newly selected protocol value.
   * @param oldValue the previously selected protocol value.
   * @private
   */
  _protocolChanged : function ( protocolCombo, newValue, oldValue )
  {
    var me = this;
    var secureCheckbox = me.down( '#secure' );

    secureCheckbox.setBoxLabel( QDesk.Msg.get(
        'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.boxLabelSecure_' + newValue ) );

    if ( newValue == QDesk.model.jmx.JmxProtocol.REMOTING_JMX )
    {
      secureCheckbox.setValue( true );
      secureCheckbox.setDisabled( true );
    }
    else
    {
      secureCheckbox.setValue( false );
      secureCheckbox.setDisabled( false );
    }

    // TODO: change boxTooltip

    me._maybeUpdateAuthenticationFields();
    me._maybeEnableButtons();
  },


  _serviceModeChanged : function ( radio, newValue, oldValue )
  {
    var me = this;

    var secureCheckbox = me.down( '#secure' );

    /* Service Parameters */
    if ( radio.getItemId() == 'serviceParamsRadio' && newValue )
    {
      me.down( '#serviceUrlRadio' ).setValue( false );
      me.down( '#serviceUrlFields' ).setDisabled( true );

      me.down( '#serviceParamsFields' ).setDisabled( false );

      var protocolValue = me.down( "#protocol" ).getValue();

      secureCheckbox.setBoxLabel( QDesk.Msg.get(
          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.boxLabelSecure_' + protocolValue ) );
    }

    /* Service URL */
    if ( radio.getItemId() == 'serviceUrlRadio' && newValue )
    {
      secureCheckbox.setBoxLabel( QDesk.Msg.get(
          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.boxLabelSecure' ) );

      me.down( '#serviceParamsRadio' ).setValue( false );
      me.down( '#serviceParamsFields' ).setDisabled( true );

      me.down( '#serviceUrlFields' ).setDisabled( false );

      secureCheckbox.setBoxLabel( QDesk.Msg.get(
          'QDesk.view.work.navigation.connections.SchedulerConnectionDataForm.boxLabelSecure' ) );
    }

    me._maybeUpdateAuthenticationFields();
    me._maybeEnableButtons();
  },


  _secureChanged : function ( checkbox, newValue, oldValue )
  {
    var me = this;

    me._maybeUpdateAuthenticationFields();
    me._maybeEnableButtons();
  },


  _maybeUpdateAuthenticationFields : function ()
  {
    var me = this;

    var serviceParamsRadio = me.down( '#serviceParamsRadio' );
    var protocolCombo = me.down( '#protocol' );

    var usernameField = me.down( '#username' );
    var passwordField = me.down( '#password' );

    var secureCheckbox = me.down( '#secure' );

    // username and password are optional by default
    usernameField.allowBlank = true;
    passwordField.allowBlank = true;

    // Service Parameters
    if ( serviceParamsRadio.getValue() )
    {
      if ( secureCheckbox.getValue() )
      {
        if ( protocolCombo.getValue() == QDesk.model.jmx.JmxProtocol.JMXMP )
        {
          // username and password are required
          usernameField.allowBlank = false;
          passwordField.allowBlank = false;
        }
      }
    }

    usernameField.clearInvalid();
    usernameField.validate();

    passwordField.clearInvalid();
    passwordField.validate();
  },


  _maybeEnableButtons : function ()
  {
    var me = this;

    var discoverButton = me.down( '#discoverButton' );
    var testButton = me.down( '#testButton' );

    var protocolCombo = me.down( '#protocol' );
    var hostField = me.down( '#host' );
    var portField = me.down( '#port' );

    var serviceUrlField = me.down( '#serviceUrl' );

    var usernameField = me.down( '#username' );
    var passwordField = me.down( '#password' );

    var schedulerObjectNameField = me.down( '#schedulerObjectName' );

    var serviceParamsRadio = me.down( '#serviceParamsRadio' );
    var serviceUrlRadio = me.down( '#serviceUrlRadio' );

    if ( serviceParamsRadio.getValue() )
    {
      discoverButton.setDisabled(
          !( hostField.isValid() && portField.isValid() && usernameField.isValid() && passwordField.isValid() ) );

      testButton.setDisabled(
          !( hostField.isValid() && portField.isValid() && usernameField.isValid() && passwordField.isValid() &&
              schedulerObjectNameField.isValid() ) );
    }

    if ( serviceUrlRadio.getValue() )
    {
      discoverButton.setDisabled( !serviceUrlField.isValid() && usernameField.isValid() && passwordField.isValid() );

      testButton.setDisabled( !( serviceUrlField.isValid() && usernameField.isValid() && passwordField.isValid() &&
          schedulerObjectNameField.isValid() ) );
    }
  }
} );
