/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.navigation.connections.SchedulerConnectionSettingsForm', {
  extend : 'QDesk.view.base.BaseConfigFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.model.common.TimeZone',
    'QDesk.model.license.ProductEdition',
    'QDesk.model.scheduler.Scheduler',
    'QDesk.util.license.LicensedProduct',
    'QDesk.view.base.BaseConfigFormPanel'
  ],

  alias : 'widget.SchedulerConnectionSettingsForm',

  qdesk : {
    /*
     * Mode of the form (see SchedulerConnectionDataFormMode).
     */
    mode : undefined,

    /*
     * Scheduler type (see QDesk.model.scheduler.SchedulerType).
     */
    schedulerType : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerType' );

//    var licensedProductEdition = QDesk.util.license.LicensedProduct.productEdition;
//
//    /* This form panel is licensed for the Standard, or Enterprise edition only. */
//    Ext.apply( config.qdesk, {
//      licensed : licensedProductEdition == QDesk.model.license.ProductEdition.STANDARD ||
//          licensedProductEdition == QDesk.model.license.ProductEdition.ENTERPRISE
//    } );

    var timeZoneStore = Ext.create( 'Ext.data.ArrayStore', {
      model : 'QDesk.model.common.TimeZone'
    } );
    timeZoneStore.loadData( QDesk.System.getAvailableTimeZones() );

    Ext.apply( config, {
      items : [

        {
          xtype : 'fieldset',
          fieldDefaults : {
            labelWidth : 110,
            width : 440
          },
          title : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.fieldsetLabelDateAndTime' ),
          items : [
            {
              xtype : 'combo',
              name : 'timeZone',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.config.general.GeneralPanel.fieldLabelTimeZone' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.config.general.GeneralPanel.fieldTooltipTimeZone' ),
              displayField : 'displayName',
              valueField : 'id',
              allowBlank : false,
              editable : false,
              forceSelection : true,
              queryMode : 'local',
              store : timeZoneStore,
              qdesk : {
                licensedFor : [
                  QDesk.model.license.ProductEdition.LITE,
                  QDesk.model.license.ProductEdition.STANDARD,
                  QDesk.model.license.ProductEdition.ENTERPRISE
                ]
              }
            },
            {
              xtype : 'WDateFormatCombo',
              name : 'dateFormat',
              labelWidth : 110,
              formatConfig : {
                allowBlank : false,
                fieldLabel : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.fieldLabelDateFormat' ),
                fieldTooltip : QDesk.Msg.get(
                    'QDesk.view.work.config.general.GeneralPanel.fieldTooltipDateFormat' ),
                qdesk : {
                  licensedFor : [
                    QDesk.model.license.ProductEdition.LITE,
                    QDesk.model.license.ProductEdition.STANDARD,
                    QDesk.model.license.ProductEdition.ENTERPRISE
                  ]
                }
              }
            },
            {
              xtype : 'WTimestampFormatCombo',
              name : 'timestampFormat',
              labelWidth : 110,
              formatConfig : {
                allowBlank : false,
                fieldLabel : QDesk.Msg.get( 'QDesk.view.work.config.general.GeneralPanel.fieldLabelTimestampFormat' ),
                fieldTooltip : QDesk.Msg.get(
                    'QDesk.view.work.config.general.GeneralPanel.fieldTooltipTimestampFormat' ),
                qdesk : {
                  licensedFor : [
                    QDesk.model.license.ProductEdition.LITE,
                    QDesk.model.license.ProductEdition.STANDARD,
                    QDesk.model.license.ProductEdition.ENTERPRISE
                  ]
                }
              }
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  setValues : function ( data )
  {
    var me = this;

    me.callParent( [ data ] );

    var combo = me.down( 'combo[name=timeZone]' );
    combo.setValue( data.timeZone.id );
  }
} );
