/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.common.ConnectionsOperationResultsWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.view.work.scheduler.common.ConnectionsOperationResultsGrid'
  ],

  alias : 'widget.ConnectionsOperationResultsWindow',

  cls : 'connectionsOperationResultsWindow',
  //iconCls : 'connectionsOperationResultsWindowIcon',

  modal : true,
  resizable : false,

  plain : true,

  width : 600,

  qdesk : {
    /*
     * A dialog type (see QDesk.model.gui.DialogType).
     */
    dialogType : null,

    /*
     * A message text.
     */
    message : null,

    /*
     * An array of data objects with QDesk.model.connection.ConnectionOperationResult data.
     */
    results : null,

    /**
     * An optional function to invoke when the window is closed.
     */
    callback : null
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.dialogType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.dialogType' );

    if ( !config.qdesk.message )
      Ext.Error.raise( 'Required config parameter missing: qdesk.message' );

    if ( !config.qdesk.results )
      Ext.Error.raise( 'Required config parameter missing: qdesk.results' );

    Ext.apply( config, {
      title : me._getWindowTitle( config.qdesk.dialogType ),
      items : [
        {
          xtype : 'panel',
          border : 0,
          bodyStyle : {
            background : '#ced9e7'
          },
          layout : {
            type : 'vbox',
            align : 'stretch',
            pack : 'start'
          },
          items : [
            // top panel with the icon and message text
            {
              xtype : 'panel',
              border : 0,
              bodyStyle : {
                background : '#ced9e7'
              },
              layout : {
                type : 'hbox',
                align : 'stretch',
                pack : 'start'
              },
              padding : 10,
              items : [
                {
                  xtype : 'component',
                  width : 32,
                  height : 32,
                  cls : me._getIconCls( config.qdesk.dialogType )
                },
                {
                  xtype : 'component',
                  //margin : '7 0 0 0',
                  padding : '0 0 0 10',
                  html : config.qdesk.message
                }
              ]
            },

            {
              xtype : 'ConnectionsOperationResultsGrid',
              itemId : 'resultsGrid',
              bodyPadding : 0,
              border : 1,
              maxHeight : 150,
              qdesk : {
                data : config.qdesk.results
              }
            }
          ]
        }
      ],

      dockedItems : [ {
        xtype : 'toolbar',
        dock : 'bottom',
        ui : 'footer',
        items : [
          { xtype : 'component', flex : 1 },
          { itemId : 'okButton', text : 'OK', minWidth : 75 } ]
      } ]
    } );

    me.callParent( [ config ] );

    me.down( '#okButton' ).on( 'click', function ()
    {
      me.close();
    } );

    // invoke the callback function upon window close
    me.on( 'close', function ()
    {
      if ( Ext.isFunction( me.qdesk.callback ) )
      {
        me.qdesk.callback.apply( this );
      }
    } )
  },


  _getWindowTitle : function ( dialogType )
  {
    switch ( dialogType )
    {
      case QDesk.model.gui.DialogType.INFO:
        return QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleInfo' );

      case QDesk.model.gui.DialogType.WARNING:
        return QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleWarning' );

      case QDesk.model.gui.DialogType.ERROR:
        return QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleError' );

      default:
        throw 'Unrecognized dialogType value: ' + dialogType;
    }
  },


  _getIconCls : function ( dialogType )
  {
    switch ( dialogType )
    {
      case QDesk.model.gui.DialogType.INFO:
        return Ext.MessageBox.INFO;

      case QDesk.model.gui.DialogType.WARNING:
        return Ext.MessageBox.WARNING;

      case QDesk.model.gui.DialogType.ERROR:
        return Ext.MessageBox.ERROR;

      default:
        throw 'Unrecognized dialogType value: ' + dialogType;
    }
  }
} );
