/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.ViewNextFireTimesGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.store.scheduler.quartz.QuartzNextFireTimesStore'
  ],

  alias : 'widget.QuartzViewNextFireTimesGrid',

  cls : 'quartzViewNextFireTimesGrid',

  qdesk : {
    /**
     * Required {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzNextFireTimesStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.ViewNextFireTimesGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'combo',
          itemId : 'maxResultsSelect',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.ViewNextFireTimesGrid.fieldLabelMaxResultsSelect' ),
          store : new Ext.data.ArrayStore( {
            fields : [
              'value',
              'text'
            ],
            data : [
              [ 10, '10' ],
              [ 20, '20' ],
              [ 50, '50' ],
              [ 100, '100' ],
              [ 500, '500' ],
              [ 1000, '1000' ]
              //          [-1 , QDesk.Msg.get( 'QDesk.unlimitedForCombo' )]  - disabled, can take a long time to load
            ]
          } ),
          displayField : 'text',
          valueField : 'value',
          queryMode : 'local',
          labelAlign : 'right',
          labelWidth : 70,
          width : 140,
          editable : false,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.ViewNextFireTimesGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ],

      columns : [
        {
          itemId : 'nextFireTimeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.ViewNextFireTimesGrid.textNextFireTimeColumn' ),
          dataIndex : 'targetValue',
          sortable : false,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var targetFmtValue = record.get( 'targetFmtValue' );
            return targetFmtValue == null ? '' : Ext.String.htmlEncode( targetFmtValue );
          }
        }
      ],

      store : store,

      qdesk : {
        connectionCtx : config.qdesk.connectionCtx,
        gridId : 'quartz.viewNextFireTimesGrid'
      }
    } );

    me.callParent( [ config ] );

    var maxResults = QDesk.ApplicationConfig.get( me.qdesk.gridId + '.maxResults' );
    me.down( '#maxResultsSelect' ).setValue( maxResults );
  },


  /**
   * Overridden function that returns additional config values to be saved
   * with other grid config values.
   */
  getExtraConfig : function ()
  {
    var me = this;

    var maxResults = me.down( '#maxResultsSelect' ).getValue();

    var extra = {};
    extra[ me.qdesk.gridId + '.maxResults' ] = maxResults;

    return extra;
  }
} );
