/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.audit.SchedulerAuditTrailTab', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerXPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.widget.message.WarningMessagePanel',
    'QDesk.view.work.audit.AuditTrailGrid'
  ],

  alias : 'widget.QuartzSchedulerAuditTrailTab',

  iconCls : 'quartzSchedulerAuditTrailSubTabIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  qdRequiresAnyPermissions : [ QDesk.model.security.Permission.AUDIT_TRAIL_VIEW ],
  qdNotAuthorizedIconCls : 'auditTrailTabIconNAUTH',


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    var connectionCtx = config.qdesk.connectionCtx;
    // var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    //var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    // var jvmAgentReady = jvmAgentInfo.isReady();
    // var publicApiReady = schedulerInfo.isPublicApiReady();

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.General;

    if ( licensedFeatures.AuditTrail )
    {
      var items = [];

      items.push(
          {
            xtype : 'AuditTrailGrid',
            itemId : 'auditTrailGrid',
            flex : 1,
            qdesk : {
              connectionCtx : connectionCtx,
              gridId : 'quartz.schedulerAuditTrailGrid'
            }
          }
      );

      /*
       * Intentionally over-nested a bit so that we use identical layout to other tabs that
       * have a license restrictions related message panel above the grid.
       */
      Ext.apply( config, {
        items : [
          {
            xtype : 'panel',
            border : false,
            layout : 'fit',
            flex : 1,
            items : [
              {
                xtype : 'panel',
                border : false,
                layout : {
                  type : 'vbox',
                  align : 'stretch'
                },
                items : items
              }
            ]
          }
        ]
      } );
    }
    else
    {
      Ext.apply( config, {
        items : [
          {
            xtype : 'WarningMessagePanel',
            style : {
              margin : '5px'
            },
            qdesk : {
              msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
                  QDesk.util.license.LicensedProduct.productName )
            }
          }
        ]
      } );
    }

    Ext.apply( config, {
      title : me._getTitle(),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.audit.SchedulerAuditTrailTab.tooltip' )
    } );

    me.callParent( [ config ] );
  },


  _getTitle : function ()
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.General;

    var title = QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.audit.SchedulerAuditTrailTab.title' );
    if ( !licensedFeatures.AuditTrail )
    {
      title = '<span class="notLicensed">' + title + '</span>'
    }

    return title;
  }
} );