/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid', {
  extend : 'Ext.grid.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.scheduler.JobChainTargetType',
    'QDesk.model.scheduler.JobChainTargetExecType',
    'QDesk.model.scheduler.quartz.QuartzJobChainTarget',

    'QDesk.view.widget.WReadOnlyCheckColumn',
    'QDesk.view.work.scheduler.common.chain.EditJobChainTargetFormMode',
    'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetWindow',

    'QDesk.util.UxUtils',
    'QDesk.util.quartz.QuartzJobChainUtils'
  ],

  alias : 'widget.QuartzJobChainTargetsGrid',

  viewConfig : {
    stripeRows : true
  },

  scrollable : true,

  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.getXY() );
    }
  },

  qdesk : {
    /**
     * Required {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    // create store
    var store = Ext.create( 'Ext.data.Store', {
      model : 'QDesk.model.scheduler.quartz.QuartzJobChainTarget',
      data : [],
      sorters : [
        {
          property : 'connectionName',
          direction : 'ASC'
        },
        {
          property : 'jobName',
          direction : 'ASC'
        }
      ]
    } );

    Ext.apply( config, {
      store : store
    } );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'addJobChainTargetButton',
          scale : 'small',
          iconCls : 'addJobChainTarget',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.tooltipAddJobChainTargetButton' ),
          disabled : false
        },
        {
          xtype : 'button',
          itemId : 'deleteJobChainTargetButton',
          scale : 'small',
          iconCls : 'deleteJobChainTarget',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.tooltipDeleteJobChainTargetButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'editJobChainTargetButton',
          scale : 'small',
          iconCls : 'editJobChainTarget',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.tooltipEditJobChainTargetButton' ),
          disabled : true
        }
      ],

      columns : [
        {
          itemId : 'schedulerColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.textSchedulerColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.tooltipSchedulerColumn' ),
          dataIndex : 'type',
          sortable : true,
          groupable : false,
          /* Need custom renderer to localize the enum value */
          renderer : function ( value, metaData, record )
          {
            return QDesk.util.quartz.QuartzJobChainUtils.getTargetSchedulerDescription( value,
                record.get( 'remoteConnectionName' ) );
          },
          flex : 1
        },
        {
          itemId : 'jobNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.textJobNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.tooltipJobNameColumn' ),
          dataIndex : 'jobName',
          sortable : true,
          groupable : false,
          flex : 0.5,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'jobGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.textJobGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.tooltipJobGroupNameColumn' ),
          dataIndex : 'jobGroupName',
          sortable : true,
          groupable : true,
          flex : 0.5,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'enabledColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.textEnabledColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.tooltipEnabledColumn' ),
          align : 'center',
          dataIndex : 'enabled',
          sortable : false,
          groupable : false,
          width : 70
        },

        {
          itemId : 'execTypeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.textExecTypeColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.tooltipExecTypeColumn' ),
          dataIndex : 'execType',
          sortable : false,
          groupable : false,
          /* Need custom renderer to localize the enum value */
          renderer : function ( value, metaData, record )
          {
            return QDesk.util.quartz.QuartzJobChainUtils.getTargetExecTypeDescription( value,
                record.get( 'execTypeDelay' ), record.get( 'execTypeDateExpression' ) );
          },
          flex : 0.7
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    me.down( '#addJobChainTargetButton' ).on( 'click', function ()
    {
      me.fireEvent( 'addJobChainTarget' );
    } );

    me.down( '#deleteJobChainTargetButton' ).on( 'click', function ()
    {
      var selections = me.getSelectionModel().getSelection();
      me.fireEvent( 'deleteJobChainTarget', selections[ 0 ] );
    } );

    me.down( '#editJobChainTargetButton' ).on( 'click', function ()
    {
      var selections = me.getSelectionModel().getSelection();
      me.fireEvent( 'editJobChainTarget', selections[ 0 ] );
    } );

    me.getSelectionModel().on( 'selectionchange', function ( selModel, selections )
    {
      me.down( '#deleteJobChainTargetButton' ).setDisabled( selections.length === 0 );
      me.down( '#editJobChainTargetButton' ).setDisabled( selections.length === 0 );
    } );


    var jobChainTargetsStore = me.getStore();


    me.on( 'addJobChainTarget', function ()
    {
      var window = Ext.widget( 'EditJobChainTargetWindow', {
        qdesk : {
          connectionCtx : me.qdesk.connectionCtx,
          mode : QDesk.view.work.scheduler.common.chain.EditJobChainTargetFormMode.ADD
        }
      } );

      window.down( 'form' ).on( 'jobChainTargetAdded', function ( jobChainTarget )
      {
        jobChainTargetsStore.add( jobChainTarget );
        window.close();
      } );

      var parentWindow = me.up( 'window' );
      if ( parentWindow )
      {
        QDesk.util.UxUtils.maskEl( parentWindow.getEl(), QDesk.Msg.get( 'QDesk.waitMsg' ) );

        window.on( 'afterrender', function ()
        {
          QDesk.util.UxUtils.unmaskEl( parentWindow.getEl() );
        } );
      }

      // defer showing the window so that the mask has a chance to display
      Ext.defer( function ()
      {
        window.show();
      }, 100 );
    } );


    me.on( 'editJobChainTarget', function ( jobChainTarget )
    {
      var window = Ext.widget( 'EditJobChainTargetWindow', {
        qdesk : {
          connectionCtx : me.qdesk.connectionCtx,
          target : jobChainTarget,
          mode : QDesk.view.work.scheduler.common.chain.EditJobChainTargetFormMode.EDIT
        }
      } );

      window.down( 'form' ).on( 'jobChainTargetUpdated', function ( newJobChainTarget, oldJobChainTarget )
      {
        jobChainTargetsStore.remove( oldJobChainTarget );
        jobChainTargetsStore.add( newJobChainTarget );
        window.close();
      } );

      var parentWindow = me.up( 'window' );
      if ( parentWindow )
      {
        QDesk.util.UxUtils.maskEl( parentWindow.getEl(), QDesk.Msg.get( 'QDesk.waitMsg' ) );

        window.on( 'afterrender', function ()
        {
          QDesk.util.UxUtils.unmaskEl( parentWindow.getEl() );
        } );
      }

      // defer showing the window so that the mask has a chance to display
      Ext.defer( function ()
      {
        window.show();
      }, 100 );
    } );

    me.on( 'deleteJobChainTarget', function ( jobChainTarget )
    {
      jobChainTargetsStore.remove( jobChainTarget );
    } );
  },


  /**
   * Invoked to create a context menu for the specified job in the JobChainTargetsGrid row.
   * @param jobChainTarget a job chain target.
   * @private
   */
  _createRowCtxMenu : function ( jobChainTarget )
  {
    var me = this;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // delete job chain target
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.targetCtxMenuDelete' ),
          iconCls : 'deleteJobChainTarget',
          handler : function ()
          {
            me.fireEvent( 'deleteJobChainTarget', jobChainTarget );
          }
        },
        // edit job chain target
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid.targetCtxMenuEdit' ),
          iconCls : 'editJobChainTarget',
          handler : function ()
          {
            me.fireEvent( 'editJobChainTarget', jobChainTarget );
          }
        }
      ]
    } );
  }
} );
