/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.chain.JobChainsTab', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerXPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.view.work.scheduler.quartz.chain.SchedulerJobChainsView',
    'QDesk.view.work.scheduler.quartz.chain.JobJobChainsView',
    'QDesk.view.work.scheduler.quartz.chain.TriggerJobChainsView',

    'QDesk.view.widget.tab.XTabPanel'
  ],

  alias : 'widget.QuartzJobChainsTab',

  iconCls : 'quartzJobChainsTabIcon',

  // border : false,
  /* The top tab should not be scrollable, but nested panels (views) should be! */
  scrollable : false,

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    var connectionCtx = config.qdesk.connectionCtx;
    // var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    // var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    // var jvmAgentReady = jvmAgentInfo.isReady();
    // var publicApiReady = schedulerInfo.isPublicApiReady();

    var items = [];

    items.push(
        {
          xtype : 'QuartzSchedulerJobChainsView',
          itemId : 'schedulerJobChainsView',
          qdesk : {
            connectionCtx : connectionCtx
          }
        } );

    items.push(
        {
          xtype : 'QuartzJobJobChainsView',
          itemId : 'jobJobChainsView',
          qdesk : {
            connectionCtx : connectionCtx
          }
        } );

    items.push(
        {
          xtype : 'QuartzTriggerJobChainsView',
          itemId : 'triggerJobChainsView',
          qdesk : {
            connectionCtx : connectionCtx
          }
        } );

    Ext.apply( config, {
      items : [
        {
          xtype : 'XTabPanel',
          border : false,
          flex : 1,  // for the parent vbox/stretch layout
          items : items
        }
      ]
    } );

    Ext.apply( config, {
      title : me._getTitle(),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.JobChainsTab.tooltip' )
    } );

    me.callParent( [ config ] );
  },


  _getTitle : function ()
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var title = QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.JobChainsTab.title' );
    if ( !licensedFeatures.JobChaining )
    {
      title = '<span class="notLicensed">' + title + '</span>'
    }

    return title;
  },


  /**
   * Returns the currently active view (QuartzSchedulerJobChainsView, QuartzJobJobChainsView,
   * QuartzTriggerJobChainsView).
   * @returns the currently active view.
   */
  getActiveView : function ()
  {
    var me = this;

    var tabPanel = me.down( 'XTabPanel' );
    return tabPanel.getActiveTab();
  }
} );
