Ext.define( 'QDesk.view.work.scheduler.quartz.chain.SchedulerJobChainsViewController', {
  extend : 'Ext.app.ViewController',

  requires : [
    'QDesk.controller.command.scheduler.quartz.chain.AddJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.CloneJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.DisableJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.EditJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.EnableJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid',
    'QDesk.controller.command.scheduler.quartz.chain.ViewJobChainAuditTrail',

    'QDesk.util.license.LicensedProductFeatures'
  ],

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    me.callParent( [ config ] );
  },

  init : function ()
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    // SchedulerJobChainsView
    var view = me.getView();

    view.on( 'activate',
        function ()
        {
          if ( !view.isInitialized() )
          {
            me._bindEventHandlers( view );
            view.setInitialized( true );

            Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid', {
              connectionCtx : connectionCtx
            } ).execute();
          }
        } );

    me.callParent();
  },


  /**
   * Binds events of the specified view to their respective handlers.
   */
  _bindEventHandlers : function ( view )
  {
    var me = this;
    me._bindEventHandlersJobChainsGrid( view );
  },


  /**
   * Binds events of the embedded JobChainsGrid to their respective handlers.
   */
  _bindEventHandlersJobChainsGrid : function ( view )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var connectionCtx = me.qdesk.connectionCtx;
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    var jvmAgentReady = jvmAgentInfo.isReady();
    // var publicApiReady = schedulerInfo.isPublicApiReady();

    if ( licensedFeatures.JobChaining && jvmAgentReady )
    {
      var jobChainsGrid = view.down( '#jobChainsGrid' );

      // refresh button
      jobChainsGrid.on( 'refresh',
          function ()
          {
            me._refreshJobChains();
          } );

      jobChainsGrid.on( 'addJobChain',
          function ()
          {
            me._addJobChain();
          } );

      jobChainsGrid.on( 'enableJobChain',
          function ( jobChain )
          {
            me._enableJobChain( jobChain );
          } );

      jobChainsGrid.on( 'disableJobChain',
          function ( jobChain )
          {
            me._disableJobChain( jobChain );
          } );

      jobChainsGrid.on( 'deleteJobChain',
          function ( jobChain )
          {
            me._deleteJobChain( jobChain );
          } );

      jobChainsGrid.on( 'editJobChain',
          function ( jobChain )
          {
            me._editJobChain( jobChain );
          } );

      jobChainsGrid.on( 'cloneJobChain',
          function ( jobChain )
          {
            me._cloneJobChain( jobChain );
          } );

      jobChainsGrid.on( 'viewJobChainAuditTrail',
          function ( jobChain )
          {
            me._viewJobChainAuditTrail( jobChain );
          } );
    }
  },


  _refreshJobChains : function ()
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid', {
      connectionCtx : connectionCtx
    } ).execute();
  },


  _addJobChain : function ()
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.AddJobChain', {
      connectionCtx : connectionCtx,
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid', {
          connectionCtx : connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _enableJobChain : function ( jobChain )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.EnableJobChain', {
      connectionCtx : connectionCtx,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid', {
          connectionCtx : connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _disableJobChain : function ( jobChain )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.DisableJobChain', {
      connectionCtx : connectionCtx,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid', {
          connectionCtx : connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _deleteJobChain : function ( jobChain )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain', {
      connectionCtx : connectionCtx,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid', {
          connectionCtx : connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _editJobChain : function ( jobChains )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.EditJobChain', {
      connectionCtx : connectionCtx,
      jobChainId : jobChains.get( 'id' ),
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid', {
          connectionCtx : connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _cloneJobChain : function ( jobChain )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.CloneJobChain', {
      connectionCtx : connectionCtx,
      jobChainId : jobChain.get( 'id' ),
      /*
       * Upon success refresh the job chains tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.RefreshSchedulerJobChainsGrid', {
          connectionCtx : connectionCtx
        } ).execute();
      }
    } ).execute();
  },


  _viewJobChainAuditTrail : function ( jobChain )
  {
    var me = this;

    var connectionCtx = me.qdesk.connectionCtx;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.ViewJobChainAuditTrail', {
      connectionCtx : connectionCtx,
      jobChainId : jobChain.get( 'id' )
    } ).execute();
  }
} );