/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.model.scheduler.ExecStatus',
    'QDesk.model.scheduler.ExecType',
    'QDesk.model.security.Permission',

    'QDesk.store.scheduler.quartz.QuartzExecHistoryStore',

    'QDesk.util.CommonUtils',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.DateFormat',
    'QDesk.util.quartz.QuartzJobChainUtils',

    'QDesk.view.widget.WDateTimePeriod',
    'QDesk.view.widget.WMaxTodayDateField',
    'QDesk.view.widget.WSearchField',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.QuartzExecHistoryGrid',

  cls : 'quartzExecHistoryGrid',

  border : false,
  scrollable : true,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    var connectionCtx = config.qdesk.connectionCtx;
    // var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    // var jvmAgentReady = jvmAgentInfo.isReady();
    // var publicApiReady = schedulerInfo.isPublicApiReady();

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzExecHistoryStore' );

    var fetchPeriod = QDesk.util.CommonUtils.getDefaultFetchPeriod( schedulerInfo.getSchedulerTimeZone(),
        QDesk.ApplicationConfig.get( config.qdesk.gridId + '.fetchPeriodLength' ) );

    var maxFetchSize = QDesk.ApplicationConfig.get( config.qdesk.gridId + '.maxFetchSize' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'button',
          itemId : 'exportCsvButton',
          scale : 'small',
          iconCls : 'exportCsv',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipExportCsvButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_EXEC_HISTORY_EXPORT ],
          qdNotAuthorizedIconCls : 'exportCsvNAUTH'
        },
        '-',
        {
          xtype : 'WDateTimePeriod',
          itemId : 'fetchPeriod',
          allowBlank : false,
          disabled : true,
          value : fetchPeriod,
          listeners : {
            change : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextInvalidFilters' ) );
                }
              }
            }
          }
        },
        {
          xtype : 'combo',
          itemId : 'maxFetchSizeSelect',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.fieldLabelMaxFetchSizeSelect' ),
          store : new Ext.data.ArrayStore( {
            fields : [
              'value',
              'text'
            ],
            data : [
              [ 10, '10' ],
              [ 20, '20' ],
              [ 50, '50' ],
              [ 100, '100' ],
              [ 500, '500' ],
              [ 1000, '1000' ]
              // [-1 , QDesk.Msg.get( 'QDesk.unlimitedForCombo' )]  - disabled, can take a long time to load
            ]
          } ),
          displayField : 'text',
          valueField : 'value',
          queryMode : 'local',
          labelAlign : 'right',
          labelWidth : 68,
          width : 168,
          editable : false,
          disabled : true,
          value : maxFetchSize,
          listeners : {
            change : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextInvalidFilters' ) );
                }
              }
            }
          }
        },
        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextFieldFilterQueryText' ),
          //store : execHistoryStore,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : true,
          listeners : {
            search : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextInvalidFilters' ) );
                }
              }
            },
            clear : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextInvalidFilters' ) );
                }
              }
            }
          }
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ]
    } );

    var columns = [];

    columns.push(
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textIdColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipIdColumn' ),
          dataIndex : 'id',
          sortable : true,
          groupable : false,
          flex : 0.6
        },
        {
          itemId : 'startedAtColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textStartedAtColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipStartedAtColumn' ),
          dataIndex : 'startedAt',
          sortable : true,
          groupable : false,
          flex : 1.7,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },
        {
          itemId : 'finishedAtColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textFinishedAtColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipFinishedAtColumn' ),
          dataIndex : 'finishedAt',
          sortable : true,
          groupable : false,
          flex : 1.7,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },
        {
          itemId : 'durationColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textDurationColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipDurationColumn' ),
          dataIndex : 'duration',
          renderer : function ( value )
          {
            return QDesk.util.FormatUtils.formatDuration( value );
          },
          sortable : true,
          groupable : false,
          flex : 1
        },

        {
          itemId : 'execStatusColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textExecStatusColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipExecStatusColumn' ),
          dataIndex : 'execStatus',
          sortable : true,
          groupable : false,
          flex : 0.6,
          /* Need custom renderer to localize exec status values */
          renderer : function ( value )
          {
            return QDesk.util.EnumUtils.getExecStatusName( value );
          }
        },

        {
          itemId : 'execTypeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textExecTypeColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipExecTypeColumn' ),
          dataIndex : 'execType',
          sortable : true,
          groupable : false,
          flex : 0.9,
          renderer : function ( value, metaData, record )
          {
            var text = QDesk.util.EnumUtils.getExecTypeName( value );

            var iconClass, tooltipText;


            if ( value == QDesk.model.scheduler.ExecType.NORMAL )
            {
              iconClass = 'execTypeNormal';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipExecTypeNormal' );
            }
            else if ( value == QDesk.model.scheduler.ExecType.QD_SYSTEM )
            {
              iconClass = 'execTypeQDSystem';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipExecTypeQDSystem' );
            }
            else if ( value == QDesk.model.scheduler.ExecType.QD_MANUAL )
            {
              iconClass = 'execTypeQDManual';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipExecTypeQDManual' );
            }
            else if ( value == QDesk.model.scheduler.ExecType.QD_JOB_CHAIN )
            {
              var jobDataMap = record.get( 'jobDataMap' );
              var jobChainDetails = QDesk.util.quartz.QuartzJobChainUtils.extractJobChainDetails( jobDataMap );

              var jobChainId = jobChainDetails[ 'QD_JOB_CHAIN_ID' ];
              var jobChainName = jobChainDetails[ 'QD_JOB_CHAIN_NAME' ];

              var jobChainSrcJobGroupName = jobChainDetails[ 'QD_JOB_CHAIN_SRC_JOB_GROUP_NAME' ];
              var jobChainSrcJobName = jobChainDetails[ 'QD_JOB_CHAIN_SRC_JOB_NAME' ];

              var jobChainSrcTriggerGroupName = jobChainDetails[ 'QD_JOB_CHAIN_SRC_TRIGGER_GROUP_NAME' ];
              var jobChainSrcTriggerName = jobChainDetails[ 'QD_JOB_CHAIN_SRC_TRIGGER_NAME' ];

              var jobChainSrcExecHistoryId = jobChainDetails[ 'QD_JOB_CHAIN_SRC_EXEC_HISTORY_ID' ];

              var jobChainFlowId = jobChainDetails[ 'QD_JOB_CHAIN_FLOW_ID' ];

              iconClass = 'execTypeQDJobChain';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipExecTypeQDJobChain',
                  jobChainId,
                  jobChainName,
                  jobChainSrcJobGroupName,
                  jobChainSrcJobName,
                  jobChainSrcTriggerGroupName,
                  jobChainSrcTriggerName,
                  jobChainSrcExecHistoryId,
                  jobChainFlowId
              );
            }
            else if ( value == QDesk.model.scheduler.ExecType.QD_WEB_SERVICE )
            {
              iconClass = 'execTypeQDWebService';
              tooltipText = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipExecTypeQDWebService' );
            }
            else
            {
              Ext.Error.raise( 'Unexpected exec type value: ' + value );
            }

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + Ext.String.htmlEncode( tooltipText ) + '"/>' +
                '<span style="padding-left: 5px;">' + text + '</span>';
          }
        },

        // error
        {
          xtype : 'actioncolumn',
          itemId : 'errorColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textErrorColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textErrorColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipErrorColumn' ),
          sortable : false,
          groupable : false,
          width : 70,
          items : [
            {
              icon : 'images/icon/error-16x16.png',
              tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipErrorColumn' ),
              getClass : function ( value, metaData, record )
              {
                // hide the icon in case the error is not available
                var error = record.get( 'error' );
                return error ? 'clickable' : 'x-hidden';
              },
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var error = record.get( 'error' );
                this.fireEvent( 'requestViewException', error );
              }
            }
          ]
        },

        /* Scheduler related columns */
        {
          itemId : 'schedulerNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textSchedulerNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipSchedulerNameColumn' ),
          dataIndex : 'schedulerName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'schedulerInstanceIdColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textSchedulerInstanceIdColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipSchedulerInstanceIdColumn' ),
          dataIndex : 'schedulerInstanceId',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'schedulerObjectNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textSchedulerObjectNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipSchedulerObjectNameColumn' ),
          dataIndex : 'schedulerObjectName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'schedulerVersionColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textSchedulerVersionColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipSchedulerVersionColumn' ),
          dataIndex : 'schedulerVersion',
          sortable : true,
          groupable : false,
          flex : 1.1,
          renderer : function ( value )
          {
            return value.major + '.' + value.minor + '.' + value.maintenance;
          }
        },
        /* End of scheduler related columns */

        /* Job related columns */
        {
          itemId : 'jobGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textJobGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipJobGroupNameColumn' ),
          dataIndex : 'jobGroupName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'jobNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textJobNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipJobNameColumn' ),
          dataIndex : 'jobName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textJobClassNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipJobClassNameColumn' ),
          dataIndex : 'jobClassName',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'jobClassLocationColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textJobClassLocationColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipJobClassLocationColumn' ),
          dataIndex : 'jobClassLocation',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'actioncolumn',
          itemId : 'jobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textJobDataMapColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textJobDataMapColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipJobDataMapColumnIcon' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var jobDataMap = record.get( 'jobDataMap' );
                this.fireEvent( 'requestViewJobDataMap', jobDataMap );
              }
            }
          ]
        },
        /* End of job related columns */

        /* Trigger related columns */
        {
          itemId : 'triggerGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textTriggerGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipTriggerGroupNameColumn' ),
          dataIndex : 'triggerGroupName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'triggerNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textTriggerNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipTriggerNameColumn' ),
          dataIndex : 'triggerName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },

        {
          itemId : 'calendarNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textCalendarNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipCalendarNameColumn' ),
          dataIndex : 'calendarName',
          sortable : true,
          groupable : false,
          flex : 0.5,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        /* End of trigger related columns */

        // result
        {
          xtype : 'actioncolumn',
          itemId : 'resultColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textResultColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textResultColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipResultColumn' ),
          sortable : false,
          groupable : false,
          width : 80,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/exec-history-result-16x16.png',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipResultColumnIcon' ),
              getClass : function ( value, metaData, record )
              {
                // hide the icon in case the result is not available
                var resultNull = record.get( 'resultNull' );
                return resultNull ? 'x-hidden' : 'clickable';
              },
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var execHistoryId = record.get( 'id' );
                this.fireEvent( 'requestViewResult', execHistoryId );
              }
            }
          ]
        },

        // userData
        {
          xtype : 'actioncolumn',
          itemId : 'userDataColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textUserDataColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textUserDataColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipUserDataColumn' ),
          sortable : false,
          groupable : false,
          width : 90,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/exec-history-user-data-16x16.png',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipUserDataColumn' ),
              getClass : function ( value, metaData, record )
              {
                // hide the icon in case the user data is not available
                var userDataNull = record.get( 'userDataNull' );
                return userDataNull ? 'x-hidden' : 'clickable';
              },
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var execHistoryId = record.get( 'id' );
                this.fireEvent( 'requestViewUserData', execHistoryId );
              }
            }
          ]
        },

        // log
        {
          xtype : 'actioncolumn',
          itemId : 'logColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textLogColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textLogColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipLogColumn' ),
          sortable : false,
          groupable : false,
          width : 70,
          items : [
            {
              icon : 'images/icon/scheduler/exec-history-log-16x16.png',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipLogColumnIcon' ),
              getClass : function ( value, metaData, record )
              {
                // hide the icon in case the log data is not available
                var logNull = record.get( 'logNull' );
                return logNull ? 'x-hidden' : 'clickable';
              },
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var execHistoryId = record.get( 'id' );
                this.fireEvent( 'requestViewLog', execHistoryId );
              }
            }
          ]
        },

        /* Thread related data */
        {
          itemId : 'threadGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textThreadGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipThreadGroupNameColumn' ),
          dataIndex : 'threadGroupName',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'threadNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textThreadNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipThreadNameColumn' ),
          dataIndex : 'threadName',
          sortable : true,
          groupable : false,
          flex : 2
        },
        {
          xtype : 'numbercolumn',
          itemId : 'threadPriorityColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textThreadPriorityColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipThreadPriorityColumn' ),
          dataIndex : 'threadPriority',
          sortable : true,
          groupable : false,
          flex : 1
        },

        /* Host related data */
        {
          itemId : 'hostNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textHostNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipHostNameColumn' ),
          dataIndex : 'hostName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },

        /* JVM related data */
        {
          xtype : 'numbercolumn',
          itemId : 'jvmPidColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textJvmPidColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipJvmPidColumn' ),
          dataIndex : 'jvmPid',
          sortable : true,
          groupable : false,
          flex : 0.6
        },
        {
          itemId : 'jvmNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textJvmNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipJvmNameColumn' ),
          dataIndex : 'jvmName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'jvmVendorColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textJvmVendorColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipJvmVendorColumn' ),
          dataIndex : 'jvmVendor',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'jvmVersionColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textJvmVersionColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipJvmVersionColumn' ),
          dataIndex : 'jvmVersion',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'jvmRuntimeVersionColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textJvmRuntimeVersionColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipJvmRuntimeVersionColumn' ),
          dataIndex : 'jvmRuntimeVersion',
          sortable : true,
          groupable : false,
          flex : 0.5
        },

        /* OS related data */
        {
          itemId : 'osNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textOsNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipOsNameColumn' ),
          dataIndex : 'osName',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'osVersionColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textOsVersionColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipOsVersionColumn' ),
          dataIndex : 'osVersion',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'osArchColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.textOsArchColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.tooltipOsArchColumn' ),
          dataIndex : 'osArch',
          sortable : true,
          groupable : false,
          flex : 0.5
        }
    );

    Ext.apply( config, {
      columns : columns,
      store : store,

      qdesk : {
        connectionCtx : connectionCtx,
        gridId : config.qdesk.gridId
      }
    } );

    me.callParent( [ config ] );

    // rows have different CSS style that reflect the status
    me.getView().getRowClass = function ( record )
    {
      var execStatus = record.get( 'execStatus' );
      if ( execStatus == QDesk.model.scheduler.ExecStatus.SUCCESS )
        return 'statusSuccess';
      else if ( execStatus == QDesk.model.scheduler.ExecStatus.VETO )
        return 'statusVeto';
      else if ( execStatus == QDesk.model.scheduler.ExecStatus.ERROR )
        return 'statusError';
      else return '';
    };
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me.getStore().on( 'load', function ( store, records, successful, operation )
    {
      var emptyText = null;

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          var jobsTab = me.up( '#jobsTab' );
          var triggersTab = me.up( '#triggersTab' );

          if ( jobsTab )
          {
            var selectedJobs = jobsTab.down( '#jobsGrid' ).getSelectionModel().getSelection();
            if ( Ext.isEmpty( selectedJobs ) )
            {
              emptyText =
                  QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextNoJobSelected' );
            }
            else
            {
              if ( me.filtersValid() )
              {
                emptyText =
                    QDesk.Msg.get(
                        'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextNoMatchingDataForJob' );
              }
              else
              {
                emptyText =
                    QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextInvalidFilters' );
              }
            }
          }
          else if ( triggersTab )
          {
            var selectedTriggers = triggersTab.down( '#triggersGrid' ).getSelectionModel().getSelection();
            if ( Ext.isEmpty( selectedTriggers ) )
            {
              emptyText =
                  QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextNoTriggerSelected' );
            }
            else
            {
              if ( me.filtersValid() )
              {
                emptyText =
                    QDesk.Msg.get(
                        'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextNoMatchingDataForTrigger' );
              }
              else
              {
                emptyText =
                    QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextInvalidFilters' );
              }
            }
          }
          else
          {
            if ( me.filtersValid() )
            {
              emptyText =
                  QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextNoMatchingData' );
            }
            else
            {
              emptyText =
                  QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextInvalidFilters' );
            }
          }
        }
        else
        {
          // records not empty => show the records, not the empty text
        }
      }
      else
      {
        emptyText =
            QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExecHistoryGrid.emptyTextErrorLoadingData' );
      }

      if ( !Ext.isEmpty( emptyText ) )
      {
        me.empty( emptyText );
      }
    } );
  },


  /**
   * Overridden function that returns additional config values to be saved
   * with other grid config values.
   */
  getExtraConfig : function ()
  {
    var me = this;

    var extra = {};

    var fetchPeriod = me.down( '#fetchPeriod' ).getValue();
    if ( fetchPeriod != null )
    {
      var fetchPeriodLength = QDesk.util.CommonUtils.getDiffInDays( fetchPeriod.from, fetchPeriod.to );
      extra[ me.qdesk.gridId + '.fetchPeriodLength' ] = fetchPeriodLength;
    }

    var maxFetchSize = me.down( '#maxFetchSizeSelect' ).getValue();
    extra[ me.qdesk.gridId + '.maxFetchSize' ] = maxFetchSize;

    return extra;
  },


  /**
   * Returns true if the filters in the grid's toolbar are valid, false otherwise.
   * @return true if filters are valid, false otherwise.
   */
  filtersValid : function ()
  {
    var me = this;

    if ( !me.rendered )
      return true;

    var fetchPeriodPicker = me.down( '#fetchPeriod' );
    var valid = fetchPeriodPicker.isValid();

    // here we can add other filters

    return valid;
  },


  /**
   * Empties the grid and shows the specified emptyMsg (if not null).
   * @param emptyText an empty message to show.
   */
  empty : function ( emptyText )
  {
    var me = this;

    var store = me.getStore();
    store.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
    store.getProxy().extraParams = {};
    store.loadRecords( [] );

    if ( !Ext.isEmpty( emptyText ) )
    {
      var view = me.getView();
      view.emptyText = emptyText;
      view.refresh();
    }
  }
} );
