/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.store.scheduler.quartz.QuartzJobClassStore',

    'QDesk.view.work.scheduler.quartz.JobDataMapGrid',
    'QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode'
  ],

  alias : 'widget.QuartzJobDetailForm',

  defaultButton : 'actionButton',

  fieldDefaults : {
    labelWidth : 90
  },

  /* QDesk specific data */
  qdesk : {
    /**
     * Required {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined,

    /**
     * Required config parameter (JobDetailFormMode instance) describing the behavior of this component.
     */
    jobDetailFormMode : undefined,

    /**
     * Required config parameter containing the job detail data to populate this form with.
     */
    jobDetail : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.jobDetailFormMode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jobDetailFormMode' );

    if ( !config.qdesk.jobDetail )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jobDetail' );

    config.items =
        me._createItems( config.qdesk.connectionCtx, config.qdesk.jobDetailFormMode, config.qdesk.jobDetail );

    config.buttons = me._createButtons( config.qdesk.jobDetailFormMode );

    me.callParent( [ config ] );
  },


  _createItems : function ( connectionCtx, mode, jobDetail )
  {
    var me = this;

    var availableJobClassesStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzJobClassStore' );

    availableJobClassesStore.getProxy().extraParams = {
      connectionId : connectionCtx.getConnectionId()
    };

    return [
      {
        xtype : 'textfield',
        itemId : 'jobName',
        name : 'jobName',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldLabelJobName' ),
        fieldTooltip : me._getJobNameTooltip( mode ),
        emptyText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldEmptyTxtJobName' ),
        allowBlank : false,
        maxLength : 200,

        readOnly : mode == QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER ||
            mode == QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.EDIT,

        cls : mode == QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER ||
        mode == QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.EDIT ? 'readOnly' : ''
      },
      {
        xtype : 'textfield',
        itemId : 'jobGroupName',
        name : 'jobGroupName',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldLabelJobGroupName' ),
        fieldTooltip : me._getJobGroupNameTooltip( mode ),
        emptyText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldEmptyTxtJobGroupName' ),
        allowBlank : false,
        maxLength : 200,

        readOnly : mode == QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER ||
            mode == QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.EDIT,

        cls : mode == QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER ||
        mode == QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.EDIT ? 'readOnly' : ''
      },

      {
        xtype : 'combo',
        itemId : 'jobClass',
        name : 'jobClass',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldLabelJobClass' ),
        fieldTooltip : me._getJobClassTooltip( mode ),
        emptyText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldEmptyTxtJobClass' ),
        allowBlank : false,
        queryMode : 'local',

        store : availableJobClassesStore,
        valueField : 'className',
        displayField : 'className',

        maxLength : 250,

        readOnly : mode == QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER,

        cls : mode == QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER ? 'readOnly' : ''
      },

      {
        xtype : 'checkboxfield',
        itemId : 'durability',
        name : 'durability',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldLabelDurability' ),
        fieldTooltip : me._getDurabilityTooltip( mode ),
        allowBlank : true,

        /* All newly created jobs must be durable, otherwise Quartz complains about it */
        readOnly : mode === QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER,

        cls : mode === QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER ? 'readOnly' : ''
      },
      {
        xtype : 'checkboxfield',
        itemId : 'shouldRecover',
        name : 'shouldRecover',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldLabelShouldRecover' ),
        fieldTooltip : me._getShouldRecoverTooltip( mode ),
        allowBlank : true,
        readOnly : mode == QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER,

        cls : mode === QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER ? 'readOnly' : ''
      },
      {
        xtype : 'textarea',
        itemId : 'description',
        name : 'description',
        fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldLabelDescription' ),
        fieldTooltip : me._getDescriptionTooltip( mode ),
        emptyText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldEmptyTxtDescription' ),
        anchor : '100%',
        maxLength : 250,
        readOnly : mode == QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER,

        cls : mode === QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER ? 'readOnly' : ''
      },
      {
        xtype : 'QuartzJobDataMapGrid',
        itemId : 'jobDataMapGrid',
        title : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.titleJobDataMapGrid' ),
        style : 'margin-top: 10px', // space between the form and the grid
        height : 180,

        qdesk : {
          editable : me._isJobDataMapEditable( mode ),
          jobDataMap : jobDetail.get( 'jobDataMap' ) || []
        }
      }
    ]
  },


  /**
   * Returns true if the job data map grid in the form is editable, false otherwise.
   * @param mode a JobDetailFormMode value.
   * @return true if the job data map grid in the form is editable, false otherwise.
   */
  _isJobDataMapEditable : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.ADD:
        return true;

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.EDIT:
        return true;

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.CLONE:
        return true;

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER:
        return true;

      default:
        Ext.Error.raise( "Unrecognized job detail form mode: " + mode );
        return null;
    }
  },


  /**
   * Returns an array with a config object that represents the form buttons.
   * @param mode a JobDetailFormMode value.
   */
  _createButtons : function ( mode )
  {
    var me = this;

    return [
      {
        itemId : 'cancelButton',
        text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
      },
      {
        itemId : 'actionButton',
        reference : 'actionButton',
        text : me._getActionButtonText( mode ),
        formBind : true,
        qdRequiresAnyPermissions : me._getRequiredPermissions( mode )
      }
    ];
  },


  /**
   * Returns the text (title) for the JobDetailForm action button.
   * @param mode a JobDetailFormMode value.
   * @return the action button text.
   */
  _getActionButtonText : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.buttonLabelClone' );

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER:
        return QDesk.Msg.get( 'QDesk.buttonLabelTrigger' );

      default:
        Ext.Error.raise( "Unrecognized job detail form mode: " + mode );
        return null;
    }
  },


  /**
   * Returns an array with permissions required by the specified mode.
   * @param mode a JobDetailFormMode value.
   * @return the required permissions.
   */
  _getRequiredPermissions : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.ADD:
        return [ QDesk.model.security.Permission.QUARTZ_JOB_ADD ];

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.EDIT:
        return [ QDesk.model.security.Permission.QUARTZ_JOB_UPDATE ];

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.CLONE:
        return [ QDesk.model.security.Permission.QUARTZ_JOB_ADD ];

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER:
        return [ QDesk.model.security.Permission.QUARTZ_JOB_TRIGGER_MANUALLY ];

      default:
        Ext.Error.raise( "Unrecognized job detail form mode: " + mode );
        return null;
    }
  },


  _getJobNameTooltip : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipJobNameAdd' );

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipJobNameClone' );

      default:
        return null;
    }
  },


  _getJobGroupNameTooltip : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipJobGroupNameAdd' );

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipJobGroupNameClone' );

      default:
        return null;
    }
  },


  _getJobClassTooltip : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipJobClassAdd' );

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipJobClassEdit' );

      default:
        return null;
    }
  },


  _getDurabilityTooltip : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipDurabilityAdd' );

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipDurabilityEdit' );

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipDurabilityClone' );

      default:
        return null;
    }
  },


  _getShouldRecoverTooltip : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipShouldRecoverAdd' );

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipShouldRecoverEdit' );

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipShouldRecoverClone' );

      default:
        return null;
    }
  },


  _getDescriptionTooltip : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipDescriptionAdd' );

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipDescriptionEdit' );

      case QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobDetailForm.fieldTooltipDescriptionClone' );

      default:
        return null;
    }
  }
} );
