/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.store.scheduler.quartz.QuartzPlannedJobExecutionStore',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.widget.WClearableField',
    'QDesk.view.widget.WDateTimePeriod',
    'QDesk.view.widget.WExecHealthIndicator',
    'QDesk.view.widget.WReadOnlyCheckColumn'
  ],

  alias : 'widget.QuartzPlannedJobExecutionsGrid',

  border : false,
  scrollable : true,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.fetchPeriod )
      Ext.Error.raise( 'Required config parameter missing: qdesk.fetchPeriod' );

    var connectionCtx = config.qdesk.connectionCtx;
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    // var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    var jvmAgentReady = jvmAgentInfo.isReady();
    // var publicApiReady = schedulerInfo.isPublicApiReady();

    Ext.apply( config, {
      viewConfig : {
        deferEmptyText : false,
        emptyText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.emptyTextInitial' )
      }
    } );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'WDateTimePeriod',
          itemId : 'timePeriod',
          allowBlank : false,
          disabled : false,
          value : config.qdesk.fetchPeriod,
          listeners : {
            change : {
              fn : function ()
              {
                me.fireEvent( 'changeFilters' );
              }
            }
          }
        },

        '-',

        {
          xtype : 'WClearableField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.emptyTextFieldFilterQueryText' ),
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : false,
          listeners : {
            enter : {
              fn : function ()
              {
                me.fireEvent( 'changeFilters' );
              }
            },
            clear : {
              fn : function ()
              {
                me.fireEvent( 'changeFilters' );
              }
            }
          }
        },

        {
          xtype : 'button',
          itemId : 'submitButton',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.buttonLabelSubmit' ),
          iconCls : 'quartzViewPlannedJobExecutions',
          ui : 'default-small',
          margin : '0 0 0 5',
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'quartzViewPlannedJobExecutionsNAUTH'
        },

        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ]
    } );

    var columns = [];

    columns.push(
        {
          itemId : 'fireTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipFireTimeColumn' ),
          dataIndex : 'fireTime',
          sortable : false,
          groupable : false,
          flex : 0.8,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },

        {
          itemId : 'jobNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobNameColumn' ),
          dataIndex : 'jobDetail.name',
          sortable : false,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var jobDetail = record.get( 'jobDetail' );
            return Ext.String.htmlEncode( jobDetail.name );
          }
        },

        {
          itemId : 'jobGroupNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobGroupNameColumn' ),
          dataIndex : 'jobDetail.group.name',
          sortable : false,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var jobDetail = record.get( 'jobDetail' );
            return Ext.String.htmlEncode( jobDetail.group.name );
          }
        },

        {
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobClassNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobClassNameColumn' ),
          sortable : false,
          groupable : false,
          flex : 1.3,
          renderer : function ( value, metaData, record )
          {
            var jobDetail = record.get( 'jobDetail' );
            return jobDetail.jobClass;
          }
        }
    );

    if ( jvmAgentReady )
    {
      var maxExecHealthIndicatorEntries = jvmAgentInfo.getConfigProperty(
          'quartz.execHealthIndicators.jobs.maxEntriesPerIndicator' );

      columns.push(
          {
            xtype : 'widgetcolumn',
            itemId : 'jobExecHealthIndicatorColumn',
            text : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobExecHealthIndicatorColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobExecHealthIndicatorColumn',
                maxExecHealthIndicatorEntries ),
            dataIndex : 'jobDetail.execHealthIndicatorValues',
            sortable : false,
            groupable : false,
            width : 115,
            widget : {
              xtype : 'WExecHealthIndicator'
            },
            onWidgetAttach : function ( column, widget, record )
            {
              var jobDetail = record.get( 'jobDetail' );
              if ( jobDetail.execHealthIndicatorValues )
              {
                widget.setValues( jobDetail.execHealthIndicatorValues );
              }
            }
          }
      );
    }

    columns.push(
        {
          xtype : 'actioncolumn',
          itemId : 'jobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobDataMapColumn' ),
          header : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobDataMapColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobDataMapColumnIcon' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var jobDetail = record.get( 'jobDetail' );
                this.fireEvent( 'requestViewJobDataMap', jobDetail.jobDataMap );
              }
            }
          ]
        },

        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'jobDurabilityColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobDurabilityColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobDurabilityColumn' ),
          align : 'center',
          dataIndex : 'jobDetail.durability',
          sortable : false,
          groupable : false,
          width : 90,
          renderer : function ( value, metaData )
          {
            var jobDetail = metaData.record.get( 'jobDetail' );
            return this.defaultRenderer( jobDetail.durability, metaData );  // does not work for nested properties - we use renderer
          }
        },

        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'jobShouldRecoverColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobShouldRecoverColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobShouldRecoverColumn' ),
          align : 'center',
          dataIndex : 'jobDetail.shouldRecover',
          sortable : false,
          groupable : false,
          width : 90,
          renderer : function ( value, metaData )
          {
            var jobDetail = metaData.record.get( 'jobDetail' );
            return this.defaultRenderer( jobDetail.shouldRecover, metaData );  // does not work for nested properties - we use renderer
          }
        },

        // include only for Quartz < 2.x
        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'jobVolatilityColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobVolatilityColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobVolatilityColumn' ),
          align : 'center',
          dataIndex : 'jobDetail.volatility',
          sortable : false,
          groupable : false,
          width : 90,
          renderer : function ( value, metaData )
          {
            var jobDetail = metaData.record.get( 'jobDetail' );
            return this.defaultRenderer( jobDetail.volatility, metaData );  // does not work for nested properties - we use renderer
          }
        }
    );

    if ( jvmAgentReady )
    {
      columns.push(
          // available only if the agent is present, otherwise the value is unknown
          {
            xtype : 'WReadOnlyCheckColumn',
            itemId : 'jobConcurrentColumn',
            text : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobConcurrentColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobConcurrentColumn' ),
            align : 'center',
            dataIndex : 'jobDetail.concurrentExecutionDisallowed',
            sortable : false,
            groupable : false,
            width : 90,
            renderer : function ( value, metaData )
            {
              var jobDetail = metaData.record.get( 'jobDetail' );
              return this.defaultRenderer( !jobDetail.concurrentExecutionDisallowed, metaData );  // does not work for nested properties - we use renderer
            }
          },

          // available only if the agent is present, otherwise the value is unknown
          {
            xtype : 'WReadOnlyCheckColumn',
            itemId : 'jobPersistJobDataAfterExecutionColumn',
            text : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textJobPersistsJobDataColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipJobPersistsJobDataColumn' ),
            align : 'center',
            dataIndex : 'jobDetail.persistJobDataAfterExecution',
            sortable : false,
            groupable : false,
            width : 120,
            renderer : function ( value, metaData )
            {
              var jobDetail = metaData.record.get( 'jobDetail' );
              return this.defaultRenderer( jobDetail.persistJobDataAfterExecution, metaData );  // does not work for nested properties - we use renderer
            }
          }
      );
    }

    columns.push(
        {
          itemId : 'triggerNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textTriggerNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipTriggerNameColumn' ),
          dataIndex : 'triggerName',
          sortable : false,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },

        {
          itemId : 'triggerGroupNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textTriggerGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipTriggerGroupNameColumn' ),
          dataIndex : 'triggerGroupName',
          sortable : false,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },

        {
          xtype : 'actioncolumn',
          itemId : 'triggerJobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textTriggerJobDataMapColumn' ),
          header : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textTriggerJobDataMapColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipTriggerJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipTriggerJobDataMapColumnIcon' ),
              handler : function ( grid, rowIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var triggerJobDataMap = record.get( 'triggerJobDataMap' );
                this.fireEvent( 'requestViewJobDataMap', triggerJobDataMap );
              }
            }
          ]
        },

        {
          xtype : 'actioncolumn',
          itemId : 'mergedJobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textMergedJobDataMapColumn' ),
          header : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.textMergedJobDataMapColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipMergedJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 150,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.tooltipMergedJobDataMapColumnIcon' ),
              handler : function ( grid, rowIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var mergedJobDataMap = record.get( 'mergedJobDataMap' );
                this.fireEvent( 'requestViewJobDataMap', mergedJobDataMap );
              }
            }
          ]
        }
    );

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzPlannedJobExecutionStore' );

    Ext.apply( config, {
      columns : columns,
      store : store,
      qdesk : {
        connectionCtx : connectionCtx,
        gridId : 'quartz.plannedJobExecutionsGrid'
      }
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me.getStore().on( 'load', function ( store, records, successful, operation )
    {
      var view = me.getView();

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          view.emptyText =
              QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.emptyTextNoMatchingData' );

          view.refresh();
        }
      }
      else
      {
        view.emptyText =
            QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid.emptyTextErrorLoadingData' );

        view.refresh();
      }
    } );
  },


  /**
   * Returns true if the filters in the grid's toolbar are valid, false otherwise.
   * @return true if filters are valid, false otherwise.
   */
  filtersValid : function ()
  {
    var me = this;

    var toolbar = me.down( 'toolbar' );
    if ( !toolbar || !toolbar.rendered )
      return true;

    var timePeriodPicker = me.down( '#timePeriod' );
    var valid = timePeriodPicker.isValid();

    // here we can add other filters

    return valid;
  },


  /**
   * Empties the grid and shows the specified emptyMsg (if not null).
   * @param emptyText an empty message to show.
   */
  empty : function ( emptyText )
  {
    var me = this;

    var store = me.getStore();
    store.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
    store.getProxy().extraParams = {};
    store.loadRecords( [] );

    if ( !Ext.isEmpty( emptyText ) )
    {
      var view = me.getView();
      view.emptyText = emptyText;
      view.refresh();
    }
  }
} );
