/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsTab', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerXPanel',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.widget.message.WarningMessagePanel',
    'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid'
  ],

  alias : 'widget.QuartzPlannedJobExecutionsTab',

  iconCls : 'quartzPlannedJobExecutionsSubTabIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    var connectionCtx = config.qdesk.connectionCtx;
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    var jvmAgentReady = jvmAgentInfo.isReady();
    var publicApiReady = schedulerInfo.isPublicApiReady();

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.PlannedJobExecutions )
    {
      if ( jvmAgentReady )
      {
        if ( publicApiReady )
        {
          var fetchPeriod = QDesk.util.CommonUtils.getDefaultFetchPeriod( schedulerInfo.getSchedulerTimeZone(),
              QDesk.ApplicationConfig.get( 'quartz.plannedJobExecutionsGrid.fetchPeriodLength' ) );

          Ext.apply( config, {
            layout : 'fit',
            items : [
              {
                xtype : 'QuartzPlannedJobExecutionsGrid',
                itemId : 'plannedJobExecutionsGrid',
                flex : 1,
                qdesk : {
                  connectionCtx : connectionCtx,
                  fetchPeriod : fetchPeriod
                }
              }
            ]
          } );
        }
        else
        {
          Ext.apply( config, {
            items : [
              {
                xtype : 'WarningMessagePanel',
                style : {
                  margin : '5px'
                },
                qdesk : {
                  msg : me.getPublicApiWarningMessage( connectionCtx )
                }
              }
            ]
          } );
        }
      }
      else
      {
        Ext.apply( config, {
          items : [
            {
              xtype : 'WarningMessagePanel',
              style : {
                margin : '5px'
              },
              qdesk : {
                msg : me.getJvmAgentWarningMessage( connectionCtx )
              }
            }
          ]
        } );
      }
    }
    else
    {
      Ext.apply( config, {
        items : [
          {
            xtype : 'WarningMessagePanel',
            style : {
              margin : '5px'
            },
            qdesk : {
              msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
                  QDesk.util.license.LicensedProduct.productName )
            }
          }
        ]
      } );
    }

    Ext.apply( config, {
      title : me._getTitle(),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsTab.tooltip' )
    } );

    me.callParent( [ config ] );
  },


  _getTitle : function ()
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var title = QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsTab.title' );
    if ( !licensedFeatures.PlannedJobExecutions )
    {
      title = '<span class="notLicensed">' + title + '</span>'
    }

    return title;
  }
} );