Ext.define( 'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGridController', {
  extend : 'Ext.app.ViewController',

  requires : [
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    me.callParent( [ config ] );
  },

  init : function ()
  {
    var me = this;

    // JobChainsGrid
    var grid = me.getView();

    me._bindEventHandlers( grid );

    me.callParent();
  },


  /**
   * Binds the notification rules grid events to their handlers.
   */
  _bindEventHandlers : function ( grid )
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var connectionCtx = me.qdesk.connectionCtx;
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    // var schedulerVersion = schedulerInfo.getSchedulerVersion();
    var jvmAgentReady = jvmAgentInfo.isReady();
    // var publicApiReady = schedulerInfo.isPublicApiReady();

    if ( licensedFeatures.ExecutionNotifications && jvmAgentReady )
    {
      grid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onNotificationRuleUnselected( grid );
        }
        else
        {
          me._onNotificationRuleSelected( selectedRecords[ 0 ], grid );
        }
      } );

      var filterQueryText = grid.down( '#filterQueryText' );

      if ( jvmAgentReady )
      {
        var notificationRulesIndexEnabled = jvmAgentInfo.isQuartzExecNotificationRulesIndexEnabled();

        if ( !notificationRulesIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGridController.notificationRulesIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesGridController.indexAgentNotReadyEmptyText' ) );
      }

      grid.getStore().on( 'beforeload',
          function ( store, operation )
          {
            grid.down( '#refreshButton' ).disable();
            grid.down( '#addButton' ).disable();
            grid.down( '#deleteButton' ).disable();
            grid.down( '#editButton' ).disable();
            grid.down( '#cloneButton' ).disable();
            grid.down( '#viewAuditTrailButton' ).disable();
            filterQueryText.disable();

            /*
             * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
             * Therefore we need to force-fire this event by clearing the selection.
             */
            grid.getSelectionModel().deselectAll();
          } );

      grid.getStore().on( 'load',
          function ( store, records, successful, operation )
          {
            grid.down( '#refreshButton' ).enable();

            // enable only if the notification rules index is enabled in the agent
            if ( notificationRulesIndexEnabled )
            {
              filterQueryText.enable();
            }

            if ( successful )
            {
              grid.down( '#addButton' ).enable();
            }
            else
            {
              store.removeAll();

              QDesk.util.UxUtils.showExtJsOperationError( operation );
            }
          } );
    }
  },


  /**
   * Invoked when a notification rule record in the ExecNotificationRulesGrid is selected.
   * @param notificationRule a QDesk.model.scheduler.quartz.QuartzExecNotificationRule instance representing the selected
   * notification rule.
   * @param grid the notification rules grid.
   */
  _onNotificationRuleSelected : function ( notificationRule, grid )
  {
    var me = this;

    grid.down( '#deleteButton' ).enable();
    grid.down( '#editButton' ).enable();
    grid.down( '#cloneButton' ).enable();
    grid.down( '#viewAuditTrailButton' ).enable();
  },


  /**
   * Invoked when a notification rule record in the ExecNotificationRulesGrid is unselected.
   * is unselected.
   * @param grid the notification rules grid.
   */
  _onNotificationRuleUnselected : function ( grid )
  {
    var me = this;

    grid.down( '#deleteButton' ).disable();
    grid.down( '#editButton' ).disable();
    grid.down( '#cloneButton' ).disable();
    grid.down( '#viewAuditTrailButton' ).disable();
  }
} );