/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesTab', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerXPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.work.scheduler.quartz.notif.exec.SchedulerExecNotificationRulesView',
    'QDesk.view.work.scheduler.quartz.notif.exec.JobExecNotificationRulesView',
    'QDesk.view.work.scheduler.quartz.notif.exec.TriggerExecNotificationRulesView',

    'QDesk.view.widget.tab.XTabPanel'
  ],

  alias : 'widget.QuartzExecNotificationRulesTab',

  iconCls : 'quartzExecNotificationRulesTabIcon',

  // border : false,
  /* The top tab should not be scrollable, but nested panels (views) should be! */
  scrollable : false,

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    var connectionCtx = config.qdesk.connectionCtx;
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();
    var schedulerInfo = connectionCtx.getSchedulerInfo();

    var schedulerVersion = schedulerInfo.getSchedulerVersion();
    var jvmAgentReady = jvmAgentInfo.isReady();
    var publicApiReady = schedulerInfo.isPublicApiReady();

    var items = [];

    items.push(
        {
          xtype : 'QuartzSchedulerExecNotificationRulesView',
          itemId : 'schedulerExecNotificationRulesView',
          qdesk : {
            connectionCtx : connectionCtx
          }
        } );

    items.push(
        {
          xtype : 'QuartzJobExecNotificationRulesView',
          itemId : 'jobExecNotificationRulesView',
          qdesk : {
            connectionCtx : connectionCtx
          }
        } );

    items.push(
        {
          xtype : 'QuartzTriggerExecNotificationRulesView',
          itemId : 'triggerExecNotificationRulesView',
          qdesk : {
            connectionCtx : connectionCtx
          }
        } );

    Ext.apply( config, {
      items : [
        {
          xtype : 'XTabPanel',
          border : false,
          flex : 1,  // for the parent vbox/stretch layout
          items : items
        }
      ]
    } );

    Ext.apply( config, {
      title : me._getTitle(),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesTab.tooltip' )
    } );

    me.callParent( [ config ] );
  },


  _getTitle : function ()
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var title = QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.notif.exec.ExecNotificationRulesTab.title' );
    if ( !licensedFeatures.ExecutionNotifications )
    {
      title = '<span class="notLicensed">' + title + '</span>'
    }

    return title;
  },


  /**
   * Returns the currently active view (QuartzSchedulerExecNotificationRulesView, QuartzJobExecNotificationRulesView,
   * QuartzTriggerExecNotificationRulesView).
   * @returns the currently active view.
   */
  getActiveView : function ()
  {
    var me = this;

    var tabPanel = me.down( 'XTabPanel' );
    return tabPanel.getActiveTab();
  }
} );
