/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.stats.MostFrequentlyExecutedTriggersChart', {
  extend : 'Ext.chart.PolarChart',

  requires : [
    'QDesk.Msg',

    'QDesk.model.scheduler.ExecStatus',
    'QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType',

    'QDesk.store.scheduler.quartz.stats.QuartzExecStatDataStore',

    'QDesk.view.work.scheduler.quartz.stats.theme.MostFrequentlyExecutedTriggersChartTheme'
  ],

  style : 'background:#fff',

  animate : true,

  border : false,
  bodyBorder : false,

  innerPadding : 10,
  insetPadding : 90,

  shadow : true,

  theme : 'QuartzMostFrequentlyExecutedTriggersChartTheme',

  legend : {
    boxStroke : '#444444',
    labelColor : '#444444',
    docked : 'right'
  },

  /*
   * Present to suppress warning: Using Sencha's download server could expose your data and pose a security risk. Please see Ext.draw.Container#download method docs for more info.
   */
  downloadServerUrl : 'http://localhost',

  qdesk : {
    /**
     * Required config parameter with the statistics period length for this chart.
     */
    periodLength : undefined,

    /**
     * Required config parameter with the statistics period start date for this chart.
     */
    periodStart : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.periodLength )
      Ext.Error.raise( 'Required config parameter missing: qdesk.periodLength' );

    if ( !config.qdesk.periodStart )
      Ext.Error.raise( 'Required config parameter missing: qdesk.periodStart' );

    Ext.apply( config, {
      store : Ext.create( 'QDesk.store.scheduler.quartz.stats.QuartzExecStatDataStore' ),

      series : [
        {
          type : 'pie',
          angleField : 'countSuccessAndError',
          donut : 20,
          tooltip : {
            dismissDelay : 0,
            trackMouse : true,
            renderer : function ( tooltip, storeItem, ctx )
            {
              var html = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.stats.MostFrequentlyExecutedTriggersChart.tooltipPieSection',
                  Ext.String.htmlEncode( storeItem.get( 'triggerGroupName' ) ),
                  Ext.String.htmlEncode( storeItem.get( 'triggerName' ) ),
                  Ext.util.Format.number( storeItem.get( 'countSuccess' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'timeSuccess' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'avgTimeSuccess' ), '0,000.00' ),
                  Ext.util.Format.number( storeItem.get( 'countError' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'timeError' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'avgTimeError' ), '0,000.00' ),
                  Ext.util.Format.number( storeItem.get( 'countVeto' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'countSuccessAndError' ), '0,000' ) );

              tooltip.setHtml( html );
            }
          },
          highlight : true,
          label : {
            field : 'triggerDisplayName',
            calloutLine : {
              length : 80,
              width : 3
            },
            display : 'outside',
            contrast : true
            //font : '15px Verdana'
          }
        }
      ]
    } );

    me.callParent( [ config ] );
  }
} );