/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',

    'QDesk.model.scheduler.quartz.QuartzTriggerType',

    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode',

    'QDesk.view.work.scheduler.quartz.triggers.EditCalendarIntervalTriggerForm',
    'QDesk.view.work.scheduler.quartz.triggers.EditCronTriggerForm',
    'QDesk.view.work.scheduler.quartz.triggers.EditDailyTimeIntervalTriggerForm',
    'QDesk.view.work.scheduler.quartz.triggers.EditDateIntervalTriggerForm',
    'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm',
    'QDesk.view.work.scheduler.quartz.triggers.EditSimpleTriggerForm'
  ],

  alias : 'widget.QuartzEditTriggerWindow',

  cls : 'quartzEditTriggerWindow',
  iconCls : 'windowIcon',

  modal : true,
  resizable : false,

//  minWidth : 500,
  width : 600,

  qdesk : {
    /**
     * Required {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined,

    /*
     * Mode of the form (see EditTriggerFormMode).
     */
    mode : undefined,

    /*
     * Trigger type (see QDesk.model.scheduler.quartz.QuartzTriggerType).
     */
    triggerType : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.triggerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerType' );

    Ext.apply( config, {
      title : me._getWindowTitle( config.qdesk.mode ),
      items : [
        {
          // form xtype depends on the requested trigger type
          xtype : me._getFormXType( config.qdesk.triggerType ),
          itemId : 'form',

          qdesk : {
            connectionCtx : config.qdesk.connectionCtx,
            mode : config.qdesk.mode,
            triggerType : config.qdesk.triggerType
          }
        }
      ]
    } );

    me.callParent( [ config ] );

    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } )
  },


  /**
   * Returns the title of this window.
   * @param mode a QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode value.
   * @return {string} the title.
   * @private
   */
  _getWindowTitle : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerWindow.windowTitleAdd' );

      case QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerWindow.windowTitleEdit' );

      case QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerWindow.windowTitleClone' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  },


  /**
   * Returns the XType of the edit form for the specified trigger type.
   * @param triggerType a trigger type.
   * @returns {string} the XType.
   * @private
   */
  _getFormXType : function ( triggerType )
  {
    switch ( triggerType )
    {
      case QDesk.model.scheduler.quartz.QuartzTriggerType.SIMPLE:
        return 'QuartzEditSimpleTriggerForm';

      case QDesk.model.scheduler.quartz.QuartzTriggerType.CRON:
        return 'QuartzEditCronTriggerForm';

      case QDesk.model.scheduler.quartz.QuartzTriggerType.CALENDAR_INTERVAL:
        return 'QuartzEditCalendarIntervalTriggerForm';

      case QDesk.model.scheduler.quartz.QuartzTriggerType.DAILY_TIME_INTERVAL:
        return 'QuartzEditDailyTimeIntervalTriggerForm';

      case QDesk.model.scheduler.quartz.QuartzTriggerType.DATE_INTERVAL:
        return 'QuartzEditDateIntervalTriggerForm';

      case QDesk.model.scheduler.quartz.QuartzTriggerType.NTH_INCLUDED_DAY:
        return 'QuartzEditNthIncludedDayTriggerForm';

      default:
        Ext.Error.raise( 'Unrecognized trigger type: ' + triggerType );
        return null;
    }
  }
} );
