/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.security.groups.GroupsTab', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.view.work.security.groups.GroupsGrid',
    'QDesk.view.widget.message.InfoMessagePanel',
    'QDesk.view.work.security.groups.EditGroupForm',

    'QDesk.controller.command.security.groups.AddGroup',
    'QDesk.controller.command.security.groups.DeleteGroup',
    'QDesk.controller.command.security.groups.EditGroup',
    'QDesk.controller.command.security.groups.SaveGroup'
  ],

  alias : 'widget.GroupsTab',

  cls : 'groupsTab',
  bodyCls : 'groupsTabBody',

  iconCls : 'groupsTabIcon',

  border : false,

  closable : true,
  scrollable : true,

  layout : {
    type : 'border'
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsTab.title' ),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsTab.tooltip' ),
      items : [
        {
          xtype : 'panel',
          border : false,
          layout : {
            type : 'vbox',
            align : 'stretch'
          },
          collapsible : true,
          collapseMode : 'mini',
          split : true,
          width : 350,
          minWidth : 350,
          region : 'west',
          items : [
            {
              xtype : 'GroupsGrid',
              itemId : 'groupsGrid',
              flex : 1
            }
          ]
        },
        {
          xtype : 'panel',
          itemId : 'editPanel',
          border : false,
          flex : 1,
          minWidth : 420,
          region : 'center',
          layout : 'card',
          items : [
            {
              xtype : 'panel',
              itemId : 'infoPanel',
              border : false,
              bodyPadding : 5,
              items : [
                {
                  xtype : 'InfoMessagePanel',
                  qdesk : {
                    msg : QDesk.Msg.get( 'QDesk.view.work.security.groups.GroupsTab.infoText' )
                  }
                }
              ]
            }
            // {
            //   xtype : 'EditGroupForm',  // this form panel is created and added dynamically
            // }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    var groupsGrid = me.down( '#groupsGrid' );

    groupsGrid.on( 'addGroup', function ()
    {
      me._cleanupEditPanel();

      Ext.create( 'QDesk.controller.command.security.groups.AddGroup', {} ).execute();
    } );


    groupsGrid.on( 'deleteGroup', function ( groupStub )
    {
      Ext.create( 'QDesk.controller.command.security.groups.DeleteGroup', {
        groupStub : groupStub,
        success : function ()
        {
          groupsGrid.refresh();
        }
      } ).execute();
    } );


    // groupsGrid.on( 'lockGroup', function ( groupStub )
    // {
    //   Ext.create( 'QDesk.controller.command.security.groups.LockGroup', {
    //     groupStub : groupStub,
    //     success : function ()
    //     {
    //       groupsGrid.refresh();
    //     }
    //   } ).execute();
    // } );
    //
    //
    // groupsGrid.on( 'unlockGroup', function ( groupStub )
    // {
    //   Ext.create( 'QDesk.controller.command.security.groups.UnlockGroup', {
    //     groupStub : groupStub,
    //     success : function ()
    //     {
    //       groupsGrid.refresh();
    //     }
    //   } ).execute();
    // } );


    groupsGrid.on( 'groupSelected', function ( groupStub )
    {
      me._cleanupEditPanel();

      Ext.create( 'QDesk.controller.command.security.groups.EditGroup', {
        groupStub : groupStub
      } ).execute();
    } );


    groupsGrid.on( 'groupUnselected', function ()
    {
      me._cleanupEditPanel();
    } );
  },


  /**
   * Adds the specified EditGroupForm panel to the edit panel and makes the added panel active.
   * @param formPanel a EditGroupForm panel to add.
   */
  showEditGroupForm : function ( formPanel )
  {
    var me = this;

    var groupsGrid = me.down( '#groupsGrid' );

    formPanel.on( 'saveGroup', function ( group )
    {
      Ext.create( 'QDesk.controller.command.security.groups.SaveGroup', {
        formPanel : formPanel,
        group : group,
        success : function ()
        {
          me._cleanupEditPanel();

          groupsGrid.refresh();
        }
      } ).execute();
    } );

    var editPanel = me.down( '#editPanel' );
    editPanel.add( formPanel );
    editPanel.getLayout().setActiveItem( formPanel );
  },


  /**
   * Removes all panels from the edit panel but the info panel.
   * @private
   */
  _cleanupEditPanel : function ()
  {
    var me = this;

    var editPanel = me.down( '#editPanel' );

    // remove all panels (edit group form panel) from the edit panel (right panel), but the infoPanel
    Ext.each( editPanel.getLayout().getLayoutItems(), function ( panel )
    {
      if ( panel.getItemId() == 'infoPanel' )
      {
        // edit panel uses card layout, so we activate the info panel
        editPanel.getLayout().setActiveItem( panel );
      }
      else
      {
        editPanel.remove( panel );
      }
    } );
  }
} );