/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.security.users.EditUserForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',
    'QDesk.model.security.UserAvatarType',

    'QDesk.store.enums.security.PermissionStore',
    'QDesk.store.security.groups.GroupStore',

    'QDesk.util.FormUtils',
    'QDesk.util.security.UserUtils',

    'QDesk.view.widget.message.WarningMessagePanel',
    'QDesk.view.work.security.users.EditUserFormMode',

    'QDesk.view.widget.WPhoneNumber'
  ],

  alias : 'widget.EditUserForm',

  bodyPadding : '5 10 10 10',

  border : false,

  scrollable : true,

  fieldDefaults : {
    labelWidth : 80
  },

  defaultButton : 'actionButton',

  qdesk : {
    /*
     * Mode of the form (see EditUserFormMode).
     */
    mode : undefined
  },

  EXTERNAL_USER_PASSWD_PLACEHOLDER : '<password-placeholder>',


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    var groupStore = Ext.create( 'QDesk.store.security.groups.GroupStore', {
      sorters : [
        {
          property : 'displayName',
          direction : 'ASC'
        }
      ]
    } );

    /*
     * Important to escape HTML in user displayName attribute that is shown in the groups item selector field.
     * JMO: this is a bit of a hack as I have not found a way to make the item selector use a custom renderer
     * that would escape the displayed values.
     */
    groupStore.on( 'load', function ( store, records, successful )
    {
      if ( successful )
      {
        Ext.each( records, function ( record )
        {
          record.set( 'displayName', Ext.String.htmlEncode( record.get( 'displayName' ) ) );
        } );
      }
    } );

    Ext.apply( config, {
      items : [
        {
          xtype : 'WarningMessagePanel',
          itemId : 'warningMessagePanel',
          hidden : true,
          style : {
            marginBottom : '5px'
          }
        },

        {
          xtype : 'hiddenfield',
          itemId : 'id',
          name : 'id'
        },

        {
          xtype : 'hiddenfield',
          itemId : 'system',
          name : 'system'
        },

        {
          xtype : 'textfield',
          itemId : 'externalId',
          name : 'externalId',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.security.users.EditUserForm.fieldLabelExternalId' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.security.users.EditUserForm.fieldTooltipExternalId' ),
          // emptyText : QDesk.Msg.get(
          //     'QDesk.view.work.security.users.EditUserForm.fieldEmptyTxtExternalId' ),
          allowBlank : true,
          hidden : true,
          readOnly : true,
          maxLength : 128,
          maxWidth : 350
        },

        {
          xtype : 'textfield',
          itemId : 'firstName',
          name : 'firstName',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.security.users.EditUserForm.fieldLabelFirstName' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.security.users.EditUserForm.fieldTooltipFirstName' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.security.users.EditUserForm.fieldEmptyTxtFirstName' ),
          allowBlank : false,
          maxLength : 128,
          maxWidth : 350
        },
        {
          xtype : 'textfield',
          itemId : 'surname',
          name : 'surname',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.security.users.EditUserForm.fieldLabelSurname' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.security.users.EditUserForm.fieldTooltipSurname' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.security.users.EditUserForm.fieldEmptyTxtSurname' ),
          allowBlank : false,
          maxLength : 128,
          maxWidth : 350
        },

        {
          xtype : 'textfield',
          itemId : 'email',
          name : 'email',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.security.users.EditUserForm.fieldLabelEmail' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.security.users.EditUserForm.fieldTooltipEmail' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.security.users.EditUserForm.fieldEmptyTxtEmail' ),
          allowBlank : false,
          maxLength : 128,
          vtype : 'email',
          maxWidth : 350
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.fieldsetLabelLoginCredentials' ),
          padding : '0 10 0 10',
          defaults : {
            labelWidth : 120
            //anchor : '100%'
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'username',
              name : 'username',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.fieldLabelUsername' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.fieldTooltipUsername' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.fieldEmptyTxtUsername' ),
              allowBlank : false,
              maxLength : 128,
              maxWidth : 340,
              minWidth : 340
            },

            {
              xtype : 'textfield',
              itemId : 'password',
              name : 'password',
              inputType : 'password',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.fieldLabelPassword' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.fieldTooltipPassword' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.fieldEmptyTxtPassword' ),
              allowBlank : false,
              minLength : 6, /* Passwords must be at least 6 characters long */
              maxLength : 128,
              maxWidth : 340,
              minWidth : 340
            },
            {
              xtype : 'textfield',
              itemId : 'password2',
              name : 'password2',
              inputType : 'password',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.fieldLabelPassword2' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.fieldTooltipPassword2' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.fieldEmptyTxtPassword2' ),
              allowBlank : false,
              maxLength : 128,
              maxWidth : 340,
              minWidth : 340,
              validator : function ( value )
              {
                if ( Ext.isEmpty( value ) )
                {
                  return true;
                }
                else
                {
                  var passwordField = me.down( '#password' );
                  return value === passwordField.getValue() ? true :
                      QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.validationPasswordsDoNotMatch' );
                }
              }
            }
          ]
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.fieldsetLabelAvatar' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.fieldsetTooltipAvatar' ),
          padding : '0 10 0 10',
          layout : {
            type : 'hbox',
            align : 'center',
            pack : 'left'
          },
          items : [
            {
              xtype : 'image',
              itemId : 'avatarPreview',
              cls : 'avatarPreview',
              margin : '0 10 0 0',
              height : 35,
              width : 35,
              src : 'images/icon/avatar-default-35x35.png'  // default avatar image
            },

            {
              xtype : 'fieldcontainer',
              items : [
                {
                  xtype : 'radiogroup',
                  itemId : 'avatarType',
                  columns : 1,
                  vertical : true,
                  allowBlank : false,
                  layout : {
                    type : 'vbox',
                    align : 'stretch'
                  },
                  qdesk : {
                    required : true
                  },
                  items : [
                    {
                      xtype : 'radio',
                      boxLabel : QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.boxLabelAvatarNone' ),
                      boxTooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.boxTooltipAvatarNone' ),
                      name : 'avatarType',
                      inputValue : QDesk.model.security.UserAvatarType.NONE
                    },

                    // {
                    //   xtype : 'radio',
                    //   boxLabel : QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.boxLabelAvatarLocal' ),
                    //   boxTooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.boxTooltipAvatarLocal' ),
                    //   name : 'avatarType',
                    //   inputValue : QDesk.model.security.UserAvatarType.LOCAL
                    // },

                    {
                      xtype : 'radio',
                      boxLabel : QDesk.Msg.get(
                          'QDesk.view.work.security.users.EditUserForm.boxLabelAvatarExternalCustomUrl' ),
                      boxTooltip : QDesk.Msg.get(
                          'QDesk.view.work.security.users.EditUserForm.boxTooltipAvatarExternalCustomUrl' ),
                      name : 'avatarType',
                      inputValue : QDesk.model.security.UserAvatarType.EXTERNAL_CUSTOM_URL
                    },
                    {
                      xtype : 'textfield',
                      itemId : 'avatarExternalCustomUrl',
                      name : 'avatarExternalCustomUrl',
                      fieldLabel : QDesk.Msg.get(
                          'QDesk.view.work.security.users.EditUserForm.fieldLabelAvatarExternalCustomUrl' ),
                      fieldTooltip : QDesk.Msg.get(
                          'QDesk.view.work.security.users.EditUserForm.fieldTooltipAvatarExternalCustomUrl' ),
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.security.users.EditUserForm.fieldEmptyTxtAvatarExternalCustomUrl' ),
                      labelWidth : 50,
                      allowBlank : false,
                      maxLength : 128,
                      vtype : 'url',
                      maxWidth : 420,
                      minWidth : 360
                    },

                    {
                      xtype : 'radio',
                      boxLabel : QDesk.Msg.get(
                          'QDesk.view.work.security.users.EditUserForm.boxLabelAvatarExternalGravatar' ),
                      boxTooltip : QDesk.Msg.get(
                          'QDesk.view.work.security.users.EditUserForm.boxTooltipAvatarExternalGravatar' ),
                      name : 'avatarType',
                      inputValue : QDesk.model.security.UserAvatarType.EXTERNAL_GRAVATAR
                    }
                  ]
                }
              ]
            }
          ]
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.fieldsetLabelWorkPhoneNumber' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.fieldsetTooltipWorkPhoneNumber' ),
          padding : '0 10 0 10',
          defaults : {
            // labelWidth : 115,
            // anchor : '100%'
          },
          items : [
            {
              xtype : 'WPhoneNumber',
              itemId : 'workPhoneNumber',
              name : 'workPhoneNumber',
              allowBlank : true
            }
          ]
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.fieldsetLabelMobilePhoneNumber' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.fieldsetTooltipMobilePhoneNumber' ),
          padding : '0 10 0 10',
          defaults : {
            // labelWidth : 115,
            // anchor : '100%'
          },
          items : [
            {
              xtype : 'WPhoneNumber',
              itemId : 'mobilePhoneNumber',
              name : 'mobilePhoneNumber',
              allowBlank : true
            }
          ]
        },

        {
          xtype : 'tabpanel',
          padding : '0 0 10 0',
          items : [
            {
              xtype : 'panel',
              padding : '10 10 5 10',
              border : false,
              iconCls : 'permissionsTabIcon',
              title : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.tabLabelPermissions' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.tabTooltipPermissions' ),
              layout : 'fit',
              // defaults : {
              //   anchor : '100%'
              // },
              items : [
                {
                  xtype : 'itemselectorfield',
                  itemId : 'permissions',
                  name : 'permissionIds',
                  fromTitle : QDesk.Msg.get(
                      'QDesk.view.work.security.users.EditUserForm.fieldTitlePermissionsFrom' ),
                  toTitle : QDesk.Msg.get(
                      'QDesk.view.work.security.users.EditUserForm.fieldTitlePermissionsTo' ),
                  allowBlank : true,
                  displayField : 'valueLabelWithTooltip',
                  valueField : 'value',
                  //minSelections : 1,
                  buttons : [ 'add', 'remove' ],
                  store : 'enums.security.PermissionStore',
                  height : 400
                }
              ]
            },
            {
              xtype : 'panel',
              padding : '10 10 5 10',
              border : false,
              iconCls : 'groupsTabIcon',
              title : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.tabLabelGroups' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.security.users.EditUserForm.tabTooltipGroups' ),
              layout : 'fit',
              // defaults : {
              //   anchor : '100%'
              // },
              items : [
                {
                  xtype : 'itemselectorfield',
                  itemId : 'groups',
                  name : 'groupIds',
                  fromTitle : QDesk.Msg.get(
                      'QDesk.view.work.security.users.EditUserForm.fieldTitleGroupsFrom' ),
                  toTitle : QDesk.Msg.get(
                      'QDesk.view.work.security.users.EditUserForm.fieldTitleGroupsTo' ),
                  allowBlank : true,
                  displayField : 'displayName',
                  valueField : 'id',
                  //minSelections : 1,
                  buttons : [ 'add', 'remove' ],
                  store : groupStore,
                  height : 400
                }
              ]
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'viewEffectivePermissionsButton',
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.buttonLabelViewEffectivePermissions' ),
          iconCls : 'viewEffectiveUserPermissions',
          disabled : config.qdesk.mode == QDesk.view.work.security.users.EditUserFormMode.ADD,  // available only for saved users
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.USER_ADD,
            QDesk.model.security.Permission.USER_UPDATE
          ],
          qdNotAuthorizedIconCls : 'viewEffectiveUserPermissionsNAUTH'
        },

        {
          itemId : 'actionButton',
          reference : 'actionButton',
          text : me._getActionButtonText( config.qdesk.mode ),
          formBind : true,
          qdRequiresAnyPermissions : me._getRequiredPermissions( config.qdesk.mode )
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();

    me._setDefaultFormValues();
  },


  /**
   * Loads the specified user record ({@link User} instance) into this form.
   * @param user the user record.
   */
  loadRecord : function ( user )
  {
    var me = this;
    me.callParent( arguments );

    me.getForm().setValues( {
      password2 : user.get( 'password' )
    } );

    var warningMsgPanel = me.down( '#warningMessagePanel' );

    me.down( '#externalId' ).setVisible( false );

    var isSystemUser = user.get( 'system' );
    if ( isSystemUser )
    {
      warningMsgPanel.setMessage( QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.systemUserWarning' ) );
      warningMsgPanel.setVisible( true );

      me.down( '#firstName' ).setReadOnly( true );
      me.down( '#surname' ).setReadOnly( true );
      me.down( '#username' ).setReadOnly( true );

      // TODO: implement
      // me.down( '#permissions' ).setReadOnly( true );//
      // me.down( '#groups' ).setReadOnly( true );
    }

    var isExternalUser = user.get( 'externalId' ) !== null;
    if ( isExternalUser )
    {
      warningMsgPanel.setMessage( QDesk.Msg.get( 'QDesk.view.work.security.users.EditUserForm.externalUserWarning' ) );
      warningMsgPanel.setVisible( true );

      me.down( '#externalId' ).setVisible( true );

      me.down( '#firstName' ).setReadOnly( true );
      me.down( '#surname' ).setReadOnly( true );
      me.down( '#email' ).setReadOnly( true );

      me.down( '#username' ).setReadOnly( true );

      me.down( '#password' ).setReadOnly( true );
      me.down( '#password' ).setValue( me.EXTERNAL_USER_PASSWD_PLACEHOLDER );
      me.down( '#password2' ).setReadOnly( true );
      me.down( '#password2' ).setValue( me.EXTERNAL_USER_PASSWD_PLACEHOLDER );

      me.down( '#avatarType' ).setReadOnly( true );

      me.down( '#workPhoneNumber' ).setReadOnly( true );
      me.down( '#mobilePhoneNumber' ).setReadOnly( true );
    }

    /*
     * Avatar.
     */
    var avatarType = user.get( 'avatarType' );
    var avatarUrl = user.get( 'avatarUrl' );
    switch ( avatarType )
    {
      case QDesk.model.security.UserAvatarType.NONE:
        break;

        // case QDesk.model.security.UserAvatarType.LOCAL:
        //   break;

      case QDesk.model.security.UserAvatarType.EXTERNAL_CUSTOM_URL:
        me.down( '#avatarExternalCustomUrl' ).setValue( avatarUrl );
        break;

      case QDesk.model.security.UserAvatarType.EXTERNAL_GRAVATAR:
        break;

      default:
        Ext.Error.raise( 'Unexpected user avatar type value: ' + avatarType );
    }
  },


  /**
   * Returns the {@link User} instance with data from this form.
   */
  getUserRecord : function ()
  {
    var me = this;

    var form = me.getForm();
    var fieldValues = form.getFieldValues();

    return QDesk.util.security.UserUtils.createUser( fieldValues );
  },


  /**
   * Returns the array with all stores used by this form.
   * @return the array with all stores used by this form.
   */
  getFormStores : function ()
  {
    var me = this;

    var stores = [];

    stores.push( me.down( '#permissions' ).getStore() );
    stores.push( me.down( '#groups' ).getStore() );

    return stores;
  },


  /**
   * Binds event handlers to all form widgets.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    me.on( 'afterrender', function ()
    {
      if ( me.qdesk.mode == QDesk.view.work.security.users.EditUserFormMode.ADD )
      {
        QDesk.util.FormUtils.focusFirstEditableField( me );
      }
      else
      {
        // QDesk.util.FormUtils.focusDefaultButton( me );
      }
    } );

    var avatarTypeRadioGroup = me.down( '#avatarType' );

    me.down( '#email' ).on( 'change',
        function ( field, newValue, oldValue )
        {
          var avatarType = avatarTypeRadioGroup.getValue().avatarType;
          if ( avatarType == QDesk.model.security.UserAvatarType.EXTERNAL_GRAVATAR )
          {
            QDesk.util.security.UserUtils.updateAvatarPreview( me );
          }
        }
    );

    me.down( '#avatarExternalCustomUrl' ).on( 'change',
        function ( field, newValue, oldValue )
        {
          var avatarType = avatarTypeRadioGroup.getValue().avatarType;
          if ( avatarType == QDesk.model.security.UserAvatarType.EXTERNAL_CUSTOM_URL )
          {
            QDesk.util.security.UserUtils.updateAvatarPreview( me );
          }
        }
    );

    avatarTypeRadioGroup.on( 'change',
        function ( radiogroup, newValue, oldValue )
        {
          var avatarExternalCustomUrlField = me.down( '#avatarExternalCustomUrl' );

          var avatarType = newValue.avatarType;
          if ( avatarType )
          {
            switch ( avatarType )
            {
              case QDesk.model.security.UserAvatarType.NONE:
                avatarExternalCustomUrlField.disable();
                break;

              case QDesk.model.security.UserAvatarType.LOCAL:
                avatarExternalCustomUrlField.disable();
                break;

              case QDesk.model.security.UserAvatarType.EXTERNAL_CUSTOM_URL:
                avatarExternalCustomUrlField.enable();
                break;

              case QDesk.model.security.UserAvatarType.EXTERNAL_GRAVATAR:
                avatarExternalCustomUrlField.disable();
                break;

              default:
                Ext.Error.raise( 'Unexpected user avatar type value: ' + avatarType );
            }
          }
          else
          {
            // no value selected (should never happen)
            avatarExternalCustomUrlField.disable();
          }

          // update the user avatar preview image
          QDesk.util.security.UserUtils.updateAvatarPreview( me );
        }
    );

    me.down( '#viewEffectivePermissionsButton' ).on( 'click',
        function ()
        {
          var user = me.getUserRecord();
          me.fireEvent( 'viewEffectiveUserPermissions', user );
        }
    );

    me.down( '#actionButton' ).on( 'click',
        function ()
        {
          var user = me.getUserRecord();
          me.fireEvent( 'saveUser', user );
        }
    );
  },


  /**
   * Sets the default values of individual form fields.
   */
  _setDefaultFormValues : function ()
  {
    var me = this;

    // TODO: implement
  },


  _getActionButtonText : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.security.users.EditUserFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.work.security.users.EditUserFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  },


  _getRequiredPermissions : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.security.users.EditUserFormMode.ADD:
        return [ QDesk.model.security.Permission.USER_ADD ];

      case QDesk.view.work.security.users.EditUserFormMode.EDIT:
        return [ QDesk.model.security.Permission.USER_UPDATE ];

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  }
} );
