/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.security.users.UsersGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',
    'QDesk.model.security.UserAccountLockReason',
    'QDesk.model.security.UserAccountStatus',

    'QDesk.store.security.users.UserStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.UxUtils',

    'Ext.ux.form.ClearableCombo'
  ],

  alias : 'widget.UsersGrid',

  cls : 'usersGrid',

  viewConfig : {
    stripeRows : true
  },

  scrollable : true,

  border : false,

  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.getXY() );
    }
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    var userStore = Ext.create( 'QDesk.store.security.users.UserStore' );

    Ext.apply( config, {
      store : userStore,
      qdesk : {
        gridId : 'usersGrid'
      },

      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'button',
          itemId : 'addUserButton',
          scale : 'small',
          iconCls : 'addUser',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipAddUserButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.USER_ADD ],
          qdNotAuthorizedIconCls : 'addUserNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'deleteUserButton',
          scale : 'small',
          iconCls : 'deleteUser',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipDeleteUserButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.USER_DELETE ],
          qdNotAuthorizedIconCls : 'deleteUserNAUTH'
        },
        '-',
        // {
        //   xtype : 'button',
        //   itemId : 'viewEffectiveUserPermissionsButton',
        //   scale : 'small',
        //   iconCls : 'viewEffectiveUserPermissions',
        //   tooltip : QDesk.Msg.get(
        //       'QDesk.view.work.security.users.UsersGrid.tooltipViewEffectiveUserPermissionsButton' ),
        //   disabled : true,
        //   qdRequiresAnyPermissions : [
        //     QDesk.model.security.Permission.USER_ADD,
        //     QDesk.model.security.Permission.USER_UPDATE
        //   ],
        //   qdNotAuthorizedIconCls : 'viewEffectiveUserPermissionsNAUTH'
        // },
        {
          xtype : 'button',
          itemId : 'viewUserAuditTrailButton',
          scale : 'small',
          iconCls : 'viewUserAuditTrail',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipViewUserAuditTrailButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.AUDIT_TRAIL_VIEW ],
          qdNotAuthorizedIconCls : 'viewUserAuditTrailNAUTH'
        },

        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          emptyText : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.emptyTextFieldFilterQueryText' ),
          store : userStore,
          paramName : 'filterQuery',
          width : 150,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ],

      columns : [
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.textIdColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipIdColumn' ),
          dataIndex : 'id',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          itemId : 'accountStatusColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.textAccountStatusColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipAccountStatusColumn',
              QDesk.util.EnumUtils.getUserAccountStatusName( QDesk.model.security.UserAccountStatus.ACTIVE ),
              QDesk.util.EnumUtils.getUserAccountStatusName( QDesk.model.security.UserAccountStatus.LOCKED ),
              QDesk.util.EnumUtils.getUserAccountStatusName( QDesk.model.security.UserAccountStatus.DELETED ) ),
          dataIndex : 'accountStatus',
          sortable : false,
          groupable : false,
          width : 100,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metaData, record )
          {
            var text = QDesk.util.EnumUtils.getUserAccountStatusName( value );

            var iconClass, tooltipText;

            if ( value === QDesk.model.security.UserAccountStatus.ACTIVE )
            {
              iconClass = 'userAccountStatusActive';
              tooltipText = QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipAccountStatusActive' );
            }
            else if ( value === QDesk.model.security.UserAccountStatus.LOCKED )
            {
              var accountLockReason = record.get( 'accountLockReason' );
              var accountLockedAt = record.get( 'accountLockedAt' );
              var accountLockedTill = record.get( 'accountLockedTill' );

              switch ( accountLockReason )
              {
                case QDesk.model.security.UserAccountLockReason.MANUAL:
                  tooltipText = QDesk.Msg.get(
                      'QDesk.view.work.security.users.UsersGrid.tooltipAccountStatusLockedManual',
                      accountLockedAt.targetFmtValue );
                  break;

                case QDesk.model.security.UserAccountLockReason.AUTO_TOO_MANY_LOGIN_ATTEMPTS:
                  tooltipText = QDesk.Msg.get(
                      'QDesk.view.work.security.users.UsersGrid.tooltipAccountStatusLockedAutoTooManyLoginAttempts',
                      accountLockedAt.targetFmtValue,
                      accountLockedTill.targetFmtValue );
                  break;

                default:
                  tooltipText = QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipAccountStatusLocked',
                      accountLockedAt.targetFmtValue,
                      accountLockedTill.targetFmtValue,
                      accountLockReason );
                  break;
              }

              iconClass = 'userAccountStatusLocked';
            }
            else if ( value === QDesk.model.security.UserAccountStatus.DELETED )
            {
              iconClass = 'userAccountStatusDeleted';
              tooltipText = QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipAccountStatusDeleted' );
            }
            else
            {
              Ext.Error.raise( 'Unexpected user account status value: ' + value );
            }

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + Ext.String.htmlEncode( tooltipText ) + '"/>' +
                '<span style="padding-left: 5px;">' + text + '</span>';
          }
        },
        {
          itemId : 'surnameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.textSurnameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipSurnameColumn' ),
          dataIndex : 'surname',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'firstNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.textFirstNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipFirstNameColumn' ),
          dataIndex : 'firstName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },

        {
          itemId : 'usernameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.textUsernameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipUsernameColumn' ),
          dataIndex : 'username',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'systemColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.textSystemColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipSystemColumn' ),
          dataIndex : 'system',
          sortable : false,
          groupable : false,
          width : 65,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metaData, record )
          {
            var iconClass, tooltipText;

            if ( value )
            {
              iconClass = 'tick';
              tooltipText = QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.tooltipSystem' );

              // img src must be present in order to get rid of the image border that is shown in Chrome
              return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                  ' nonClickable" data-qtip="' + Ext.String.htmlEncode( tooltipText ) + '"/>';
            }
            else
            {
              return '';
            }
          }
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  /**
   * Refreshes the Users grid.
   */
  refresh : function ()
  {
    var me = this;
    var store = me.getStore();

    var filterQueryText = me.down( '#filterQueryText' ).getValue();

    var extraStoreParams = {};
    if ( !Ext.isEmpty( filterQueryText ) )
    {
      Ext.apply( extraStoreParams, {
        filterQuery : filterQueryText
      } );
    }

    store.getProxy().extraParams = extraStoreParams;
    store.loadPage( 1 );
  },


  /**
   * Invoked to create a context menu for the specified user stub.
   * @param userStub a user stub.
   * @private
   */
  _createRowCtxMenu : function ( userStub )
  {
    var me = this;

    var isMe = userStub.get( 'id' ) === QDesk.CurrentUser.getUserProfile().userId;
    var accountStatus = userStub.get( 'accountStatus' );
    var isSystemUser = userStub.get( 'system' );

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // lock user
        {
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.userCtxMenuLock' ),
          iconCls : 'lockUser',
          // system users or users with non-ACTIVE account status cannot be locked (manually from the GUI)
          disabled : isMe || isSystemUser || accountStatus !== QDesk.model.security.UserAccountStatus.ACTIVE,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.USER_LOCK_UPDATE ],
          qdNotAuthorizedIconCls : 'lockUserNAUTH',
          handler : function ()
          {
            me.fireEvent( 'lockUser', userStub );
          }
        },
        // unlock user
        {
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.userCtxMenuUnlock' ),
          iconCls : 'unlockUser',
          // system users or users with non-LOCKED account status cannot be unlocked
          disabled : isSystemUser || accountStatus !== QDesk.model.security.UserAccountStatus.LOCKED,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.USER_LOCK_UPDATE ],
          qdNotAuthorizedIconCls : 'unlockUserNAUTH',
          handler : function ()
          {
            me.fireEvent( 'unlockUser', userStub );
          }
        },
        '-',
        // view effective user permissions
        {
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.userCtxMenuViewEffectivePermissions' ),
          iconCls : 'viewEffectiveUserPermissions',
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.USER_ADD,
            QDesk.model.security.Permission.USER_UPDATE
          ],
          qdNotAuthorizedIconCls : 'viewEffectiveUserPermissionsNAUTH',
          handler : function ()
          {
            me.fireEvent( 'viewEffectiveUserPermissions', userStub );
          }
        },
        // view audit trail
        {
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.userCtxMenuViewAuditTrail' ),
          iconCls : 'viewUserAuditTrail',
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.AUDIT_TRAIL_VIEW
          ],
          qdNotAuthorizedIconCls : 'viewAuditTrailNAUTH',
          handler : function ()
          {
            me.fireEvent( 'viewUserAuditTrail', userStub );
          }
        },
        '-',
        // delete user
        {
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.userCtxMenuDelete' ),
          iconCls : 'deleteUser',
          // system users cannot be deleted
          disabled : isSystemUser,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.USER_DELETE ],
          qdNotAuthorizedIconCls : 'deleteUserNAUTH',
          handler : function ()
          {
            me.fireEvent( 'deleteUser', userStub );
          }
        }
      ]
    } );
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    var store = me.getStore();

    // refresh button
    var refreshButton = me.down( '#refreshButton' );
    refreshButton.on( 'click',
        function ()
        {
          me.refresh();
        } );

    // add user button
    var addUserButton = me.down( '#addUserButton' );
    addUserButton.on( 'click',
        function ()
        {
          // unselect a selected records in the grid if there is such a record
          me.getSelectionModel().deselectAll();

          me.fireEvent( 'addUser' );
        } );

    // delete user button
    var deleteUserButton = me.down( '#deleteUserButton' );
    deleteUserButton.on( 'click',
        function ()
        {
          var userStub = me.getSelectionModel().getSelection()[ 0 ];
          me.fireEvent( 'deleteUser', userStub );
        } );

    // view user audit trail button
    var viewUserAuditTrailButton = me.down( '#viewUserAuditTrailButton' );
    viewUserAuditTrailButton.on( 'click',
        function ()
        {
          var userStub = me.getSelectionModel().getSelection()[ 0 ];
          me.fireEvent( 'viewUserAuditTrail', userStub );
        } );

    var filterQueryText = me.down( '#filterQueryText' );

    store.on( 'beforeload', function ( store, operation )
    {
      refreshButton.disable();
      addUserButton.disable();
      deleteUserButton.disable();
      viewUserAuditTrailButton.disable();
      filterQueryText.disable();

      /*
       * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
       * Therefore we need to force-fire this event by clearing the selection.
       */
      me.getSelectionModel().deselectAll();
    } );

    store.on( 'load', function ( store, records, successful, operation )
    {
      refreshButton.enable();

      var view = me.getView();

      if ( successful )
      {
        addUserButton.enable();
        filterQueryText.enable();

        if ( Ext.isEmpty( records ) )
        {
          view.emptyText =
              QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.emptyTextNoMatchingData' );

          view.refresh();
        }
      }
      else
      {
        store.removeAll();

        view.emptyText =
            QDesk.Msg.get( 'QDesk.view.work.security.users.UsersGrid.emptyTextErrorLoadingData' );

        view.refresh();

        QDesk.util.UxUtils.showExtJsOperationError( operation );
      }
    } );

    me.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length === 0 )
          {
            deleteUserButton.disable();
            viewUserAuditTrailButton.disable();

            me.fireEvent( 'userUnselected' );
          }
          else
          {
            var userStub = selectedRecords[ 0 ];

            // enable delete button if the user is NOT a system user
            if ( !userStub.get( 'system' ) )
            {
              deleteUserButton.enable();
            }

            viewUserAuditTrailButton.enable();

            me.fireEvent( 'userSelected', userStub );
          }
        } );

    // when the grid has been rendered, load the data
    me.on( 'afterrender', function ()
    {
      me.refresh();
    } )
  }
} );
