/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.security.users.UsersTab', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.view.work.security.users.UsersGrid',
    'QDesk.view.widget.message.InfoMessagePanel',
    'QDesk.view.work.security.users.EditUserForm',

    'QDesk.controller.command.audit.ViewUserAuditTrail',
    'QDesk.controller.command.security.users.AddUser',
    'QDesk.controller.command.security.users.DeleteUser',
    'QDesk.controller.command.security.users.EditUser',
    'QDesk.controller.command.security.users.LockUser',
    'QDesk.controller.command.security.users.SaveUser',
    'QDesk.controller.command.security.users.UnlockUser',
    'QDesk.controller.command.security.users.ViewEffectiveUserPermissions'
  ],

  alias : 'widget.UsersTab',

  cls : 'usersTab',
  bodyCls : 'usersTabBody',

  iconCls : 'usersTabIcon',

  border : false,

  closable : true,
  scrollable : true,

  layout : {
    type : 'border'
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersTab.title' ),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersTab.tooltip' ),
      items : [
        {
          xtype : 'panel',
          border : false,
          layout : {
            type : 'vbox',
            align : 'stretch'
          },
          collapsible : true,
          collapseMode : 'mini',
          split : true,
          width : 350,
          minWidth : 350,
          region : 'west',
          items : [
            {
              xtype : 'UsersGrid',
              itemId : 'usersGrid',
              flex : 1
            }
          ]
        },
        {
          xtype : 'panel',
          itemId : 'editPanel',
          border : false,
          flex : 1,
          minWidth : 450,
          region : 'center',
          layout : 'card',
          items : [
            {
              xtype : 'panel',
              itemId : 'infoPanel',
              border : false,
              bodyPadding : 5,
              items : [
                {
                  xtype : 'InfoMessagePanel',
                  qdesk : {
                    msg : QDesk.Msg.get( 'QDesk.view.work.security.users.UsersTab.infoText' )
                  }
                }
              ]
            }
            // {
            //   xtype : 'EditUserForm',  // this form panel is created and added dynamically
            // }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    var usersGrid = me.down( '#usersGrid' );

    usersGrid.on( 'addUser', function ()
    {
      me._cleanupEditPanel();

      Ext.create( 'QDesk.controller.command.security.users.AddUser', {} ).execute();
    } );

    usersGrid.on( 'viewEffectiveUserPermissions', function ( userStub )
    {
      Ext.create( 'QDesk.controller.command.security.users.ViewEffectiveUserPermissions', {
        userId : userStub.get( 'id' )
      } ).execute();
    } );

    usersGrid.on( 'viewUserAuditTrail', function ( userStub )
    {
      Ext.create( 'QDesk.controller.command.audit.ViewUserAuditTrail', {
        userId : userStub.get( 'id' )
      } ).execute();
    } );

    usersGrid.on( 'deleteUser', function ( userStub )
    {
      Ext.create( 'QDesk.controller.command.security.users.DeleteUser', {
        userStub : userStub,
        success : function ()
        {
          usersGrid.refresh();
        }
      } ).execute();
    } );

    usersGrid.on( 'lockUser', function ( userStub )
    {
      Ext.create( 'QDesk.controller.command.security.users.LockUser', {
        userStub : userStub,
        success : function ()
        {
          usersGrid.refresh();
        }
      } ).execute();
    } );

    usersGrid.on( 'unlockUser', function ( userStub )
    {
      Ext.create( 'QDesk.controller.command.security.users.UnlockUser', {
        userStub : userStub,
        success : function ()
        {
          usersGrid.refresh();
        }
      } ).execute();
    } );

    usersGrid.on( 'userSelected', function ( userStub )
    {
      me._cleanupEditPanel();

      Ext.create( 'QDesk.controller.command.security.users.EditUser', {
        userStub : userStub
      } ).execute();
    } );

    usersGrid.on( 'userUnselected', function ()
    {
      me._cleanupEditPanel();
    } );
  },


  /**
   * Adds the specified EditUserForm panel to the edit panel and makes the added panel active.
   * @param formPanel a EditUserForm panel to add.
   */
  showEditUserForm : function ( formPanel )
  {
    var me = this;

    var usersGrid = me.down( '#usersGrid' );

    formPanel.on( 'viewEffectiveUserPermissions',
        function ( user )
        {
          Ext.create( 'QDesk.controller.command.security.users.ViewEffectiveUserPermissions', {
            userId : user.get( 'id' )
          } ).execute();
        } );

    formPanel.on( 'saveUser',
        function ( user )
        {
          Ext.create( 'QDesk.controller.command.security.users.SaveUser', {
            formPanel : formPanel,
            user : user,
            success : function ()
            {
              me._cleanupEditPanel();

              usersGrid.refresh();
            }
          } ).execute();
        } );

    var editPanel = me.down( '#editPanel' );
    editPanel.add( formPanel );
    editPanel.getLayout().setActiveItem( formPanel );
  },


  /**
   * Removes all panels from the edit panel but the info panel.
   * @private
   */
  _cleanupEditPanel : function ()
  {
    var me = this;

    var editPanel = me.down( '#editPanel' );

    // remove all panels (edit user form panel) from the edit panel (right panel), but the infoPanel
    Ext.each( editPanel.getLayout().getLayoutItems(), function ( panel )
    {
      if ( panel.getItemId() == 'infoPanel' )
      {
        // edit panel uses card layout, so we activate the info panel
        editPanel.getLayout().setActiveItem( panel );
      }
      else
      {
        editPanel.remove( panel );
      }
    } );
  }
} );