/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.system.ws.ServiceEndpointsGrid', {
  extend : 'Ext.grid.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',
    'QDesk.model.system.ws.ServiceEndpointStatus',

    'QDesk.store.system.ws.ServiceEndpointStore',

    'QDesk.util.UxUtils',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.system.ws.ServiceEndpointUtils',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.ServiceEndpointsGrid',

  cls : 'serviceEndpointsGrid',

  border : false,
  scrollable : true,

  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.getXY() );
    }
  },


  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '{[QDesk.util.system.ws.ServiceEndpointUtils.createRowDetailsHtml( values, this.owner.grid )]}'
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    var store = Ext.create( 'QDesk.store.system.ws.ServiceEndpointStore' );

    var tbar = [];

    tbar.push(
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'button',
          itemId : 'startButton',
          scale : 'small',
          iconCls : 'serviceEndpointStart',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.tooltipStartEndpointButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SERVICE_ENDPOINT_START ],
          qdNotAuthorizedIconCls : 'serviceEndpointStartNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'stopButton',
          scale : 'small',
          iconCls : 'serviceEndpointStop',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.tooltipStopEndpointButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SERVICE_ENDPOINT_STOP ],
          qdNotAuthorizedIconCls : 'serviceEndpointStopNAUTH'
        },
        '-',
        {
          xtype : 'button',
          itemId : 'resetStatisticsButton',
          scale : 'small',
          iconCls : 'serviceEndpointResetStatistics',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.system.ws.ServiceEndpointsGrid.tooltipResetEndpointStatisticsButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SERVICE_ENDPOINT_RESET_STATISTICS ],
          qdNotAuthorizedIconCls : 'serviceEndpointResetStatisticsNAUTH'
        }
    );

    Ext.apply( config, {
      tbar : tbar,
      columns : [
        // {
        //   itemId : 'idColumn',
        //   text : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.textIdColumn' ),
        //   tooltip : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.tooltipIdColumn' ),
        //   dataIndex : 'id',
        //   sortable : false,
        //   groupable : false,
        //   flex : 0.7
        // },

        {
          itemId : 'versionColumn',
          align : 'right',
          text : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.textVersionColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.tooltipVersionColumn' ),
          dataIndex : 'version',
          sortable : false,
          groupable : false,
          flex : 0.4,
          renderer : function ( value )
          {
            return QDesk.util.FormatUtils.formatVersion( value );
          }
        },

        {
          itemId : 'statusColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.textStatusColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.tooltipStatusColumn' ),
          dataIndex : 'status',
          sortable : false,
          groupable : false,
          flex : 0.5,
          renderer : function ( value )
          {
            var statusClass = 'status' + value;
            return '<span class="' + statusClass + '">' + QDesk.util.EnumUtils.getWebServiceEndpointStatus( value ) +
                '</span>';
          }
        },

        {
          itemId : 'deprecatedColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.textDeprecatedColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.tooltipDeprecatedColumn' ),
          dataIndex : 'deprecated',
          sortable : false,
          groupable : false,
          flex : 0.6,
          renderer : function ( value )
          {
            return QDesk.util.FormatUtils.formatBoolean( value );
          }
        },

        {
          itemId : 'requiredPermissionsColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.textRequiredPermissionsColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.tooltipRequiredPermissionsColumn' ),
          sortable : false,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            //var requiresAllPermission = record.get( 'requiresAllPermission' );
            var requiresAnyPermission = record.get( 'requiresAnyPermission' );
            return QDesk.util.FormatUtils.formatPermissions( requiresAnyPermission );
          }
        },

        {
          itemId : 'serverImplClassColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.textServerImplClassColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.tooltipServerImplClassColumn' ),
          dataIndex : 'serverImplClass',
          sortable : false,
          groupable : false,
          flex : 2.5,
          renderer : function ( value, metaData, record )
          {
            return '<span class="pre">' + Ext.String.htmlEncode( value ) + '</span>';
          }
        },

        {
          itemId : 'clientImplClassColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.textClientImplClassColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.tooltipClientImplClassColumn' ),
          dataIndex : 'clientImplClass',
          sortable : false,
          groupable : false,
          flex : 2.5,
          renderer : function ( value, metaData, record )
          {
            return '<span class="pre">' + Ext.String.htmlEncode( value ) + '</span>';
          }
        }
      ],

      store : store,
      emptyText : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.emptyText' )
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me._bindEventHandlers();

    // rows have different CSS style that reflects the deprecated flag
    me.getView().getRowClass = function ( record )
    {
      var deprecated = record.get( 'deprecated' );
      return deprecated ? 'deprecated' : 'non-deprecated';
    }
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    // refresh button
    me.down( '#refreshButton' ).on( 'click',
        function ()
        {
          me.fireEvent( 'refresh' );
        } );

    // start button
    me.down( '#startButton' ).on( 'click',
        function ()
        {
          var endpoint = me.getSelectionModel().getSelection()[ 0 ];
          me.fireEvent( 'startEndpoint', endpoint );
        } );

    // stop endpoint button
    me.down( '#stopButton' ).on( 'click',
        function ()
        {
          var endpoint = me.getSelectionModel().getSelection()[ 0 ];
          me.fireEvent( 'stopEndpoint', endpoint );
        } );

    // reset statistics button
    me.down( '#resetStatisticsButton' ).on( 'click',
        function ()
        {
          var endpoint = me.getSelectionModel().getSelection()[ 0 ];
          me.fireEvent( 'resetEndpointStatistics', endpoint );
        } );

    me.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length === 0 )
          {
            me.down( '#startButton' ).disable();
            me.down( '#stopButton' ).disable();
            me.down( '#resetStatisticsButton' ).disable();

            me.fireEvent( 'entryUnselected' );
          }
          else
          {
            var serviceEndpoint = selectedRecords[ 0 ];
            var serviceEndpointStatus = serviceEndpoint.get( 'status' );

            if ( serviceEndpointStatus == QDesk.model.system.ws.ServiceEndpointStatus.STARTED )
            {
              me.down( '#startButton' ).disable();
              me.down( '#stopButton' ).enable();
            }
            else if ( serviceEndpointStatus == QDesk.model.system.ws.ServiceEndpointStatus.STOPPED )
            {
              me.down( '#startButton' ).enable();
              me.down( '#stopButton' ).disable();
            }

            me.down( '#resetStatisticsButton' ).enable();

            me.fireEvent( 'entrySelected', selectedRecords[ 0 ] );
          }
        } );

    me.getStore().on( 'beforeload',
        function ( store, operation )
        {
          me.down( '#refreshButton' ).disable();
          me.down( '#startButton' ).disable();
          me.down( '#stopButton' ).disable();

          /*
           * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
           * Therefore we need to force-fire this event by clearing the selection.
           */
          me.getSelectionModel().deselectAll();
        } );


    me.getStore().on( 'load',
        function ( store, records, successful, operation )
        {
          me.down( '#refreshButton' ).enable();

          var view = me.getView();

          if ( successful )
          {
            me.fireEvent( 'loadSuccess', store, records );
          }
          else
          {
            store.removeAll();

            view.emptyText =
                QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.emptyTextErrorLoadingData' );
            view.refresh();

            me.fireEvent( 'loadFailed', operation );

            QDesk.util.UxUtils.showExtJsOperationError( operation );
          }
        } );
  },


  /**
   * Invoked to create a context menu for the specified endpoint in the ServiceEndpointsGrid row.
   * @param endpoint a service endpoint.
   * @private
   */
  _createRowCtxMenu : function ( endpoint )
  {
    var me = this;

    var endpointStatus = endpoint.get( 'status' );

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // start endpoint
        {
          text : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.endpointCtxMenuStartEndpoint' ),
          iconCls : 'serviceEndpointStart',
          disabled : endpointStatus == QDesk.model.system.ws.ServiceEndpointStatus.STARTED,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SERVICE_ENDPOINT_START ],
          qdNotAuthorizedIconCls : 'serviceEndpointStartNAUTH',
          handler : function ()
          {
            me.fireEvent( 'startEndpoint', endpoint );
          }
        },
        // stop endpoint
        {
          text : QDesk.Msg.get( 'QDesk.view.work.system.ws.ServiceEndpointsGrid.endpointCtxMenuStopEndpoint' ),
          iconCls : 'serviceEndpointStop',
          disabled : endpointStatus == QDesk.model.system.ws.ServiceEndpointStatus.STOPPED,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SERVICE_ENDPOINT_STOP ],
          qdNotAuthorizedIconCls : 'serviceEndpointStopNAUTH',
          handler : function ()
          {
            me.fireEvent( 'stopEndpoint', endpoint );
          }
        },
        '-',
        // reset endpoint statistics
        {
          text : QDesk.Msg.get(
              'QDesk.view.work.system.ws.ServiceEndpointsGrid.endpointCtxMenuResetEndpointStatistics' ),
          iconCls : 'serviceEndpointResetStatistics',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SERVICE_ENDPOINT_RESET_STATISTICS ],
          qdNotAuthorizedIconCls : 'serviceEndpointResetStatisticsNAUTH',
          handler : function ()
          {
            me.fireEvent( 'resetEndpointStatistics', endpoint );
          }
        }
      ]
    } );
  }
} );
