/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Panel that contains a web service title and a grid with all web service endpoints.
 */
Ext.define( 'QDesk.view.work.system.ws.ServiceEndpointsTab', {
  extend : 'QDesk.view.widget.panel.XPanel',

  requires : [
    'QDesk.model.system.ws.ServiceEndpoint',

    'QDesk.view.widget.message.InfoMessagePanel',
    'QDesk.view.work.system.ws.ServiceEndpointsGrid'
  ],

  alias : 'widget.ServiceEndpointsTab',

  cls : 'serviceEndpointsTab',
  bodyCls : 'serviceEndpointsTabBody',

  border : false,

  layout : {
    type : 'vbox',
    align : 'stretch'
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.service )
      Ext.Error.raise( 'Required config parameter missing: qdesk.service' );

    var service = config.qdesk.service;

    Ext.apply( config, {
      title : service.get( 'name' ),
      items : [
        {
          xtype : 'InfoMessagePanel',
          margin : 5,
          qdesk : {
            msg : service.get( 'description' )
          }
        },
        {
          xtype : 'ServiceEndpointsGrid',
          flex : 1
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  getService : function ()
  {
    var me = this;
    return me.getInitialConfig().qdesk.service;
  }
} );
