/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.user.EditProfileForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.UserAvatarType',

    'QDesk.util.security.UserUtils',

    'QDesk.view.widget.message.WarningMessagePanel',
    'QDesk.view.widget.WPhoneNumber'
  ],

  alias : 'widget.EditProfileForm',

  bodyPadding : '5 10 10 10',

  border : false,

  scrollable : true,

  fieldDefaults : {
    labelWidth : 80
  },

  EXTERNAL_USER_PASSWD_PLACEHOLDER : '<password-placeholder>',


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'WarningMessagePanel',
          itemId : 'warningMessagePanel',
          hidden : true,
          style : {
            marginBottom : '5px'
          }
        },

        {
          xtype : 'hiddenfield',
          itemId : 'id',
          name : 'id'
        },

        {
          xtype : 'hiddenfield',
          itemId : 'system',
          name : 'system'
        },

        {
          xtype : 'textfield',
          itemId : 'externalId',
          name : 'externalId',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.user.EditProfileForm.fieldLabelExternalId' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.user.EditProfileForm.fieldTooltipExternalId' ),
          // emptyText : QDesk.Msg.get(
          //     'QDesk.view.work.user.EditProfileForm.fieldEmptyTxtExternalId' ),
          allowBlank : true,
          hidden : true,
          readOnly : true,
          maxWidth : 350
        },

        {
          xtype : 'textfield',
          itemId : 'firstName',
          name : 'firstName',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.user.EditProfileForm.fieldLabelFirstName' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.user.EditProfileForm.fieldTooltipFirstName' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.user.EditProfileForm.fieldEmptyTxtFirstName' ),
          allowBlank : false,
          maxWidth : 350
        },
        {
          xtype : 'textfield',
          itemId : 'surname',
          name : 'surname',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.user.EditProfileForm.fieldLabelSurname' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.user.EditProfileForm.fieldTooltipSurname' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.user.EditProfileForm.fieldEmptyTxtSurname' ),
          allowBlank : false,
          maxWidth : 350
        },

        {
          xtype : 'textfield',
          itemId : 'email',
          name : 'email',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.user.EditProfileForm.fieldLabelEmail' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.user.EditProfileForm.fieldTooltipEmail' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.user.EditProfileForm.fieldEmptyTxtEmail' ),
          allowBlank : false,
          vtype : 'email',
          maxWidth : 350
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.fieldsetLabelLoginCredentials' ),
          padding : '0 10 0 10',
          defaults : {
            labelWidth : 120
            //anchor : '100%'
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'username',
              name : 'username',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.user.EditProfileForm.fieldLabelUsername' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.user.EditProfileForm.fieldTooltipUsername' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.user.EditProfileForm.fieldEmptyTxtUsername' ),
              allowBlank : false,
              maxWidth : 340,
              minWidth : 340
            },

            {
              xtype : 'textfield',
              itemId : 'password',
              name : 'password',
              inputType : 'password',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.user.EditProfileForm.fieldLabelPassword' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.user.EditProfileForm.fieldTooltipPassword' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.user.EditProfileForm.fieldEmptyTxtPassword' ),
              allowBlank : false,
              minLength : 6, /* Passwords must be at least 6 characters long */
              maxWidth : 340,
              minWidth : 340
            },
            {
              xtype : 'textfield',
              itemId : 'password2',
              name : 'password2',
              inputType : 'password',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.user.EditProfileForm.fieldLabelPassword2' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.user.EditProfileForm.fieldTooltipPassword2' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.user.EditProfileForm.fieldEmptyTxtPassword2' ),
              allowBlank : false,
              maxWidth : 340,
              minWidth : 340,
              validator : function ( value )
              {
                if ( Ext.isEmpty( value ) )
                {
                  return true;
                }
                else
                {
                  var passwordField = me.down( '#password' );
                  return value === passwordField.getValue() ? true :
                      QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.validationPasswordsDoNotMatch' );
                }
              }
            }
          ]
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.fieldsetLabelAvatar' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.fieldsetTooltipAvatar' ),
          padding : '0 10 0 10',
          layout : {
            type : 'hbox',
            align : 'center',
            pack : 'left'
          },
          items : [
            {
              xtype : 'image',
              itemId : 'avatarPreview',
              cls : 'avatarPreview',
              margin : '0 10 0 0',
              height : 35,
              width : 35,
              src : 'images/icon/avatar-default-35x35.png'  // default avatar image
            },

            {
              xtype : 'fieldcontainer',
              items : [
                {
                  xtype : 'radiogroup',
                  itemId : 'avatarType',
                  columns : 1,
                  vertical : true,
                  allowBlank : false,
                  layout : {
                    type : 'vbox',
                    align : 'stretch'
                  },
                  qdesk : {
                    required : true
                  },
                  items : [
                    {
                      xtype : 'radio',
                      boxLabel : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.boxLabelAvatarNone' ),
                      boxTooltip : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.boxTooltipAvatarNone' ),
                      name : 'avatarType',
                      inputValue : QDesk.model.security.UserAvatarType.NONE
                    },

                    // {
                    //   xtype : 'radio',
                    //   boxLabel : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.boxLabelAvatarLocal' ),
                    //   boxTooltip : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.boxTooltipAvatarLocal' ),
                    //   name : 'avatarType',
                    //   inputValue : QDesk.model.security.UserAvatarType.LOCAL
                    // },

                    {
                      xtype : 'radio',
                      boxLabel : QDesk.Msg.get(
                          'QDesk.view.work.user.EditProfileForm.boxLabelAvatarExternalCustomUrl' ),
                      boxTooltip : QDesk.Msg.get(
                          'QDesk.view.work.user.EditProfileForm.boxTooltipAvatarExternalCustomUrl' ),
                      name : 'avatarType',
                      inputValue : QDesk.model.security.UserAvatarType.EXTERNAL_CUSTOM_URL
                    },
                    {
                      xtype : 'textfield',
                      itemId : 'avatarExternalCustomUrl',
                      name : 'avatarExternalCustomUrl',
                      fieldLabel : QDesk.Msg.get(
                          'QDesk.view.work.user.EditProfileForm.fieldLabelAvatarExternalCustomUrl' ),
                      fieldTooltip : QDesk.Msg.get(
                          'QDesk.view.work.user.EditProfileForm.fieldTooltipAvatarExternalCustomUrl' ),
                      emptyText : QDesk.Msg.get(
                          'QDesk.view.work.user.EditProfileForm.fieldEmptyTxtAvatarExternalCustomUrl' ),
                      labelWidth : 50,
                      allowBlank : false,
                      vtype : 'url',
                      maxWidth : 420,
                      minWidth : 360
                    },

                    {
                      xtype : 'radio',
                      boxLabel : QDesk.Msg.get(
                          'QDesk.view.work.user.EditProfileForm.boxLabelAvatarExternalGravatar' ),
                      boxTooltip : QDesk.Msg.get(
                          'QDesk.view.work.user.EditProfileForm.boxTooltipAvatarExternalGravatar' ),
                      name : 'avatarType',
                      inputValue : QDesk.model.security.UserAvatarType.EXTERNAL_GRAVATAR
                    }
                  ]
                }
              ]
            }
          ]
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.fieldsetLabelWorkPhoneNumber' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.fieldsetTooltipWorkPhoneNumber' ),
          padding : '0 10 0 10',
          defaults : {
            // labelWidth : 115,
            // anchor : '100%'
          },
          items : [
            {
              xtype : 'WPhoneNumber',
              itemId : 'workPhoneNumber',
              name : 'workPhoneNumber',
              allowBlank : true
            }
          ]
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.fieldsetLabelMobilePhoneNumber' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.fieldsetTooltipMobilePhoneNumber' ),
          padding : '0 10 0 10',
          defaults : {
            // labelWidth : 115,
            // anchor : '100%'
          },
          items : [
            {
              xtype : 'WPhoneNumber',
              itemId : 'mobilePhoneNumber',
              name : 'mobilePhoneNumber',
              allowBlank : true
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'viewEffectivePermissionsButton',
          text : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.buttonLabelViewEffectivePermissions' ),
          iconCls : 'viewEffectiveUserPermissions',
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'viewEffectiveUserPermissionsNAUTH'
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();

    me._setDefaultFormValues();
  },


  /**
   * Loads the specified user record ({@link User} instance) into this form.
   * @param user the user record.
   */
  loadRecord : function ( user )
  {
    var me = this;
    me.callParent( arguments );

    me.getForm().setValues( {
      password2 : user.get( 'password' )
    } );

    var warningMsgPanel = me.down( '#warningMessagePanel' );

    me.down( '#externalId' ).setVisible( false );

    // users cannot modify their own username in My Profile editor
    me.down( '#username' ).setReadOnly( true );

    var isSystemUser = user.get( 'system' );
    if ( isSystemUser )
    {
      warningMsgPanel.setMessage( QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.systemUserWarning' ) );
      warningMsgPanel.setVisible( true );

      me.down( '#firstName' ).setReadOnly( true );
      me.down( '#surname' ).setReadOnly( true );
    }

    var isExternalUser = user.get( 'externalId' ) !== null;
    if ( isExternalUser )
    {
      warningMsgPanel.setMessage( QDesk.Msg.get( 'QDesk.view.work.user.EditProfileForm.externalUserWarning' ) );
      warningMsgPanel.setVisible( true );

      me.down( '#externalId' ).setVisible( true );

      me.down( '#firstName' ).setReadOnly( true );
      me.down( '#surname' ).setReadOnly( true );
      me.down( '#email' ).setReadOnly( true );

      me.down( '#password' ).setReadOnly( true );
      me.down( '#password' ).setValue( me.EXTERNAL_USER_PASSWD_PLACEHOLDER );
      me.down( '#password2' ).setReadOnly( true );
      me.down( '#password2' ).setValue( me.EXTERNAL_USER_PASSWD_PLACEHOLDER );

      me.down( '#avatarType' ).setReadOnly( true );

      me.down( '#workPhoneNumber' ).setReadOnly( true );
      me.down( '#mobilePhoneNumber' ).setReadOnly( true );
    }

    /*
     * Avatar.
     */
    var avatarType = user.get( 'avatarType' );
    var avatarUrl = user.get( 'avatarUrl' );
    switch ( avatarType )
    {
      case QDesk.model.security.UserAvatarType.NONE:
        break;

        // case QDesk.model.security.UserAvatarType.LOCAL:
        //   break;

      case QDesk.model.security.UserAvatarType.EXTERNAL_CUSTOM_URL:
        me.down( '#avatarExternalCustomUrl' ).setValue( avatarUrl );
        break;

      case QDesk.model.security.UserAvatarType.EXTERNAL_GRAVATAR:
        break;

      default:
        Ext.Error.raise( 'Unexpected user avatar type value: ' + avatarType );
    }
  },


  /**
   * Returns the {@link User} instance with data from this form.
   */
  getUserRecord : function ()
  {
    var me = this;

    var form = me.getForm();
    var fieldValues = form.getFieldValues();

    return QDesk.util.security.UserUtils.createUser( fieldValues );
  },


  /**
   * Binds event handlers to all form widgets.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    var avatarTypeRadioGroup = me.down( '#avatarType' );

    me.down( '#email' ).on( 'change',
        function ( field, newValue, oldValue )
        {
          var avatarType = avatarTypeRadioGroup.getValue().avatarType;
          if ( avatarType == QDesk.model.security.UserAvatarType.EXTERNAL_GRAVATAR )
          {
            QDesk.util.security.UserUtils.updateAvatarPreview( me );
          }
        }
    );

    me.down( '#avatarExternalCustomUrl' ).on( 'change',
        function ( field, newValue, oldValue )
        {
          var avatarType = avatarTypeRadioGroup.getValue().avatarType;
          if ( avatarType == QDesk.model.security.UserAvatarType.EXTERNAL_CUSTOM_URL )
          {
            QDesk.util.security.UserUtils.updateAvatarPreview( me );
          }
        }
    );

    avatarTypeRadioGroup.on( 'change',
        function ( radiogroup, newValue, oldValue )
        {
          var avatarExternalCustomUrlField = me.down( '#avatarExternalCustomUrl' );

          var avatarType = newValue.avatarType;
          if ( avatarType )
          {
            switch ( avatarType )
            {
              case QDesk.model.security.UserAvatarType.NONE:
                avatarExternalCustomUrlField.disable();
                break;

              case QDesk.model.security.UserAvatarType.LOCAL:
                avatarExternalCustomUrlField.disable();
                break;

              case QDesk.model.security.UserAvatarType.EXTERNAL_CUSTOM_URL:
                avatarExternalCustomUrlField.enable();
                break;

              case QDesk.model.security.UserAvatarType.EXTERNAL_GRAVATAR:
                avatarExternalCustomUrlField.disable();
                break;

              default:
                Ext.Error.raise( 'Unexpected user avatar type value: ' + avatarType );
            }
          }
          else
          {
            // no value selected (should never happen)
            avatarExternalCustomUrlField.disable();
          }

          // update the user avatar preview image
          QDesk.util.security.UserUtils.updateAvatarPreview( me );
        }
    );

    me.down( '#viewEffectivePermissionsButton' ).on( 'click',
        function ()
        {
          var user = me.getUserRecord();
          me.fireEvent( 'viewEffectiveUserPermissions', user );
        }
    );
  },


  /**
   * Sets the default values of individual form fields.
   */
  _setDefaultFormValues : function ()
  {
    var me = this;

    // TODO: implement
  }
} );
