/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.user.EditProfileWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.view.work.user.EditProfileForm'
  ],

  alias : 'widget.EditProfileWindow',

  cls : 'editProfileWindow',

  modal : true,
  resizable : false,

  minWidth : 650,
  width : 650,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.user.EditProfileWindow.windowTitle' ),
      // iconCls : iconCls,
      items : [
        {
          xtype : 'EditProfileForm',
          itemId : 'form'
        }
      ],

      dockedItems : [
        {
          xtype : 'toolbar',
          dock : 'bottom',
          ui : 'footer',
          defaults : {
            minWidth : 75
          },
          items : [
            {
              xtype : 'component',
              flex : 1
            },
            {
              xtype : 'button',
              itemId : 'cancelButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
            },
            {
              xtype : 'button',
              itemId : 'actionButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelUpdate' ),
              disabled : true,
              qdRequiresAnyPermissions : []
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    var actionButton = me.down( '#actionButton' );

    var formPanel = me.down( '#form' );

    formPanel.on( 'validitychange',
        function ( panel, valid )
        {
          var externalUserId = formPanel.down( '#externalId' ).getValue();

          // disable actionButton if the user is an external user, or the form is invalid
          actionButton.setDisabled( !Ext.isEmpty( externalUserId ) || !valid );
        } );

    // cancel button closes the window
    me.down( '#cancelButton' ).on( 'click',
        function ()
        {
          me.close();
        } );

    actionButton.on( 'click',
        function ()
        {
          var user = formPanel.getUserRecord();
          me.fireEvent( 'saveProfile', user );
        }
    );
  }
} );
